Package: wintime
Type: Package
Title: Win Time Methods for Time-to-Event Data in Clinical Trials
Version: 0.4.4
Authors@R: c(person("James", "Troendle", email = "james.troendle@nih.gov", role = c("aut", "cre")),
           person("Samuel", "Lawrence", email = "roaders79@gmail.com", role = "aut"))
Description: Performs an analysis of time-to-event clinical trial data using various "win time" methods, 
  including 'ewt', 'ewtr', 'rmt', 'ewtp', 'rewtp', 'ewtpr', 'rewtpr', 'max', 'wtr', 'rwtr', 'pwt', and 'rpwt'. These methods are used to calculate and compare 
  treatment effects on ordered composite endpoints. The package handles event times, event indicators, and treatment 
  arm indicators and supports calculations on observed and resampled data. Detailed explanations of each method and 
  usage examples are provided in "Use of win time for ordered composite endpoints in clinical trials," by Troendle et al. 
  (2024)<https://pubmed.ncbi.nlm.nih.gov/38417455/>. For more information, see the package documentation or the vignette titled "Introduction to wintime."
URL: https://pubmed.ncbi.nlm.nih.gov/38417455/
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Imports: survival
Suggests: testthat (>= 3.0.0), knitr, rmarkdown (>= 2.0)
Config/testthat/edition: 3
Depends: R (>= 3.5.0)
NeedsCompilation: no
Packaged: 2026-04-23 13:56:23 UTC; troendlj
Author: James Troendle [aut, cre],
  Samuel Lawrence [aut]
Maintainer: James Troendle <james.troendle@nih.gov>
Repository: CRAN
Date/Publication: 2026-04-23 14:40:02 UTC
Built: R 4.6.0; ; 2026-04-26 05:11:43 UTC; unix

# --------------------
# Imports
# --------------------
importFrom(survival, coxph)
importFrom(survival, survfit)
importFrom(survival, Surv)
importFrom("stats", "coef", "lm", "pnorm", "sd", "vcov", "runif", "var")

# --------------------
# Exports
# --------------------
export(wintime, markov, km, COMP)

## ----- Load Packages----------------------------------------------------------
library(wintime)
library(survival)

## ----- Create Data------------------------------------------------------------

# Event time vectors
TIME_1 <- c(256,44,29,186,29,80,11,380,102,33)
TIME_2 <- c(128,44,95,186,69,66,153,380,117,33)
TIME_3 <- c(435,44,95,186,69,270,1063,380,117,33)

# Event time matrix
Time <- rbind(TIME_1, TIME_2, TIME_3)

# Event indicator vectors
DELTA_1 <- c(1,0,1,0,1,1,1,0,1,0)
DELTA_2 <- c(1,0,0,0,0,1,1,0,0,0)
DELTA_3 <- c(0,0,0,0,0,0,0,0,0,0)

# Event indicator matrix
Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)

# Treatment arm indicator vector
trt <- c(1,1,1,1,1,0,0,0,0,0)

# Covariate vectors
cov1 <- c(66,67,54,68,77,65,55,66,77,54)
cov2 <- c(3,6,4,2,3,5,8,5,3,5)
cov3 <- c(34.6,543.6,45.8,54.7,44.3,55.6,65.9,54.7,77.9,31.2)

# Covariate matrix
cov <- cbind(cov1, cov2, cov3)

cat("Time =", "\n")
print(Time)
cat("Delta =", "\n")
print(Delta)
cat("trt", "\n")
print(trt)
cat("cov =", "\n")
print(cov)

## ----- wtr--------------------------------------------------------------------
# Run wtr
result <- wintime("wtr", Time, Delta, trt)
print(result)

## ----- rwtr-------------------------------------------------------------------
# Run rwtr
result <- wintime("rwtr", Time, Delta, trt)
print(result)

## ----- pwt--------------------------------------------------------------------
# Run pwt
result <- wintime("pwt", Time, Delta, trt)
print(result)

## ----- ewtr-------------------------------------------------------------------
# Run ewtr with covariates 
result <- wintime("ewtr", Time, Delta, trt, cov = cov)
print(result)

## ----- ewt--------------------------------------------------------------------
# Run ewt
result <- wintime("ewt", Time, Delta, trt)
print(result)

## ----- max--------------------------------------------------------------------
# Run max
result <- wintime("max", Time, Delta, trt, cov = cov)
print(result)

## ----- rmt--------------------------------------------------------------------
# Run rmt
result <- wintime("rmt", Time, Delta, trt, time_restriction = round(1.5*365.25))
print(result)

## ----- ewt resampling---------------------------------------------------------
# Run ewt with 10 resamples
result <- wintime("ewt", Time, Delta, trt, resample_num = 10, seed = 123)
print(result)

## ----- wtr resampling---------------------------------------------------------
# Run wtr with 5 resamples
result <- wintime("wtr", Time, Delta, trt, resample_num = 5)
print(result)

## ----- ewt bootstraps---------------------------------------------------------
# Run ewt on 10 bootstraps
result <- wintime("ewt", Time, Delta, trt, resample_num = 10, resample = "boot")
print(result)

## ----- ewtr w/ KM model-------------------------------------------------------
# Run ewtr with a KM model
result <- wintime("ewtr", Time, Delta, trt, cov = cov, model = "km")
print(result)

## ----- set parameters---------------------------------------------------------
# Number of control arm patients
n0 <- sum(trt == 0)

# Number of treatment arm patients
n1 <- sum(trt == 1)

# Number of endpoints
m <- nrow(Time)

## ----- markov-----------------------------------------------------------------
# Run markov
result <- markov(n0, n1, m, Time, Delta)

# Control arm probabilities
dist0 <- result[[1]]
cat("dist0 =", "\n")
print(dist0)

# Treatment arm probabilities
dist1 <- result[[2]]
cat("dist1 =", "\n")
print(dist1)

# Unique control arm event times
untimes0 <- result[[3]]
cat("untimes0 =", "\n")
print(untimes0)

# Unique treatment arm event times
untimes1 <- result[[4]]
cat("untimes1 =", "\n")
print(untimes1)

# Number of unique control arm event times
nuntimes0 <- result[[5]]
cat("nuntimes0 =", "\n")
print(nuntimes0)

# Number of unique treatment arm event times
nuntimes1 <- result[[6]]
cat("nuntimes1 =", "\n")
print(nuntimes1)

# Control arm max follow time
max_follow0 <- result[[7]]
cat("max_follow0 =", "\n")
print(max_follow0)

# Treatment arm max follow time
max_follow1 <- result[[8]]
cat("max_follow1 =", "\n")
print(max_follow1)

# Combined arm probabilities
dist2 <- result[[9]]
cat("dist2 =", "\n")
print(dist2)

# Unique combined arm event times
untimes2 <- result[[10]]
cat("untimes2 =", "\n")
print(untimes2)

# Number of unique combined arm event times
nuntimes2 <- result[[11]]
cat("nuntimes2 =", "\n")
print(nuntimes2)

# Combined arm max follow time
max_follow2 <- result[[12]]
cat("max_follow2 =", "\n")
print(max_follow2)

# Combined arm transition probabilities
trans_prob2 <- result[[13]]
cat("trans_prob2 =", "\n")
print(trans_prob2)

## ----- km---------------------------------------------------------------------
# Run km
result <- km(n0, n1, m, Time, Delta)

# Control arm probabilities
dist0 <- result[[1]]
cat("dist0 =", "\n")
print(dist0)

# Treatment arm probabilities
dist1 <- result[[2]]
cat("dist1 =", "\n")
print(dist1)

# Unique control arm event times
untimes0 <- result[[3]]
cat("untimes0 =", "\n")
print(untimes0)

# Unique treatment arm event times
untimes1 <- result[[4]]
cat("untimes1 =", "\n")
print(untimes1)

# Number of unique control arm event times
nuntimes0 <- result[[5]]
cat("nuntimes0 =", "\n")
print(nuntimes0)

# Number of unique treatment arm event times
nuntimes1 <- result[[6]]
cat("nuntimes1 =", "\n")
print(nuntimes1)

# Control arm max follow time
max_follow0 <- result[[7]]
cat("max_follow0 =", "\n")
print(max_follow0)

# Treatment arm max follow time
max_follow1 <- result[[8]]
cat("max_follow1 =", "\n")
print(max_follow1)

# Combined arm probabilities
dist2 <- result[[9]]
cat("dist2 =", "\n")
print(dist2)

# Unique combined arm event times
untimes2 <- result[[10]]
cat("untimes2 =", "\n")
print(untimes2)

# Number of unique combined arm event times
nuntimes2 <- result[[11]]
cat("nuntimes2 =", "\n")
print(nuntimes2)

# Combined arm max follow time
max_follow2 <- result[[12]]
cat("max_follow2 =", "\n")
print(max_follow2)

# Combined arm KM
comkm <- result[[13]]
cat("comkm =", "\n")
print(comkm)

#' Expected win time against trial population
#'
#' Calculates the combined arm state space probabilities using a Markov model or a Kaplan-Meier model (recommended). This function uses these
#' probabilities to compare each participant's clinical state to a distribution of combined arm states.
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param nunique The number of unique combined arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow The max combined arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes A vector containing unique combined arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param dist A matrix of combined arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param markov_ind An indicator of the model type used (1 for Markov, 0 for Kaplan-Meier).
#' @param cov A n x p matrix of covariate values, where p is the number of covariates.
#' @param trt A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).
#' @param time_restriction The time restriction (days) for calculation.
#' @return A list containing: The estimated treatment effect from the linear regression model, the variance, the Z-statistic,
#' the components of the treatment effect, the variance of the components, and the maximum time used in comparisons.

# -------------------------------------------
# Expected win time against trial population
# -------------------------------------------
REWTP <- function(n,m,nunique,maxfollow,untimes,Time,Delta,dist,markov_ind,cov,trt,time_restriction) {
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]
  components <- rep(NA,m)
  components_var <- rep(NA,m)
  rewtp <- rep(0,n)
  rewtp_components <- matrix(0,nrow=m,ncol=n)
  max_time <- 0


  # Check for extending comparisons when combined arm is 100% in terminal state
  #extend=0
  #if (dist[m+1,nunique]==1) {extend=1}


  # cat('----------------------------------------------','\n')
  # cat('----------------------------------------------','\n')
  # cat('nunique=',nunique,'\n')
  # cat('untimes=',untimes,'\n')
  # cat('time[,3]=',time[,3],'\n')
  # cat('delta[,3]=',delta[,3],'\n')
  # cat('----------------------------------------------','\n')

  # Initialize temporary variables
  #tuntimes <- numeric(nunique+3)
  #tdist <- matrix(0,nrow=m+1,ncol=nunique+m)


  # Start main loop
  for (i in 1:n) {
    # Create temp variables
    # tnunique <- nunique
    # for (j in 1:nunique) {
    #   tuntimes[j] <- untimes[j]
    # }
    #
    # # Copy combined arm distributions
    # for (event in 1:(m+1)) {
    #   for (t in 1:min(nunique,ncol(dist))) {
    #     tdist[event,t] <- dist[event,t]
    #   }
    # }

    # Create addtime matrix
    #addtime <- matrix(data=NA,nrow=m,ncol=n)
    # addtime1=(!(dataset1$time1 %in% unique_event_times0) & dataset1$time1 <= unique_event_times0[nunique_event_times0])
    # addtime2=(!(dataset1$time2 %in% unique_event_times0) & dataset1$delta2==1 & dataset1$time2 <= unique_event_times0[nunique_event_times0])
    # addtime3=(!(dataset1$time3 %in% unique_event_times0) & dataset1$delta3==1 & dataset1$time3 <= unique_event_times0[nunique_event_times0])
    #addtime[1, ] <- (!(time[1, ] %in% untimes) & time[1, ] <= untimes[nunique])
    #addtime[1, ] <- (!(time[1, ] %in% untimes) & (time[1,] <= untimes[nunique] | extend==1))
    #for (k in 2:m) {
      #addtime[k, ] <- (!(time[k, ] %in% untimes) & delta[k, ] == 1 & time[k, ] <= untimes[nunique])
    #  addtime[k, ] <- (!(time[k, ] %in% untimes) & delta[k, ] == 1 & (time[k,] <= untimes[nunique] | extend==1))
    #}
    # cat("-----------------------------------------------", "\n")
    # cat("dim addtime =", dim(addtime), "\n")
    # cat("addtime =","\n")
    # print(addtime)
    # Add events
    # for (event in 1:m) {
    #   if (addtime[event,i] == TRUE) {
    #     jstop <- 0
    #     for (j in 1:tnunique) {
    #       if (tuntimes[j] < time[event,i]) {
    #         jstop <- jstop + 1
    #       }
    #       else {
    #         break
    #       }
    #     }
    #     if(jstop == 0) {
    #       jstop <- 1
    #     }
    #     #tdist[,tnunique] <- 0
    #     tnunique <- tnunique + 1
    #     if (jstop==1) {
    #       tdist[,tnunique] <- c(1,rep(0,times=m))
    #     } else {
    #       tdist[,tnunique] <- tdist[,jstop]
    #     }
    #     tuntimes[tnunique] <- time[event,i]
    #     j <- tnunique
    #
    #     # while(j > jstop) {
    #     #   tdist[,j] <- tdist[,(j-1)]
    #     #   if (j > jstop + 1) {
    #     #     # swap indices j and (j-1)
    #     #     temp <- tuntimes[j]
    #     #     tuntimes[j] <- tuntimes[j-1]
    #     #     tuntimes[j-1] <- temp
    #     #   }
    #     #   j <- j - 1
    #     # }
    #     while(tuntimes[j] < tuntimes[j-1]) {
    #       # swap indices j and (j-1)
    #       temp <- tuntimes[j]
    #       tuntimes[j] <- tuntimes[j-1]
    #       tuntimes[j-1] <- temp
    #       vtemp <- tdist[,j]
    #       tdist[,j] <- tdist[,j-1]
    #       tdist[,j-1] <- vtemp
    #       j <- j - 1
    #     }
    #
    #   }
    # }
    # tuntimes=tuntimes[1:tnunique]

    # Set jmax
    jmax <- 0
    for (j in 1:nunique) {
      if (untimes[j] < time[1,i]) {
        jmax <- jmax + 1
      }
    }
    # if (markov_ind == FALSE) {
    #   jmax <- min(maxfollow,jmax)
    # }
    if (delta[1,i] == 1 | jmax >= nunique) {
      jmax <- nunique - 1
    }
    max_time=max(max_time,untimes[jmax+1])

     # if (i==182) {
     #   cat('----------------------------------------------','\n')
     #   cat('----------------------------------------------','\n')
     #   cat('i=',i,'\n')
     #   cat('jmax=',jmax,'\n')
     #   cat('time_restriction=',time_restriction,'\n')
     #   cat('tnunique=',tnunique,'\n')
     #   cat('tuntimes=',tuntimes,'\n')
     # }

    if (jmax != 0) {
      for (j in 1:jmax) {
        if (untimes[j]>=time_restriction) {break}
        if (untimes[j+1]>time_restriction) {
          time_inc=time_restriction-untimes[j]
        } else {
          time_inc=untimes[j+1]-untimes[j]
        }
        # Set state
        state <- 0
        for (current_state in m:1) {
          temp_time_index <- m - current_state + 1
          if (time[temp_time_index,i] <= untimes[j] && delta[temp_time_index,i] == 1) {
            state <- current_state
            break
          }
        }
         # if (i==182) {
         #   cat('----------------------------------------------','\n')
         #   cat('j=',j,'\n')
         #   cat('state=',state,'\n')
         #   cat('tuntimes[j]=',tuntimes[j],'\n')
         #   cat('tuntimes[j+1]=',tuntimes[j+1],'\n')
         #   cat('time_inc=',time_inc,'\n')
         # }

        # Calculate rewtp

        # Calculate wins
        for (state_num in 0:(m-1)) {
          if (state_num == state) {
            # Add probabilities from higher states
            for (k in (state_num+1):m) {
              rewtp[i] <- rewtp[i] + dist[k+1,j] * (time_inc)
              rewtp_components[k,i] <- rewtp_components[k,i] + dist[k+1,j] * (time_inc)
            }
            break
          }
        }

        # Calculate Losses
        for (current in 1:m) {
          if (current == state) {
            # Subtract probabilities from lower states
            for (k in 1:current) {
              rewtp[i] <- rewtp[i] - dist[k,j] * (time_inc)
              rewtp_components[current,i] <- rewtp_components[current,i] - dist[k,j] * (time_inc)
            }
            break
          }
        }
      }
    }
  }
   # cat('----------------------------------------------------','\n')
   # cat('final rewtp=','\n')
   # print(rewtp)
   # cat('----------------------------------------------------','\n')

  max_time=min(max_time,time_restriction)

  # Get treatment estimate and variance for Z statistic
  fit_comp <- vector("list",m)
  if (!is.null(cov)) {
    fite=lm(rewtp~trt+cov)
    for (k in 1:m) {
      outcome <- rewtp_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt+cov)
    }
  } else {
    fite <- lm(rewtp~trt)
    for (k in 1:m) {
      outcome <- rewtp_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt)
    }
  }
  rewtp_time=coef(fite)[2]
  rewtp_time_var=vcov(fite)[2,2]
  z_rewtp <- rewtp_time/sqrt(rewtp_time_var)
  for (k in 1:m) {
    components[k] <- coef(fit_comp[[k]])[2]
    components_var[k] <- vcov(fit_comp[[k]])[2,2]
  }

  return(list(rewtp_time,rewtp_time_var,z_rewtp,components,components_var,max_time))
}

#' Expected win time against trial population
#'
#' Calculates the combined arm state space probabilities using a Markov model or a Kaplan-Meier model (recommended). This function uses these
#' probabilities to compare each participant's clinical state to a distribution of combined arm states.
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param nunique The number of unique combined arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow The max combined arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes A vector containing unique combined arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param dist A matrix of combined arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param markov_ind An indicator of the model type used (1 for Markov, 0 for Kaplan-Meier).
#' @param cov A n x p matrix of covariate values, where p is the number of covariates.
#' @param trt A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).
#' @return A list containing: The estimated treatment effect from the linear regression model, the variance, the Z-statistic,
#' the components of the treatment effect, the variance of the components, and the maximum time used in comparisons.

# -------------------------------------------
# Expected win time against trial population
# -------------------------------------------
EWTP <- function(n,m,nunique,maxfollow,untimes,Time,Delta,dist,markov_ind,cov,trt) {
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]
  components <- rep(NA,m)
  components_var <- rep(NA,m)
  ewtp <- numeric(n)
  ewtp_components <- matrix(0,nrow=m,ncol=n)
  max_time <- 0


  # cat('Combined arm nunique=',nunique,'\n')
  # cat('---------------------------------------','\n')
  # cat('untimes=','\n')
  # print(untimes)
  # cat('dist[1,]=','\n')
  # print(dist[1,])
  # cat('dist[2,]=','\n')
  # print(dist[2,])
  # cat('dist[3,]=','\n')
  # print(dist[3,])
  # cat('dist[4,]=','\n')
  # print(dist[4,])
  # cat('--------------------------------------','\n')

  # Check for extending comparisons when combined arm is 100% in terminal state
  #extend=0
  #if (dist[m+1,nunique]==1) {extend=1}

  # Initialize temporary variables
  #tuntimes <- numeric(nunique)
  #tdist <- matrix(0,nrow=m+1,ncol=nunique)

  # Start main loop
  for (i in 1:n) {
     # cat('---------------------------','\n')
     # cat('Main loop subject i=',i,'\n')
     # cat('Time[,i]=',Time[,i],'\n')
     # cat('Delta[,i]=',Delta[,i],'\n')

    # Create temp variables
    # tnunique <- nunique
    # for (j in 1:nunique) {
    #   tuntimes[j] <- untimes[j]
    # }

    # Copy combined arm distributions
    # for (event in 1:(m+1)) {
    #   for (t in 1:min(nunique,ncol(dist))) {
    #     tdist[event,t] <- dist[event,t]
    #   }
    # }

  #   cat('---------------------------------------','\n')
  #   cat('Before adding subject i times, tuntimes=','\n')
  #   print(tuntimes)
  #   cat('tdist[1,]=','\n')
  #   print(tdist[1,])
  #   cat('tdist[2,]=','\n')
  #   print(tdist[2,])
  #   cat('tdist[3,]=','\n')
  #   print(tdist[3,])
  #   cat('--------------------------------------','\n')
  #
  # #  Create addtime matrix
  #   addtime <- matrix(data=NA,nrow=m,ncol=n)
  #   addtime[1, ] <- (!(time[1, ] %in% untimes) & (time[1,] <= untimes[nunique] | extend==1))
  #   for (k in 2:m) {
  #     addtime[k, ] <- (!(time[k, ] %in% untimes) & delta[k, ] == 1 & (time[k,] <= untimes[nunique] | extend==1))
  #   }
  #
  #
  # #  Add events
  #   for (event in 1:m) {
  #     if (addtime[event,i] == TRUE) {
  #       jstop <- 0
  #       for (j in 1:tnunique) {
  #         if (tuntimes[j] < time[event,i]) {
  #           jstop <- jstop + 1
  #         }
  #         else {
  #           break
  #         }
  #       }
  #       if(jstop == 0) {
  #         jstop <- 1
  #       }
  #
  #       tnunique <- tnunique + 1
  #       if (jstop==1) {
  #         tdist[,tnunique] <- c(1,rep(0,times=m))
  #       } else {
  #         tdist[,tnunique] <- tdist[,jstop]
  #       }
  #       tuntimes[tnunique] <- time[event,i]
  #       j <- tnunique
  #
  #       while(tuntimes[j] < tuntimes[j-1]) {
  #         # swap indices j and (j-1)
  #         temp <- tuntimes[j]
  #         tuntimes[j] <- tuntimes[j-1]
  #         tuntimes[j-1] <- temp
  #         vtemp <- tdist[,j]
  #         tdist[,j] <- tdist[,j-1]
  #         tdist[,j-1] <- vtemp
  #         j <- j - 1
  #       }
  #
  #     }
  #   }
  #   tuntimes=tuntimes[1:tnunique]

    # cat('--------------------------------------','\n')
    # cat('After adding person i times: tuntimes==','\n')
    # print(tuntimes)
    # cat('tdist[1,]=','\n')
    # print(tdist[1,])
    # cat('tdist[2,]=','\n')
    # print(tdist[2,])
    # cat('tdist[3,]=','\n')
    # print(tdist[3,])
    # cat('--------------------------------------','\n')

    # cat(' # nunique event times after addition of current subject=',tnunique,'\n')
    # cat('unique event times after addition of current subject=',tuntimes,'\n')
    # cat('combined arm dist=','\n')
    # print(tdist)


    # Set jmax
    jmax <- 0
    for (j in 1:nunique) {
      if (untimes[j] < time[1,i]) {
        jmax <- jmax + 1
      }
    }
    #if (markov_ind == FALSE) {
    #  jmax <- min(maxfollow,jmax)
    #}
    if (delta[1,i] == 1 | jmax >= nunique) {
      jmax <- nunique - 1
    }
    max_time=max(max_time,untimes[jmax+1])
#    if (i==3) {
#      cat('jmax=',jmax,'\n')
#    }

    # cat('jmax=',jmax,'\n')
    # cat('---------------------------------------','\n')


    # Set state
    if (jmax != 0) {
      for (j in 1:jmax) {

        #cat('j=',j,'\n')

        state <- 0
        for (current_state in m:1) {
          temp_time_index <- m - current_state + 1
          if (time[temp_time_index,i] <= untimes[j] && delta[temp_time_index,i] == 1) {
            state <- current_state
            break
          }
        }
        #cat('state=',state,'\n')

#        if (i==3) {
#          cat('j=',j,'\n')
#          cat('state=',state,'\n')
#        }

        # Calculate ewtp

        # Calculate wins
        for (state_num in 0:(m-1)) {
          if (state_num == state) {
            # Add probabilities from higher states
            for (k in (state_num+1):m) {
              ewtp[i] <- ewtp[i] + dist[k+1,j] * (untimes[j+1] - untimes[j])
              ewtp_components[k,i] <- ewtp_components[k,i] + dist[k+1,j] * (untimes[j+1]-untimes[j])
            }
            break
          }
        }

        # Calculate Losses
        for (current in 1:m) {
          if (current == state) {
            # Subtract probabilities from lower states
            for (k in 1:current) {
              ewtp[i] <- ewtp[i] - dist[k,j] * (untimes[j+1] - untimes[j])
              ewtp_components[current,i] <- ewtp_components[current,i] - dist[k,j] * (untimes[j+1]-untimes[j])
            }
            break
          }
        }
#        if (i==3) {
#          cat('ewtp=',ewtp[3],'\n')
#        }

        #cat('ewtp[',i,']=',ewtp[i],'\n')
      }
    }
  }
  # cat('----------------------------------------------------','\n')
  # cat('ewtp=','\n')
  # print(ewtp)
  # cat('----------------------------------------------------','\n')

  # Get treatment estimate and variance for Z statistic
  fit_comp <- vector("list",m)
  if (!is.null(cov)) {
    fite=lm(ewtp~trt+cov)
    for (k in 1:m) {
      outcome <- ewtp_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt+cov)
    }
  }
  else {
    fite <- lm(ewtp~trt)
    for (k in 1:m) {
      outcome <- ewtp_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt)
    }
  }
  ewtp_time=coef(fite)[2]
  ewtp_time_var=vcov(fite)[2,2]
  z_ewtp <- ewtp_time/sqrt(ewtp_time_var)
  for (k in 1:m) {
    components[k] <- coef(fit_comp[[k]])[2]
    components_var[k] <- vcov(fit_comp[[k]])[2,2]
  }

  return(list(ewtp_time,ewtp_time_var,z_ewtp,components,components_var,max_time))
}

#' Restricted mean survival in favor of treatment
#'
#' Calculates the state space probabilities using a Kaplan-Meier model (recommended) or a Markov model. This function uses these probabilities
#' to compare both arms and calculate the expected win time of the treatment arm up to a given time point.
#'
#' @param m The number of events in the hierarchy.
#' @param time_restriction The cutoff time point (days) for the calculation.
#' @param dist_state0 A matrix of control arm state probabilities (returned from wintime::km() or wintime::markov()).
#' @param dist_state1 A matrix of treatment arm state probabilities (returned from wintime::km() or wintime::markov()).
#' @param unique_event_times0 A vector of unique control arm event times (days) (returned from wintime::km() or wintime::markov()).
#' @param unique_event_times1 A vector of unique treatment arm event times (days) (returned from wintime::km() or wintime::markov()).
#' @param nunique_event_times0 The number of unique control arm event times (returned from wintime::km() or wintime::markov()).
#' @param nunique_event_times1 The number of unique treatment arm event times (returned from wintime::km() or wintime::markov()).
#' @return A list containing: The restricted mean survival in favor of the treatment arm, the components of the treatment effect, and the maximum time used in comparisons.


# ---------------------------------------------------
# Restricted mean survival in favor of treatment
# ---------------------------------------------------
RMT <- function(m,time_restriction,dist_state0,dist_state1,unique_event_times0,unique_event_times1,nunique_event_times0,nunique_event_times1) {
#  cat("Starting RMT", "\n")
#  cat("time_restriction=",time_restriction,"\n")
  components <- rep(0,m)
  unique_event_times=unique_event_times0
  nunique_event_times=length(unique_event_times)
  new_dist_state1 <- matrix(data=0,nrow=m+1,ncol=nunique_event_times)
  max_time <- 0

  max_follow0=unique_event_times0[nunique_event_times0]
  max_follow1=unique_event_times1[nunique_event_times1]

  # cat("unique_event_times0=","\n")
  # print(unique_event_times0)
  # cat("dist_state0=","\n")
  # print(dist_state0)
  # cat("unique_event_times1=","\n")
  # print(unique_event_times1)
  # cat("dist_state1=","\n")
  # print(dist_state1)


  #cat("after initiation new_dist_state1=","\n")
  #print(new_dist_state1)
  #    cat("event_num=",event_num,"\n")
  #    if (event_num==1) {new_dist_state1[event_num,i] <- 1}

  # Loop 1
  i <- 1
  while(i <= nunique_event_times) {
    x <- length(unique_event_times1[unique_event_times1 <= unique_event_times[i]])
    for (event_num in 1:(m+1)) {
      if (x > 0) {
        new_dist_state1[event_num,i] <- dist_state1[event_num,x]
      }
      else {
        if (event_num==1) {new_dist_state1[event_num,i] <- 1}

      }
    }
    i <- i + 1
  }

  # cat("after inital loop new_dist_state1=","\n")
  # print(new_dist_state1)

  # Loop 2

  # ADD EVENT TIMES FOR TREATED SUBJECTS TO LIST FROM CONTROL SUBJECTS
  #
  tdist_state0 <- matrix(data = 0, nrow = m+1, ncol = ncol(dist_state0))
  #cat("dim tdist_state0 =", dim(tdist_state0), "\n")
  tnew_dist_state1 <- matrix(data = 0, nrow = m+1, ncol = ncol(new_dist_state1))
  i=1
  #    while (i <= 1) {
  while (i <= nunique_event_times1) {
    #cat("loop 2 i =", i, "\n")
    #cat("unique_event_times1[i] =",unique_event_times1[i], "\n")

  #  if (!(unique_event_times1[i] %in% unique_event_times) & unique_event_times1[i] <= unique_event_times[nunique_event_times]) {
    if (!(unique_event_times1[i] %in% unique_event_times)) {
    #cat('Enter ADD EVENT time to List','\n')
      jstop=length(unique_event_times[unique_event_times < unique_event_times1[i]])
      jstar=jstop+1
      if (jstop==0) {jstop=1}

      tdist_state0 <- matrix(data = 0, nrow = m+1, ncol = ncol(tdist_state0)+1)
      tnew_dist_state1 <- matrix(data = 0, nrow = m+1, ncol = ncol(tnew_dist_state1)+1)
      #cat("dim tdist_state0 =", dim(tdist_state0), "\n")



      # Loop through each row and append 0 to the end
      for (k in 1:(m+1)) {
        current_dist_state0 <- c(dist_state0[k,], 0)
        tdist_state0[k, ] <- current_dist_state0

        current_dist_state1 <- c(new_dist_state1[k,], 0)
        tnew_dist_state1[k,] <- current_dist_state1
      }

      # Assign back to original matrices
      dist_state0 <- tdist_state0
      new_dist_state1 <- tnew_dist_state1

      j <- nunique_event_times + 1
      while (j > jstop) {
        for (event_num in 1:(m+1)) {
          dist_state0[event_num,j] <- dist_state0[event_num,j-1]
          new_dist_state1[event_num,j] <- new_dist_state1[event_num,j-1]
        }
        j <- j - 1
      }

      for (event_num in 1:(m+1)) {
        new_dist_state1[event_num,jstar] <- dist_state1[event_num,i]
      }

      unique_event_times=c(unique_event_times,unique_event_times1[i])
      perm=order(unique_event_times)
      unique_event_times=unique_event_times[perm]
      nunique_event_times=length(unique_event_times)
      #cat("Updated # unique_event_times =",nunique_event_times,"\n")
      #cat("Updated unique_event_times =","\n")
      #print(unique_event_times)
    }
    i=i+1
  }

  # cat("nunique_event_times=",nunique_event_times,"\n")
  # cat("unique_event_times=","\n")
  # print(unique_event_times)
  # cat("dist_state0=","\n")
  # print(dist_state0)
  # cat("new_dist_state1=","\n")
  # print(new_dist_state1)

#  cat("unique_event_times=","\n")
#  print(unique_event_times)
#  cat("dist_state0=","\n")
#  print(dist_state0)
#  cat("new_dist_state1=","\n")
#  print(new_dist_state1)


  # RESTRICT LIST TO SMALLER OF MAX FOLLOW-UPS FOR ARMS
  # IF ONE ARM ENTERS 100% TERMINAL STATE, THEN CONTINUE TO LARGER FOLLOW-UP TIME
  #nunique_event_times=length(unique_event_times[unique_event_times <= unique_event_times1[nunique_event_times1]])

  if (max_follow0 <= max_follow1) {
    max_time=max_follow0
    #cat('max_time=',max_time,'\n')
    j=length(unique_event_times[unique_event_times <= max_time])
    #cat('j=',j,'\n')
    #cat('dist_state0[m+1,j]=',dist_state0[m+1,j],'\n')
    if (dist_state0[m+1,j]==1) {max_time=max_follow1}
    #cat('max_time=',max_time,'\n')
  } else {
    max_time=max_follow1
    j=length(unique_event_times[unique_event_times <= max_time])
    if (new_dist_state1[m+1,j]==1) {max_time=max_follow0}
  }
  #max_time=min(max_follow0,max_follow1)
  nunique_event_times=length(unique_event_times[unique_event_times <= max_time])

  #cat(" updated nunique_event_times=",nunique_event_times,"\n")
  #cat(" updated unique_event_times=",unique_event_times,"\n")
  #cat("dist_state0=","\n")
  #print(dist_state0)
  #cat("new_dist_state1=","\n")
  #print(new_dist_state1)

  rmst_time <- 0
  j <- 1

  # Loop 3
  while (j < nunique_event_times) {
    if (unique_event_times[j+1] <= time_restriction) {
#      cat("j=",j,"\n")
      # Add RMST
      for (event_num in 1:m) {
        rmst_time <- rmst_time + new_dist_state1[event_num,j] * sum(dist_state0[((event_num+1):(m+1)),j]) * (unique_event_times[j+1]-unique_event_times[j])
        for (k in (event_num+1):(m+1)) {
          components[k-1] <- components[k-1] + new_dist_state1[event_num,j] * dist_state0[k,j] * (unique_event_times[j+1]-unique_event_times[j])
        }
#        if (j==2) {
#          cat("event_num for treated for win=",event_num,"\n")
#          cat("new_dist_state1[event_num,j]=",new_dist_state1[event_num,j],"\n")
#          cat("sum(dist_state0[((event_num+1):(m+1)),j])=",sum(dist_state0[((event_num+1):(m+1)),j]),"\n")
#          cat("updated RMT=",rmst_time,"\n")
#        }
      }

      # Subtract RMST
      for (event_num in 2:(m+1)) {
        rmst_time <- rmst_time - new_dist_state1[event_num,j] * sum(dist_state0[(1:(event_num-1)),j]) * (unique_event_times[j+1]-unique_event_times[j])
        components[event_num-1] <- components[event_num-1] - new_dist_state1[event_num,j] * sum(dist_state0[(1:(event_num-1)),j]) * (unique_event_times[j+1]-unique_event_times[j])
#        if (j==2) {
#          cat("event_num for treated for loss=",event_num,"\n")
#          cat("updated RMT=",rmst_time,"\n")
#        }
      }
#      cat("updated RMT=",rmst_time,"\n")

    } else {
      if (unique_event_times[j] <= time_restriction) {
#        cat("j=",j,"\n")
        # Add RMST
        for (event_num in 1:m) {
          rmst_time <- rmst_time + new_dist_state1[event_num,j] * sum(dist_state0[((event_num+1):(m+1)),j]) * (time_restriction-unique_event_times[j])
         for (k in (event_num+1):(m+1)) {
            components[k-1] <- components[k-1] + new_dist_state1[event_num,j] * dist_state0[k,j] * (time_restriction-unique_event_times[j])
          }
#           if (j==2) {
#            cat("event_num for treated for win=",event_num,"\n")
#            cat("new_dist_state1[event_num,j]=",new_dist_state1[event_num,j],"\n")
#            cat("sum(dist_state0[((event_num+1):(m+1)),j])=",sum(dist_state0[((event_num+1):(m+1)),j]),"\n")
#            cat("updated RMT=",rmst_time,"\n")
#          }
        }

        # Subtract RMST
        for (event_num in 2:(m+1)) {
          rmst_time <- rmst_time - new_dist_state1[event_num,j] * sum(dist_state0[(1:(event_num-1)),j]) * (time_restriction-unique_event_times[j])
          components[event_num-1] <- components[event_num-1] - new_dist_state1[event_num,j] * sum(dist_state0[(1:(event_num-1)),j]) * (time_restriction-unique_event_times[j])
          #        if (j==2) {
          #          cat("event_num for treated for loss=",event_num,"\n")
          #          cat("updated RMT=",rmst_time,"\n")
          #        }
        }
#        cat("updated RMT=",rmst_time,"\n")
      }
    }
    j <- j + 1
  }

  max_time=min(max_time,time_restriction)
#  cat("final RMT=",rmst_time,"\n")
#  cat("final components of RMT=",components,"\n")
  return(list(rmst_time,components,max_time))
}

#' Fit a Markov model
#'
#' This function fits an extended Markov model to calculate the state probabilities for each arm. In the wintime package, the returned state
#' probability distributions are used in all non-pairwise methods. The extended  Markov model is recommended for the Expected win time against
#' reference (EWTR) method and the EWTR-composite max test (MAX) method.
#'
#' @param n0 The number of participants in the control arm.
#' @param n1 The number of participants in the active treatment arm.
#' @param m The number of events in the hierarchy.
#' @param Time A `m x (n0 + n1)` matrix of event times (days). Rows should represent events and columns should represent participants. Event
#' rows should be in increasing order of clinical severity.
#' @param Delta A `m x (n0 + n1)` matrix of event indicators. Rows should represent events and columns should represent participants.
#' Event rows should be in increasing order of clinical severity.
#' @return A list containing: a matrix of control arm state probabilities, a matrix of treatment arm state probabilities,
#' a vector of unique control arm event times (days), a vector of unique treatment arm event times (days), the number of unique control
#' arm event times, the number of unique treatment arm event times, the control arm max follow time (days), the treatment arm
#' max follow time (days), a matrix of combined arm state probabilities, a vector of unique combined arm event times (days),
#' the number of unique combined arm event times, the combined arm max follow time (days), a (m x m x number of combined arm event times)
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th combined arm event time,
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th trt arm event time,
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th control arm event time.
#' @examples
#' # -----------------------------
#' # Example inputs
#' # -----------------------------
#'
#' # Event time vectors
#' TIME_1 <- c(256,44,29,186,29,80,11,380,102,33)
#' TIME_2 <- c(128,44,95,186,69,66,153,380,117,33)
#' TIME_3 <- c(435,44,95,186,69,270,1063,380,117,33)
#'
#' # Event time matrix
#' Time <- rbind(TIME_1, TIME_2, TIME_3)
#'
#' # Event indicator vectors
#' DELTA_1 <- c(1,0,1,0,1,1,1,0,1,0)
#' DELTA_2 <- c(1,0,0,0,0,1,1,0,0,0)
#' DELTA_3 <- c(0,0,0,0,0,0,0,0,0,0)
#'
#' # Event indicator matrix
#' Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)
#'
#' # Treatment arm indicator vector
#' trt <- c(1,1,1,1,1,0,0,0,0,0)
#'
#' # Number of control arm patients
#' n0 <- sum(trt == 0)
#'
#' # Number of treatment arm patients
#' n1 <- sum(trt == 1)
#'
#' # Number of events in the hierarchy
#' m <- nrow(Time)
#'
#' # -------------------------
#' # markov Examples
#' # -------------------------
#'
#' z <- markov(n0, n1, m, Time, Delta)
#' print(z)
#'
#' @export

# ---------------------
# Markov model
# ---------------------
markov <- function(n0,n1,m,Time,Delta) {
  # -------------------------------
  # Validate inputs
  # -------------------------------
  if (!is.numeric(Time) || !is.numeric(Delta) || !is.numeric(n0) || !is.numeric(n1) || !is.numeric(m)) {
    stop("Input data must be numeric.")
  }
  if (length(Time) == 0 || length(Delta) == 0) {
    stop("Input matrices cannot be empty.")
  }
  if (nrow(Time) != nrow(Delta) || ncol(Time) != ncol(Delta)) {
    stop("Input matrices must have compatible dimensions.")
  }
  if (nrow(Time) != m || nrow(Delta) != m || ncol(Time) != (n0+n1) || ncol(Delta) != (n0+n1)) {
    stop("Rows of input matrices must correspond to events. Columns must correspond to participants. Ensure that the number of events
         is equal to the number of rows and that the number of participants is equal to the columns.")
  }

  n <- n0+n1
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]

  #---------------------------------------------------------
  #  Control Arm
  #  Treat_Group==0
  #---------------------------------------------------------
  event_times0=rep(NA,times=3*n0)
  trans <- array(data=0,dim=c(m,m+1,3*n0))
  nevent_times0=0

  # Loop 1
  i <- 1
  while (i <= n0) {
    current_state <- 0
    nevent <- 0
    for (num_event in m:2) {
      if (delta[num_event,i] == 1) {
        # If the current event time is greater than a higher level event, do not include it
        lt <- TRUE
        k <- 2
        while (k <= (num_event-1)) {
          if (time[k,i] <= time[num_event,i]) {
            lt <- FALSE
          }
          k <- k + 1
        }
        if (lt == TRUE) {
          nevent <- nevent + 1
          event_times0[nevent_times0+nevent] <- time[num_event,i]
          trans[current_state+1,(m-num_event)+1,nevent_times0+nevent] <- 1
          current_state <- (m-num_event)+1
        }
      }
    }
    if (delta[1,i] == 1) {
      nevent <- nevent + 1
      event_times0[nevent_times0+nevent] <- time[1,i]
      trans[current_state+1,m,nevent_times0+nevent] <- 1
      current_state <- m
    }
    else {
      nevent <- nevent + 1
      event_times0[nevent_times0+nevent] <- time[1,i]
      trans[current_state+1,m+1,nevent_times0+nevent] <- 1
    }
    nevent_times0 <- nevent_times0+nevent
    i <- i+1
  }

#cat('nevent_times0=',nevent_times0,'\n')
  event_times0=event_times0[1:nevent_times0]
#cat('event_times0=',event_times0,'\n')
#cat('dim(trans)=',dim(trans),'\n')
  trans <- trans[ , ,1:nevent_times0,drop=FALSE]
#cat('dim(trans)=',dim(trans),'\n')
  perm_event_times0=order(event_times0)
#cat('perm_event_times0=',perm_event_times0,'\n')
  ordered_event_times0=event_times0[perm_event_times0]
  trans <- trans[ , ,perm_event_times0]
#cat('dim(trans)=',dim(trans),'\n')
  unique_event_times0=ordered_event_times0
  num_state <- matrix(data=0,nrow=m+1,ncol=nevent_times0)

  # Initialize all control patients into state 0
  for (k in 1:nevent_times0) {
    num_state[1,k] <- n0
  }

  # Initialize array holding unique state transitions
  unique_trans <- array(data=0,dim=c(m,m+1,nevent_times0))

  i = 1
  nunique_event_times0 = 0

  # Loop 2
  while (i <= nevent_times0) {
    nunique_event_times0 = nunique_event_times0 + 1
    if (i > 1) {
      for (k in 1:(m+1)) {
        num_state[k, nunique_event_times0] <- num_state[k, nunique_event_times0 - 1]
      }
    }
    unique_event_times0[nunique_event_times0] = ordered_event_times0[i]

    j = i
    end = 0
    while (j < nevent_times0 & end == 0) {
      if (ordered_event_times0[j + 1] == ordered_event_times0[j]) {
        j = j + 1
      } else {
        end = 1
      }
    }
    while (i <= j) {
      # Subtract transitions out of state 0
      num_state[1, nunique_event_times0] <- num_state[1, nunique_event_times0] - sum(trans[1, , i])
      # Add and subtract transitions between state 0 and m
      for (event_num in 2:m) {
        num_state[event_num,nunique_event_times0] <- num_state[event_num,nunique_event_times0] + sum(trans[ ,event_num-1,i]) - sum(trans[event_num, ,i])
      }
      # Add transitions into state m
      num_state[m+1,nunique_event_times0] <- num_state[m+1,nunique_event_times0] + sum(trans[ ,m,i])

      # Populate matrix with unique state transitions
      unique_trans[ , ,nunique_event_times0] <- unique_trans[ , ,nunique_event_times0] + trans[ , ,i]
      i = i + 1
    }
  }

  unique_event_times0=unique_event_times0[1:nunique_event_times0]
  tnum_state <- matrix(data=0,nrow=(m+1),ncol=nunique_event_times0)
  for (k in 1:(m+1)) {
    tnum_state[k,] <- num_state[k,1:nunique_event_times0]
  }

  num_state <- tnum_state
  dim_unique_trans <- dim(unique_trans)
  tunique_trans <- array(NA, dim=c(dim_unique_trans[1], dim_unique_trans[2], nunique_event_times0))

  for (i in 1:m) {
    for (j in 1:m+1) {
      tunique_trans[i,j,1:nunique_event_times0] <- unique_trans[i,j,1:nunique_event_times0]
    }
  }

  dist_state0 <- matrix(data=0,nrow=m+1,ncol=nunique_event_times0)
  dist_state0[1,1] <- 1
  trans_prob0 <- array(data=-1,dim=c(m,m,nunique_event_times0))

  from_state0 <- 0
  for (k in 1:m) {
    from_state0 <- from_state0 + unique_trans[1,k,1]
  }

  dist_state0[1,1] <- dist_state0[1,1] * (1 - from_state0/n0)
  for (k in 2:(m+1)) {
    dist_state0[k,1] <- dist_state0[k,1] + unique_trans[1,k-1,1] / n0
  }

  for (k in 1:m) {
    trans_prob0[1,k,1] <- unique_trans[1,k,1] / n0
  }

  max_follow0=0
  for (k in 1:m) {
    if (dist_state0[k,1] > 0 && num_state[k,1] == 0) {
      max_follow0 <- unique_event_times0[1]
      break
    }
  }

   # cat('----------------------------------------------','\n')
   # cat('----------------------------------------------','\n')
   # cat('Before loop3, max_follow0=',max_follow0,'\n')


  # Loop 3
  i <- 2
  while(i <= nunique_event_times0) {
    for (event_num in 1:m) {
      if (num_state[event_num,i-1] > 0) {
        dist_state0[event_num,i] <- dist_state0[event_num,i-1] * (1-((sum(unique_trans[event_num, ,i]))-unique_trans[event_num,m+1,i])/num_state[event_num,i-1])
        for (trans_num in 1:m) {
          trans_prob0[event_num,trans_num,i] <- unique_trans[event_num,trans_num,i]/num_state[event_num,i-1]
        }
      }
      else {
        dist_state0[event_num,i] <- dist_state0[event_num,i-1]
      }
    }

    dist_state0[m+1,i] <- dist_state0[m+1,i-1]

    for (event_num in 1:m) {
      if (num_state[event_num,i-1] > 0) {
        for (trans_num in (event_num+1):(m+1)) {
          dist_state0[trans_num,i] <- dist_state0[trans_num,i] + dist_state0[event_num,i-1] * unique_trans[event_num,trans_num-1,i]/num_state[event_num,i-1]
        }
      }
      if (max_follow0 == 0) {
        if (dist_state0[event_num,i] > 0 && num_state[event_num,i] == 0) {
          max_follow0 <- unique_event_times0[i]
        }
      }
    }
    i <- i + 1
  }
  if (max_follow0==0) {max_follow0=unique_event_times0[nunique_event_times0]}

  # cat('# of event times=',nunique_event_times0,'\n')
  # cat('Control arm unique event times=','\n')
  # print(unique_event_times0)
  # cat('dist_state0[]=','\n')
  # print(dist_state0)
  # cat('num_state[]=','\n')
  # print(num_state)
  # cat('----------------------------------------------','\n')
  # cat('----------------------------------------------','\n')
  # cat('After loop3, max_follow0=',max_follow0,'\n')



#---------------------------------------------------------
#  Trt Arm
#  Treat_Group==1
#---------------------------------------------------------
  event_times1=rep(NA,times=3*n1)
  trans <- array(data=0,dim=c(m,m+1,3*n1))
  nevent_times1=0

  # Loop 1
  i <- n0+1
  while (i <= (n0+n1)) {
    current_state <- 0
    nevent <- 0
    for (num_event in m:2) {
      if (delta[num_event,i] == 1) {
        # If the current event time is greater than a higher level event, do not include it
        lt <- TRUE
        k <- 2
        while (k <= (num_event-1)) {
          if (time[k,i] <= time[num_event,i]) {
            lt <- FALSE
          }
          k <- k + 1
        }
        if (lt == TRUE) {
          nevent <- nevent + 1
          event_times1[nevent_times1+nevent] <- time[num_event,i]
          trans[current_state+1,(m-num_event)+1,nevent_times1+nevent] <- 1
          current_state <- (m-num_event)+1
        }
      }
    }
    if (delta[1,i] == 1) {
      nevent <- nevent + 1
      event_times1[nevent_times1+nevent] <- time[1,i]
      trans[current_state+1,m,nevent_times1+nevent] <- 1
      current_state <- m
    }
    else {
      nevent <- nevent + 1
      event_times1[nevent_times1+nevent] <- time[1,i]
      trans[current_state+1,m+1,nevent_times1+nevent] <- 1
    }
    nevent_times1 <- nevent_times1+nevent
    i <- i+1
  }


  event_times1=event_times1[1:nevent_times1]
  trans <- trans[ , ,1:nevent_times1,drop=FALSE]
  perm_event_times1=order(event_times1)
  ordered_event_times1=event_times1[perm_event_times1]
  trans <- trans[ , ,perm_event_times1]
  unique_event_times1=ordered_event_times1
  num_state <- matrix(data=0,nrow=m+1,ncol=nevent_times1)

  # Initialize all treatment patients into state 0
  for (k in 1:nevent_times1) {
    num_state[1,k] <- n1
  }

  # Initialize array holding unique state transitions
  unique_trans <- array(data=0,dim=c(m,m+1,nevent_times1))

  i = 1
  nunique_event_times1 = 0
  # Loop 2
  while (i <= nevent_times1) {
    nunique_event_times1 = nunique_event_times1 + 1
    if (i > 1) {
      for (k in 1:(m+1)) {
        num_state[k, nunique_event_times1] <- num_state[k, nunique_event_times1 - 1]
      }
    }
    unique_event_times1[nunique_event_times1] = ordered_event_times1[i]

    j = i
    end = 0
    while (j < nevent_times1 & end == 0) {
      if (ordered_event_times1[j + 1] == ordered_event_times1[j]) {
        j = j + 1
      } else {
        end = 1
      }
    }
    # Inner Loop
    while (i <= j) {
      # Subtract transitions out of state 0
      num_state[1, nunique_event_times1] <- num_state[1, nunique_event_times1] - sum(trans[1, , i])
      # Add and subtract transitions between state 0 and m
      for (event_num in 2:m) {
        num_state[event_num,nunique_event_times1] <- num_state[event_num,nunique_event_times1] + sum(trans[ ,event_num-1,i]) - sum(trans[event_num, ,i])
      }
      # Add transitions into state m
      num_state[m+1,nunique_event_times1] <- num_state[m+1,nunique_event_times1] + sum(trans[ ,m,i])

      # Populate array with unique state transitions
      unique_trans[ , ,nunique_event_times1] <- unique_trans[ , ,nunique_event_times1] + trans[ , ,i]
      i = i + 1
    }
  }

  unique_event_times1=unique_event_times1[1:nunique_event_times1]
  tnum_state <- matrix(data=0,nrow=(m+1),ncol=nunique_event_times1)
  for (k in 1:(m+1)) {
    tnum_state[k,] <- num_state[k,1:nunique_event_times1]
  }

  num_state <- tnum_state
  dim_unique_trans <- dim(unique_trans)
  tunique_trans <- array(NA, dim=c(dim_unique_trans[1], dim_unique_trans[2], nunique_event_times1))

  for (i in 1:m) {
    for (j in 1:m+1) {
      tunique_trans[i,j,1:nunique_event_times1] <- unique_trans[i,j,1:nunique_event_times1]
    }
  }

  dist_state1 <- matrix(data=0,nrow=m+1,ncol=nunique_event_times1)
  dist_state1[1,1] <- 1
  trans_prob1 <- array(data=-1,dim=c(m,m,nunique_event_times1))

  from_state0 <- 0
  for (k in 1:m) {
    from_state0 <- from_state0 + unique_trans[1,k,1]
  }

  dist_state1[1,1] <- dist_state1[1,1] * (1 - from_state0/n1)
  for (k in 2:(m+1)) {
    dist_state1[k,1] <- dist_state1[k,1] + unique_trans[1,k-1,1] / n1
  }

  for (k in 1:m) {
    trans_prob1[1,k,1] <- unique_trans[1,k,1] / n1
  }

  max_follow1=0
  for (k in 1:m) {
    if (dist_state1[k,1] > 0 && num_state[k,1] == 0) {
      max_follow1 <- unique_event_times1[1]
      break
    }
  }

  # Loop 3
  i <- 2
  while(i <= nunique_event_times1) {
    for (event_num in 1:m) {
      if (num_state[event_num,i-1] > 0) {
        dist_state1[event_num,i] <- dist_state1[event_num,i-1] * (1-((sum(unique_trans[event_num, ,i]))-unique_trans[event_num,m+1,i])/num_state[event_num,i-1])
        for (trans_num in 1:m) {
          trans_prob1[event_num,trans_num,i] <- unique_trans[event_num,trans_num,i]/num_state[event_num,i-1]
        }
      }
      else {
        dist_state1[event_num,i] <- dist_state1[event_num,i-1]
      }
    }

    dist_state1[m+1,i] <- dist_state1[m+1,i-1]

    for (event_num in 1:m) {
      if (num_state[event_num,i-1] > 0) {
        for (trans_num in (event_num+1):(m+1)) {
          dist_state1[trans_num,i] <- dist_state1[trans_num,i] + dist_state1[event_num,i-1] * unique_trans[event_num,trans_num-1,i]/num_state[event_num,i-1]
        }
      }
      if (max_follow1 == 0) {
        if (dist_state1[event_num,i] > 0 && num_state[event_num,i] == 0) {
          max_follow1 <- unique_event_times1[i]
        }
      }
    }
    i <- i + 1
  }
  if (max_follow1==0) {max_follow1=unique_event_times1[nunique_event_times1]}

  #---------------------------------------------------------
  #  Combined Arms
  #  Treat_Group==2
  #---------------------------------------------------------
  event_times2=rep(NA,times=3*n)
  trans <- array(data=0,dim=c(m,m+1,3*n))
  nevent_times2=0

  # Loop 1
  i <- 1
  while (i <= n) {
    current_state <- 0
    nevent <- 0
    for (num_event in m:2) {
      if (delta[num_event,i] == 1) {
        # If the current event time is greater than a higher level event, do not include it
        lt <- TRUE
        k <- 2
        while (k <= (num_event-1)) {
          if (time[k,i] <= time[num_event,i]) {
            lt <- FALSE
          }
          k <- k + 1
        }
        if (lt == TRUE) {
          nevent <- nevent + 1
          event_times2[nevent_times2+nevent] <- time[num_event,i]
          trans[current_state+1,(m-num_event)+1,nevent_times2+nevent] <- 1
          current_state <- (m-num_event)+1
        }
      }
    }
    if (delta[1,i] == 1) {
      nevent <- nevent + 1
      event_times2[nevent_times2+nevent] <- time[1,i]
      trans[current_state+1,m,nevent_times2+nevent] <- 1
      current_state <- m
    }
    else {
      nevent <- nevent + 1
      event_times2[nevent_times2+nevent] <- time[1,i]
      trans[current_state+1,m+1,nevent_times2+nevent] <- 1
    }
    nevent_times2 <- nevent_times2+nevent
    i <- i+1
  }


  event_times2=event_times2[1:nevent_times2]
  trans <- trans[ , ,1:nevent_times2,drop=FALSE]
  perm_event_times2=order(event_times2)
  ordered_event_times2=event_times2[perm_event_times2]
  trans <- trans[ , ,perm_event_times2]
  unique_event_times2=ordered_event_times2
  num_state <- matrix(data=0,nrow=m+1,ncol=nevent_times2)

  # Initialize all patients into state 0
  for (k in 1:nevent_times2) {
    num_state[1,k] <- n
  }

  # Initialize array holding unique state transitions
  unique_trans <- array(data=0,dim=c(m,m+1,nevent_times2))

  i = 1
  nunique_event_times2 = 0
  # Loop 2
  while (i <= nevent_times2) {
    nunique_event_times2 = nunique_event_times2 + 1
    if (i > 1) {
      for (k in 1:(m+1)) {
        num_state[k, nunique_event_times2] <- num_state[k, nunique_event_times2 - 1]
      }
    }
    unique_event_times2[nunique_event_times2] = ordered_event_times2[i]

    j = i
    end = 0
    while (j < nevent_times2 & end == 0) {

      if (ordered_event_times2[j + 1] == ordered_event_times2[j]) {
        j = j + 1
      } else {
        end = 1
      }
    }
    # Inner Loop
    while (i <= j) {
      # Subtract transitions out of state 0
      num_state[1, nunique_event_times2] <- num_state[1, nunique_event_times2] - sum(trans[1, , i])
      # Add and subtract transitions between state 0 and m
      for (event_num in 2:m) {
        num_state[event_num,nunique_event_times2] <- num_state[event_num,nunique_event_times2] + sum(trans[ ,event_num-1,i]) - sum(trans[event_num, ,i])
      }
      # Add transitions into state m
      num_state[m+1,nunique_event_times2] <- num_state[m+1,nunique_event_times2] + sum(trans[ ,m,i])

      # Populate array with unique state transitions
      unique_trans[ , ,nunique_event_times2] <- unique_trans[ , ,nunique_event_times2] + trans[ , ,i]
      i = i + 1
    }
  }

  unique_event_times2=unique_event_times2[1:nunique_event_times2]
  tnum_state <- matrix(data=0,nrow=(m+1),ncol=nunique_event_times2)
  for (k in 1:(m+1)) {
    tnum_state[k,] <- num_state[k,1:nunique_event_times2]
  }

  num_state <- tnum_state
  dim_unique_trans <- dim(unique_trans)
  tunique_trans <- array(NA, dim=c(dim_unique_trans[1], dim_unique_trans[2], nunique_event_times2))

  for (i in 1:m) {
    for (j in 1:m+1) {
      tunique_trans[i,j,1:nunique_event_times2] <- unique_trans[i,j,1:nunique_event_times2]
    }
  }

  dist_state2 <- matrix(data=0,nrow=m+1,ncol=nunique_event_times2)
  dist_state2[1,1] <- 1
  trans_prob2 <- array(data=-1,dim=c(m,m,nunique_event_times2))

  from_state0 <- 0
  for (k in 1:m) {
    from_state0 <- from_state0 + unique_trans[1,k,1]
  }

  dist_state2[1,1] <- dist_state2[1,1] * (1 - from_state0/n)
  for (k in 2:(m+1)) {
    dist_state2[k,1] <- dist_state2[k,1] + unique_trans[1,k-1,1] / n
  }

  for (k in 1:m) {
    trans_prob2[1,k,1] <- unique_trans[1,k,1] / n
  }

  max_follow2=0
  for (k in 1:m) {
    if (dist_state2[k,1] > 0 && num_state[k,1] == 0) {
      max_follow2 <- unique_event_times2[1]
      break
    }
  }

  # Loop 3
  i <- 2
  while(i <= nunique_event_times2) {
    for (event_num in 1:m) {
      if (num_state[event_num,i-1] > 0) {
        dist_state2[event_num,i] <- dist_state2[event_num,i-1] * (1-((sum(unique_trans[event_num, ,i]))-unique_trans[event_num,m+1,i])/num_state[event_num,i-1])
        for (trans_num in 1:m) {
          trans_prob2[event_num,trans_num,i] <- unique_trans[event_num,trans_num,i]/num_state[event_num,i-1]
        }
      }
      else {
        dist_state2[event_num,i] <- dist_state2[event_num,i-1]
      }
    }

    dist_state2[m+1,i] <- dist_state2[m+1,i-1]

    for (event_num in 1:m) {
      if (num_state[event_num,i-1] > 0) {
        for (trans_num in (event_num+1):(m+1)) {
          dist_state2[trans_num,i] <- dist_state2[trans_num,i] + dist_state2[event_num,i-1] * unique_trans[event_num,trans_num-1,i]/num_state[event_num,i-1]
        }
      }
      if (max_follow2 == 0) {
        if (dist_state2[event_num,i] > 0 && num_state[event_num,i] == 0) {
          max_follow2 <- unique_event_times2[i]
        }
      }
    }
    i <- i + 1
  }
  if (max_follow2==0) {max_follow2=unique_event_times2[nunique_event_times2]}

  # cat('----------------------------------------------','\n')
  # cat('----------------------------------------------','\n')
  # cat('From markov:max_follow0=',max_follow0,'\n')
  # cat('From markov:max_follow1=',max_follow1,'\n')
  # cat('From markov:max_follow2=',max_follow2,'\n')
  # cat('----------------------------------------------','\n')

  return(list(dist_state0 = dist_state0, dist_state1 = dist_state1, unique_event_times0 = unique_event_times0, unique_event_times1 = unique_event_times1,
              nunique_event_times0 = nunique_event_times0, nunique_event_times1 = nunique_event_times1, max_follow0 = max_follow0, max_follow1 = max_follow1,
              dist_state2 = dist_state2, unique_event_times2 = unique_event_times2, nunique_event_times2 = nunique_event_times2, max_follow2 = max_follow2,
              trans_prob2=trans_prob2,trans_prob1=trans_prob1,trans_prob0=trans_prob0))
}

library(testthat)
library(survival)
library(wintime)

# ----------------------------------
# Example inputs
# ----------------------------------

# Event time vectors
Time1 <- c(256,44,29,186,29,190,80,11,380,102,33,303)
Time2 <- c(128,44,95,186,69,190,66,153,380,117,33,131)

# Event time matrix
Time <- rbind(Time1,Time2)

# Event indicator vectors
Delta1 <- c(1,0,1,0,1,0,1,1,0,1,0,1)
Delta2 <- c(1,0,0,0,0,0,1,1,0,0,0,1)

# Event indicator matrix
Delta <- rbind(Delta1,Delta2)

# Treatment arm indicator vector
trt <- c(1,1,1,1,1,1,0,0,0,0,0,0)

# Number of control arm patients
n0 <- sum(trt == 0)

# Number of treatment arm patients
n1 <- sum(trt == 1)

# Number of events in the hierarchy
m <- nrow(Time)

# ---------------------------------
# Test wintime::markov()
# ---------------------------------
test_that("markov function handles valid inputs", {
  result <- markov(n0, n1, m, Time, Delta)
  expect_type(result, "list")
  expect_named(result, c("dist_state0", "dist_state1", "unique_event_times0", "unique_event_times1", "nunique_event_times0", "nunique_event_times1",
                         "max_follow0", "max_follow1","dist_state2","unique_event_times2","nunique_event_times2","max_follow2","trans_prob2",
                         "trans_prob1","trans_prob0"))
})

test_that("markov function handles invalid inputs", {
  # Non-numeric inputs
  expect_error(markov(n0, n1, "invalid", Time, Delta), "Input data must be numeric.")

  # Mismatched dimensions
  Delta_mismatch1 <- c(1,0,1,0,1,0,1,0,1,0,1)
  Delta_mismatch2 <- c(1,0,1,0,1,0,1,0,1,0,1)
  Delta_mismatch <- rbind(Delta_mismatch1, Delta_mismatch2)

  expect_error(markov(n0, n1, m, Time, Delta_mismatch), "Input matrices must have compatible dimensions.")

  # Incorrectly formatted matrices
  trans_Time <- t(Time)
  trans_Delta <- t(Delta)

  expect_error(markov(n0, n1, m, trans_Time, trans_Delta), "Rows of input matrices must correspond to events. Columns must correspond to participants. Ensure that the number of events
         is equal to the number of rows and that the number of participants is equal to the columns.")
})

test_that("markov function handles edge cases", {
  # Empty input matrices
  Time_empty <- matrix(numeric(0), nrow = 0, ncol = 0)
  Delta_empty <- matrix(numeric(0), nrow = 0, ncol = 0)

  expect_error(markov(n0, n1, m, Time_empty, Delta_empty), "Input matrices cannot be empty.")
})

# ------------------------------
# Test wintime::km()
# ------------------------------

test_that("km function handles valid inputs", {
  result <- km(n0, n1, m, Time, Delta)
  expect_type(result, "list")
  expect_named(result, c("dist_state0", "dist_state1", "unique_event_times0", "unique_event_times1", "nunique_event_times0", "nunique_event_times1",
                         "max_follow0", "max_follow1", "dist_state2","unique_event_times2","nunique_event_times2","max_follow2","comkm","trtkm","conkm"))
})

test_that("km function handles invalid inputs", {
  # Non-numeric inputs
  expect_error(km(n0, n1, "invalid", Time, Delta), "Input data must be numeric.")

  # Mismatched dimensions
  Delta_mismatch1 <- c(1,0,1,0,1,0,1,0,1,0,1)
  Delta_mismatch2 <- c(1,0,1,0,1,0,1,0,1,0,1)
  Delta_mismatch <- rbind(Delta_mismatch1, Delta_mismatch2)

  expect_error(km(n0, n1, m, Time, Delta_mismatch), "Input matrices must have compatible dimensions.")

  # Incorrectly formatted matrices
  trans_Time <- t(Time)
  trans_Delta <- t(Delta)

  expect_error(km(n0, n1, m, trans_Time, trans_Delta), "Rows of input matrices must correspond to events. Columns must correspond to participants. Ensure that the number of events
         is equal to the number of rows and that the number of participants is equal to the columns.")
})

test_that("km function handles edge cases", {
  # Empty input matrices
  Time_empty <- matrix(numeric(0), nrow = 0, ncol = 0)
  Delta_empty <- matrix(numeric(0), nrow = 0, ncol = 0)

  expect_error(km(n0, n1, m, Time_empty, Delta_empty), "Input matrices cannot be empty.")
})

#' Helper functions for package functions
#'
#' Win time difference
#'
#' This function calculates the win time difference integral for a single pair. This function is used in all pairwise win time methods.
#'
#' @param m The number of events in the hierarchy.
#' @param etimes A sorted vector of event times (days) (returned from wintime::setEventTimes()).
#' @param time0 A vector containing the control person's event times (days).
#' @param time1 A vector containing the treatment person's event times (days).
#' @param delta0 A vector containing the control person's event indicators.
#' @param delta1 A vector containing the treatment person's event indicators.
#' @return A list of the win time difference integral and its components.

# --------------------------------
# Helper functions
# --------------------------------
getWintimeIntegral <- function(m,etimes,time0,time1,delta0,delta1) {
  integral <- 0
  components <- rep(0,m)
  # Iterate through event times and calculate integral
  for (k in 2:length(etimes)) {
    if (etimes[k] != etimes[k-1]) {
      for (event in m:1) {
        if (time0[event] <= etimes[k-1] && time1[event] > etimes[k-1]) {
          # Treatment person wins
          integral <- integral + etimes[k] - etimes[k-1]
          components[event] <- components[event] + etimes[k] - etimes[k-1]
          break
        }
        else if (time1[event] <= etimes[k-1] && time0[event] > etimes[k-1]) {
          # Treatment person loses
          integral <- integral - etimes[k] + etimes[k-1]
          components[event] <- components[event] - etimes[k] + etimes[k-1]
          break
        }
        # If there is a tie
        else if (time1[event] <= etimes[k-1] && time0[event] <= etimes[k-1]) {
          break
        }
      }
    }
  }

  # If an event is observed on the last day, update the integral value to reflect it
  for (event in m:1) {
    # If both the control and the treatment person have an event time equal to the last day
    # and the control person has an event while the treatment person does not
    if (time0[event] == etimes[length(etimes)] && time1[event] == etimes[length(etimes)]
        && delta0[event] == 1 && delta1[event] == 0) {
      integral <- integral + 1
      components[event] <- components[event] + 1
      break
    }
    # If both the control and the treatment person have an event time equal to the last day
    # and the control person does not have an event while the treatment person does
    if (time0[event] == etimes[length(etimes)] && time1[event] == etimes[length(etimes)]
        && delta0[event] == 0 && delta1[event] == 1) {
      integral <- integral - 1
      components[event] <- components[event] - 1
      break
    }
  }
  return(list(integral,components))
}



#' Win time difference with time restriction
#'
#' This function calculates the win time difference integral for a single pair with truncation at time_restriction.
#' This function is used in all pairwise win time methods.
#'
#' @param m The number of events in the hierarchy.
#' @param etimes A sorted vector of event times (days) (returned from wintime::setEventTimes()).
#' @param time0 A vector containing the control person's event times (days).
#' @param time1 A vector containing the treatment person's event times (days).
#' @param delta0 A vector containing the control person's event indicators.
#' @param delta1 A vector containing the treatment person's event indicators.
#' @param time_restriction The time restriction (days) for calculation.
#' @return A list of the win time difference integral and its components.

# -------------------------------------------
# Win time difference with time restriction
# -------------------------------------------
getWintimeIntegral_rest <- function(m,etimes,time0,time1,delta0,delta1,time_restriction) {
  integral <- 0
  components <- rep(0,m)
  # Iterate through event times and calculate integral
  for (k in 2:length(etimes)) {
    if (etimes[k] != etimes[k-1]) {
      for (event in m:1) {
        if (time0[event] <= etimes[k-1] && time1[event] > etimes[k-1]) {
          # Treatment person wins
          if (etimes[k-1] >= time_restriction) {
            time_inc=0
          } else {
            if (etimes[k] >= time_restriction) {
              time_inc=time_restriction-etimes[k-1]
            } else {
              time_inc=etimes[k]-etimes[k-1]
            }
          }
          integral <- integral + time_inc
          components[event] <- components[event] + time_inc
          break
        }
        else if (time1[event] <= etimes[k-1] && time0[event] > etimes[k-1]) {
          # Treatment person loses
          if (etimes[k-1] >= time_restriction) {
            time_inc=0
          } else {
            if (etimes[k] >= time_restriction) {
              time_inc=time_restriction-etimes[k-1]
            } else {
              time_inc=etimes[k]-etimes[k-1]
            }
          }
          integral <- integral - time_inc
          components[event] <- components[event] - time_inc
          break
        }
        # If there is a tie
        else if (time1[event] <= etimes[k-1] && time0[event] <= etimes[k-1]) {
          break
        }
      }
    }
  }

  # If an event is observed on the last day, update the integral value to reflect it
  for (event in m:1) {
    # If both the control and the treatment person have an event time equal to the last day
    # and the control person has an event while the treatment person does not
    if (time0[event] == etimes[length(etimes)] && time1[event] == etimes[length(etimes)]
        && delta0[event] == 1 && delta1[event] == 0) {
      integral <- integral + 1
      components[event] <- components[event] + 1
      break
    }
    # If both the control and the treatment person have an event time equal to the last day
    # and the control person does not have an event while the treatment person does
    if (time0[event] == etimes[length(etimes)] && time1[event] == etimes[length(etimes)]
        && delta0[event] == 0 && delta1[event] == 1) {
      integral <- integral - 1
      components[event] <- components[event] - 1
      break
    }
  }
  return(list(integral,components))
}



#' Set event times and indicators used in the Kaplan-Meier survival curve calculation
#'
#' This function creates the time_km and delta_km matrices used for wintime::km().
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param time The row reversal of the Time matrix (days) (created inside wintime::km()).
#' @param delta The row reversal of the Delta matrix (created inside wintime::km()).
#' @return A list containing the event time matrix and the event indicator matrix used in wintime::km().

setKM <- function(n,m,time,delta) {
  # Initialize matrices to hold KM times and indicators
  time_km <- matrix(data=0,nrow=m,ncol=n)
  delta_km <- matrix(data=0,nrow=m,ncol=n)


  time_km[1, ] <- time[1, ]
  delta_km[1, ] <- delta[1, ]

  for (event_num in 2:m) {
    time_km[event_num, ] <- pmin(time_km[event_num-1, ], time[event_num, ])
    delta_km[event_num, ] <- delta[event_num, ]
  }

  # Adjust delta_km matrix
  i <- 1
  while (i <= n) {
    for (current_event in 2:m) {
      for (observed_event in 1:(current_event-1)) {
        if (time[observed_event,i] == time_km[current_event,i]) {
          delta_km[current_event,i] <- delta[observed_event,i]
          break
        }
      }
    }
    i <- i + 1
  }
  return(list(time_km,delta_km))
}

#' Created a sorted vector of event times
#'
#' This function creates a sorted vector of event times for a pair. This function is used in all pairwise functions.
#'
#' @param m The number of events in the hierarchy.
#' @param delta0 A vector of event indicators for the control person.
#' @param delta1 A vector of event indicators for the treatment person.
#' @param time0 A vector of event times (days) for the control person.
#' @param time1 A vector of event times (days) for the treatment person.
#' @param follow The maximum follow up time (days).
#' @return A sorted vector of event times (days) for a given pair.

setEventTimes <- function(m,delta0,delta1,time0,time1,follow) {
  # Initialize vector that will hold event times for both groups
  etimes <- numeric(0)

  # Control arm
  for (event in m:1) {
    # If the current event is not observed in person j,
    # or their current event time is greater than the max follow-up time
    if (delta0[event] == 0 || time0[event] > follow) {
      # Set their current event time to the max follow-up time and their indicator to 0
      time0[event] <- follow
      delta0[event] <- 0
    }

    # Append current event time to the event times vector
    etimes <- c(etimes,time0[event])
  }

  #Treatment arm
  for (event in m:1) {
    # If the current event is not observed in person i,
    # or their current event time is greater than the max follow-up time
    if (delta1[event] == 0 || time1[event] > follow) {
      # Set their current event time to the follow-up time and their indicator to 0
      time1[event] <- follow
      delta1[event] <- 0
    }
    # Append current event time to the event times vector
    etimes <- c(etimes,time1[event])
  }

  etimes <- c(etimes,follow)

  # Sorted array of event times
  etimes <- sort(etimes)
  return(etimes)
}

#' Win time ratio
#'
#' This function calculates the ratio of losses to wins on treatment. It iterates through all pairs of treatment and control patients and uses
#' their win time difference as the deciding factor of a win or loss.
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param tau The maximum follow up time (days).
#' @param tg A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should
#' be in increasing order of clinical severity.
#' @return A list containing: The ratio of losses to wins on treatment, the total number of wins, and the total number of losses.

# ------------------------------------
# Win time ratio
# ------------------------------------
WTR <- function(n,m,tau,tg,Time,Delta) {
  win <- matrix(0,nrow=n,ncol=n)
  loss <- matrix(0,nrow=n,ncol=n)

  wintot <- 0
  losstot <- 0

  total <- 0
  npairs <- 0

  # Start main loop
  for (i in 1:n) {
    for (j in 1:n) {
      if ((tg[i] == 1) && (tg[j] == 0))  {
        # Set max follow-up time for the control person to their event m time
        follow0 <- Time[m,j]
        # If event m is observed in person j
        if (Delta[m,j] == 1)  {
          follow0 <- tau
        }
        # Set max follow-up time for the treatment person to their event m time
        follow1 <- Time[m,i]

        # If event m is observed in person i
        if (Delta[m,i] == 1)  {
          follow1 <- tau
        }
        follow <- min(follow0,follow1)

        # Temp variables for times and deltas
        time0 <- Time[1:m,j]
        time1 <- Time[1:m,i]
        delta0 <- Delta[1:m,j]
        delta1 <- Delta[1:m,i]

        # Sorted array of event times
        order <- setEventTimes(m,delta0,delta1,time0,time1,follow)

        # Calculate integral
        #cat('call getWintimeIntegral','\n')
        temp <- getWintimeIntegral(m,order,time0,time1,delta0,delta1)
        integral <- temp[[1]]

        # Update win and loss matrices
        if (integral > 0.0) {
          win[i, j] <- 1
        }
        if (integral < 0.0) {
          loss[i, j] <- 1
        }
        npairs <- npairs + 1
        total <- total + integral

        # Update win and loss totals
        if (integral > 0.0) {
          wintot <- wintot + 1
        }
        if (integral < 0.0) {
          losstot <- losstot + 1
        }
      }
    }
  }
  #cat('return from WTR','\n')
  return(list(losstot/wintot,wintot,losstot))
}

#' Expected win time against reference
#'
#' Calculates the control group state space probabilities using a Markov model (recommended) or a Kaplan-Meier model. This function uses these
#' probabilities to compare each participant's clinical state to a distribution of control group states.
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param nunique The number of unique control group event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow The max control group follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes A vector containing unique control group event times (days) (returned from wintime::markov() or wintime::km()).
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param dist A matrix of control group state probabilities (returned from wintime::markov() or wintime::km()).
#' @param markov_ind An indicator of the model type used (1 for Markov, 0 for Kaplan-Meier).
#' @param cov A n x p matrix of covariate values, where p is the number of covariates.
#' @param trt A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).
#' @return A list containing: The estimated treatment effect from the linear regression model, the variance, the Z-statistic,
#' the components of the treatment effect, and the variance of the components.

# ----------------------------------------
# Expected win time against reference
# ----------------------------------------
EWTR <- function(n,m,nunique,maxfollow,untimes,Time,Delta,dist,markov_ind,cov,trt) {
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]
  components <- rep(NA,m)
  components_var <- rep(NA,m)

  # cat('From ewtr: # unique control event times=',nunique,'\n')
  # cat('From ewtr: unique control times=',untimes,'\n')
  # cat('From ewtr: maximum event time for control arm=',maxfollow,'\n')
  # cat('From ewtr: control arm dist=','\n')
  # print(dist)

  # Check for extending comparisons when control arm is 100% in terminal state
  extend=0
  if (dist[m+1,nunique]==1) {extend=1}

  # cat('-------------------------------------------','\n')
  # cat('extend=',extend,'\n')
  # cat('-------------------------------------------','\n')

  # cat('Before addtime, control arm dist=','\n')
  # print(tdist)
  # cat('Before addtimes, # of tuntimes=',tnunique,'\n')
  # cat('Before addtimes, tuntimes=','\n')
  # print(tuntimes)

  # Initialize temporary variables
  tdist <- matrix(0,nrow=m+1,ncol=nunique+m)
  ewtr <- rep(0,times=n)
  ewtr_components <- matrix(0,nrow=m,ncol=n)

  # Start main loop
  for (i in 1:n) {
     # cat('-------------------------------------------','\n')
     # cat('-------------------------------------------','\n')
     # cat('i=',i,'\n')

    # Create temp variables
    tuntimes <- rep(0,times=nunique+m)
    tnunique <- nunique
    for (j in 1:nunique) {
      tuntimes[j] <- untimes[j]
    }

    # Copy control distributions
    for (event in 1:(m+1)) {
      for (t in 1:min(nunique,ncol(dist))) {
        tdist[event,t] <- dist[event,t]
      }
    }

     # if (i==20) {
     #   cat('Before addtime, control arm dist=','\n')
     #   print(tdist)
     #   cat('Before addtimes, # of tuntimes=',tnunique,'\n')
     #   cat('Before addtimes, tuntimes=','\n')
     #   print(tuntimes)
     # }

    # Create addtime matrix
    addtime <- matrix(data=NA,nrow=m,ncol=n)
    # addtime1=(!(dataset1$time1 %in% unique_event_times0) & dataset1$time1 <= unique_event_times0[nunique_event_times0])
    # addtime2=(!(dataset1$time2 %in% unique_event_times0) & dataset1$delta2==1 & dataset1$time2 <= unique_event_times0[nunique_event_times0])
    # addtime3=(!(dataset1$time3 %in% unique_event_times0) & dataset1$delta3==1 & dataset1$time3 <= unique_event_times0[nunique_event_times0])

    #addtime[1, ] <- (!(time[1, ] %in% untimes) & time[1, ] <= untimes[nunique])
    addtime[1, ] <- (!(time[1, ] %in% tuntimes) & (time[1,] <= tuntimes[tnunique] | extend==1))
    for (k in 2:m) {
      #addtime[k, ] <- (!(time[k, ] %in% untimes) & delta[k, ] == 1 & time[k, ] <= tuntimes[tnunique])
      addtime[k, ] <- (!(time[k, ] %in% tuntimes) & delta[k, ] == 1 & (time[k,] <= tuntimes[tnunique] | extend==1))
    }
    # cat("-----------------------------------------------", "\n")
    # cat("dim addtime =", dim(addtime), "\n")
    # cat("addtime =","\n")
    # print(addtime)
    # Add events

    # cat('addtime (rows in reverse order of event types, columns are subjects)=','\n')
    # print(addtime)

    for (event in 1:m) {
#      if (i==20) {cat('event=',event,'\n')}
      if (addtime[event,i] == TRUE) {
        jstop <- 0
        for (j in 1:tnunique) {
          if (tuntimes[j] < time[event,i]) {
            jstop <- jstop + 1
          }
          else {
            break
          }
        }
        if(jstop == 0) {
          jstop <- 1
        }
 #       if (i==20) {cat('Addtime==TRUE, jstop=',jstop,'\n')}
        #tdist[,tnunique] <- 0
        tnunique <- tnunique + 1
        # if (i==18) {cat('tnunique=',tnunique,'\n')}
        if (jstop==1) {
          tdist[,tnunique] <- c(1,rep(0,times=m))
        } else {
          tdist[,tnunique] <- tdist[,jstop]
        }
        tuntimes[tnunique] <- time[event,i]
        j <- tnunique

#        if (i==20) {cat('tuntimes[tnunique]=',tuntimes[tnunique],'\n')}
#        if (i==20) {cat('j=',j,'\n')}
        #while(j > jstop) {
        #  tdist[,j] <- tdist[,(j-1)]
        #  if (j > jstop + 1) {
            # swap indices j and (j-1)
        #    temp <- tuntimes[j]
        #    tuntimes[j] <- tuntimes[j-1]
        #    tuntimes[j-1] <- temp
        #  }
        #  j <- j - 1
        #}
 #        if (i==20) {cat('tuntimes[j-1]=',tuntimes[j-1],'\n')}
        while(tuntimes[j] < tuntimes[j-1]) {
 #--------------------------------------------------
 # swap indices j and (j-1)
 #--------------------------------------------------
 #         if (i==20) {cat('swap indicies j & j-1=',j,j-1,'\n')}
          temp <- tuntimes[j]
          tuntimes[j] <- tuntimes[j-1]
          tuntimes[j-1] <- temp
          vtemp <- tdist[,j]
          tdist[,j] <- tdist[,j-1]
          tdist[,j-1] <- vtemp
          j <- j - 1
          if (j==jstop) {break}
#          if (i==20) {cat('new index j=',j,'\n')}
        }

      }
    }
    tuntimes=tuntimes[1:tnunique]

    # if (i==20) {
    #   cat('After addtime, control arm dist=','\n')
    #   print(tdist)
    #   cat('After addtimes, # of tuntimes=',tnunique,'\n')
    #   cat('After addtimes, tuntimes=','\n')
    #   print(tuntimes)
    # }

    # cat(' # nunique event times after addition of current subject=',tnunique,'\n')
    # cat('unique event times after addition of current subject=',tuntimes,'\n')
    # cat('control arm dist=','\n')
    # print(tdist)


    # Set jmax
    jmax <- 0
    for (j in 1:tnunique) {
      if (tuntimes[j] < time[1,i]) {
        jmax <- jmax + 1
      }
    }
    #if (markov_ind == FALSE) {
    #  jmax <- min(maxfollow,jmax)
    #}

    if (delta[1,i] == 1) {
      jmax <- tnunique - 1
    }

    # cat('jmax=',jmax,'\n')
    # cat('---------------------------------------','\n')

    # Set state
    if (jmax != 0) {
      for (j in 1:jmax) {

        #cat('j=',j,'\n')

        state <- 0
        for (current_state in m:1) {
          temp_time_index <- m - current_state + 1
          if (time[temp_time_index,i] <= tuntimes[j] && delta[temp_time_index,i] == 1) {
            state <- current_state
            break
          }
        }

        #cat('state=',state,'\n')

        # Calculate ewtr

        # Calculate wins
        for (state_num in 0:(m-1)) {
          if (state_num == state) {
            # Add probabilities from higher states
            for (k in (state_num+1):m) {
              ewtr[i] <- ewtr[i] + tdist[k+1,j] * (tuntimes[j+1] - tuntimes[j])
              ewtr_components[k,i] <- ewtr_components[k,i] + tdist[k+1,j] * (tuntimes[j+1]-tuntimes[j])
            }
            break
          }
        }

        # Calculate Losses
        for (current in 1:m) {
          if (current == state) {
            # Subtract probabilities from lower states
            for (k in 1:current) {
              ewtr[i] <- ewtr[i] - tdist[k,j] * (tuntimes[j+1] - tuntimes[j])
              ewtr_components[current,i] <- ewtr_components[current,i] - tdist[k,j] * (tuntimes[j+1]-tuntimes[j])
            }
            break
          }
        }
        #cat('ewtr[',i,']=',ewtr[i],'\n')
      }
    }
  }

  # Get treatment estimate and variance for Z statistic
  fit_comp <- vector("list",m)
  if (!is.null(cov)) {
    fite=lm(ewtr~trt+cov)
    for (k in 1:m) {
      outcome <- ewtr_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt+cov)
    }
  }
  else {
    fite <- lm(ewtr~trt)
    for (k in 1:m) {
      outcome <- ewtr_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt)
    }
  }
  ewtr_time=coef(fite)[2]
  ewtr_time_var=vcov(fite)[2,2]
  z_ewtr <- ewtr_time/sqrt(ewtr_time_var)
  for (k in 1:m) {
    components[k] <- coef(fit_comp[[k]])[2]
    components_var[k] <- vcov(fit_comp[[k]])[2,2]
  }

  return(list(ewtr_time,ewtr_time_var,z_ewtr,components,components_var))
}

#' Expected win time against trial population With redistribution to the right
#'
#' Calculates the combined arm state space probabilities using a Markov model or a Kaplan-Meier model (recommended).
#' This function uses these probabilities to compare each participant's clinical state to a distribution of combined arm states.
#' Calculation is extended by redistribution-to-the-right principles
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param nunique2 The number of unique combined arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow2 The max combined arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes2 A vector containing unique combined arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param dist2 A matrix of combined arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param markov_ind An indicator of the model type used (1 for Markov, 0 for Kaplan-Meier).
#' @param cov A n x p matrix of covariate values, where p is the number of covariates.
#' @param trt A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).
#' @param comkm A m x nunique2 matrix of combined arm survival probabilities (returned from wintime::markov() or wintime::km()).
#' @param trans_prob2 A (m x m x number of combined arm event times)
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th combined arm event time. (returned from wintime::markov() or wintime::km()).
#' @param nunique1 The number of unique trt arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow1 The max trt arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes1 A vector containing unique trt arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param dist1 A matrix of trt arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param trtkm A m x nunique1 matrix of trt arm survival probabilities (returned from wintime::markov() or wintime::km()).
#' @param trans_prob1 A (m x m x number of trt arm event times)
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th trt arm event time. (returned from wintime::markov() or wintime::km()).
#' @param nunique0 The number of unique control arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow0 The max control arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes0 A vector containing unique control arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param dist0 A matrix of control arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param conkm A m x nunique0 matrix of control arm survival probabilities (returned from wintime::markov() or wintime::km()).
#' @param trans_prob0 A (m x m x number of control arm event times)
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th control arm event time. (returned from wintime::markov() or wintime::km()).
#' @param nimp The number of random imputations.
#' @return A list containing: The estimated treatment effect from the linear regression model, the variance, the Z-statistic, the components of the treatment effect, the variance of the components, and the maximum time used in comparisons.

# -----------------------------------------------------------------------------
# Expected win time against trial population With Redistribution to the Right
# -----------------------------------------------------------------------------
EWTPR <- function(n,m,nunique2,maxfollow2,untimes2,Time,Delta,dist2,markov_ind,cov,trt,comkm,trans_prob2,nunique1,maxfollow1,untimes1,dist1,trtkm,trans_prob1,nunique0,maxfollow0,untimes0,dist0,conkm,trans_prob0,nimp) {
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]
  # trans_prob2[trans_prob2==-1]=0
  # trans_prob1[trans_prob1==-1]=0
  # trans_prob0[trans_prob0==-1]=0
  components <- rep(NA,m)
  components_var <- rep(NA,m)
  imp_components <- matrix(NA,nrow=m,ncol=nimp)
  imp_components_var <- matrix(NA,nrow=m,ncol=nimp)
  max_time <- NA

   # cat("-----------------------------------------------", "\n")
   # cat("Markov_ind=",markov_ind,"\n")
   # cat("-----------------------------------------------", "\n")
   # cat("nunique2 =", nunique2, "\n")
   # cat("nunique1 =", nunique1, "\n")
   # cat("nunique0 =", nunique0, "\n")
   # cat("maxfollow2 =", maxfollow2, "\n")
   # cat("maxfollow1 =", maxfollow1, "\n")
   # cat("maxfollow0 =", maxfollow0, "\n")
    # cat("-----------------------------------------------", "\n")
    # cat("untimes2=", "\n")
    # print(untimes2)
    # cat("-----------------------------------------------", "\n")
    # cat("untimes1=", "\n")
    # print(untimes1)
    # cat("-----------------------------------------------", "\n")
    # cat("untimes0=", "\n")
    # print(untimes0)
   # cat("-----------------------------------------------", "\n")
   # cat("comkm =", "\n")
   # print(comkm)
   # cat("-----------------------------------------------", "\n")
   # cat("trtkm =", "\n")
   # print(trtkm)
   # cat("-----------------------------------------------", "\n")
   # cat("conkm =", "\n")
   # print(conkm)
   # cat("-----------------------------------------------", "\n")
   # cat("dist2 =", "\n")
   # print(dist2)
   # cat("-----------------------------------------------", "\n")
   # cat("dist1 =", "\n")
   # print(dist1)
   # cat("-----------------------------------------------", "\n")
   # cat("dist0 =", "\n")
   # print(dist0)
   # cat("-----------------------------------------------", "\n")
#  cat("trans_prob2 =", "\n")
#  print(trans_prob2)
#  cat("-----------------------------------------------", "\n")
  # cat("trans_prob1 =", "\n")
  # print(trans_prob1)
  # cat("-----------------------------------------------", "\n")
  # cat("trans_prob0 =", "\n")
  # print(trans_prob0)
  # cat("-----------------------------------------------", "\n")

# Get unique times in untimes0,untimes1,untimes2
#
  untimes=rep(0,nunique0+nunique1+nunique2)
  temp=unique(c(untimes0,untimes1,untimes2))
  temp=temp[temp!=0]
#  cat("temp=", "\n")
#  print(temp)
  untimes[1:length(temp)]=temp
  nunique=length(temp)
  untimes=untimes[1:nunique]
  untimes=sort(untimes)

  # cat("-----------------------------------------------", "\n")
  # cat("# of Unique times in untimes0,untimes1,untimes2 nunique=",nunique, "\n")
  # cat("Odered Unique times in untimes0,untimes1,untimes2 untimes=", "\n")
  # print(untimes)
  # cat("-----------------------------------------------", "\n")
#
# Get conkm,trtkm on times from untimes
#
  new_conkm=rep(0,nunique*m)
  dim(new_conkm)=c(m,nunique)
  new_trtkm=rep(0,nunique*m)
  dim(new_trtkm)=c(m,nunique)
  con_count=1
  trt_count=1

  for (i in 1:nunique) {
   # cat("-----------------------------------------------", "\n")
   # cat("i=",i,"\n")
   # cat("con_count=",con_count,"\n")
   # cat("untimes[i]=",untimes[i],"\n")
   # cat("untimes0[con_count]=",untimes0[con_count],"\n")
   # cat("trt_count=",trt_count,"\n")
   # cat("untimes1[trt_count]=",untimes1[trt_count],"\n")
   # cat("-----------------------------------------------", "\n")
    if (con_count < nunique0) {
      if (untimes[i]==untimes0[con_count]) {
        new_conkm[1:m,i]=conkm[1:m,con_count]
        con_count=con_count+1
      } else {
        if (i==1) {
          new_conkm[1:m,i]=1
        } else {
          new_conkm[1:m,i]=new_conkm[1:m,i-1]
        }
      }
    }
    if (trt_count < nunique1) {
      if (untimes[i]==untimes1[trt_count]) {
        new_trtkm[1:m,i]=trtkm[1:m,trt_count]
        trt_count=trt_count+1
      } else {
        if (i==1) {
          new_trtkm[1:m,i]=1
        } else {
          new_trtkm[1:m,i]=new_trtkm[1:m,i-1]
        }
      }
    }
  }
  # cat("-----------------------------------------------", "\n")
   # cat("new conkm and trtkm finished", "\n")
   # cat("-----------------------------------------------", "\n")
    # cat("untimes =", "\n")
    # print(untimes)
    # cat("-----------------------------------------------", "\n")
    # # cat("comkm =", "\n")
    # print(comkm)
    # cat("-----------------------------------------------", "\n")
    # cat("new_trtkm =", "\n")
    # print(new_trtkm)
    # cat("-----------------------------------------------", "\n")
    # cat("new_conkm =", "\n")
    # print(new_conkm)
  # cat("-----------------------------------------------", "\n")

  #
  # Get dist0,dist1,dist2 on times from untimes
  #
  new_dist0=rep(0,nunique*(m+1))
  dim(new_dist0)=c(m+1,nunique)
  new_dist1=rep(0,nunique*(m+1))
  dim(new_dist1)=c(m+1,nunique)
  new_dist2=rep(0,nunique*(m+1))
  dim(new_dist2)=c(m+1,nunique)
  con_count=1
  trt_count=1
  com_count=1

  for (j in 1:nunique) {
    # cat("-----------------------------------------------", "\n")
    # cat("j =",j,"\n")
    # cat("con_count =",con_count,"\n")
    # cat("untimes2[j] =",untimes2[j],"\n")
    # cat("untimes0[con_count] =",untimes0[con_count],"\n")
    if (con_count <= nunique0) {
      if (untimes[j]==untimes0[con_count]) {
        new_dist0[1:(m+1),j]=dist0[1:(m+1),con_count]
        con_count=con_count+1
      } else {
        if (j==1) {
          new_dist0[1,j]=1
          new_dist0[2:(m+1),j]=0
        } else {
          new_dist0[1:(m+1),j]=new_dist0[1:(m+1),j-1]
          #cat("Trigger new_dist0[,j]=new_dist0[,j-1] when time not found in untimes0","\n")
        }
      }
    } else {
      new_dist0[1:(m+1),j]=new_dist0[1:(m+1),j-1]
      #cat("Trigger new_dist0[,j]=new_dist0[,j-1] when con_count > nunique0","\n")
    }
    #cat("new_dist0[1:(m+1),j] =",new_dist0[1:(m+1),j],"\n")
    if (trt_count <= nunique1) {
      if (untimes[j]==untimes1[trt_count]) {
        new_dist1[1:(m+1),j]=dist1[1:(m+1),trt_count]
        trt_count=trt_count+1
      } else {
        if (j==1) {
          new_dist1[1,j]=1
          new_dist1[2:(m+1),j]=0
        } else {
          new_dist1[1:(m+1),j]=new_dist1[1:(m+1),j-1]
        }
      }
    } else {
      new_dist1[1:(m+1),j]=new_dist1[1:(m+1),j-1]
    }
    if (com_count <= nunique2) {
      if (untimes[j]==untimes2[com_count]) {
        new_dist2[1:(m+1),j]=dist2[1:(m+1),com_count]
        com_count=com_count+1
      } else {
        if (j==1) {
          new_dist2[1,j]=1
          new_dist2[2:(m+1),j]=0
        } else {
          new_dist2[1:(m+1),j]=new_dist2[1:(m+1),j-1]
        }
      }
    } else {
      new_dist1[2:(m+1),j]=new_dist2[1:(m+1),j-1]
    }
  }

   # cat("-----------------------------------------------", "\n")
   # cat("new dist0 and dist1 finished", "\n")
   # cat("-----------------------------------------------", "\n")
   # cat("untimes2 =", "\n")
   # print(untimes2)
    # cat("-----------------------------------------------", "\n")
    # cat("dist2 =", "\n")
    # print(dist2)
    # cat("-----------------------------------------------", "\n")
    # cat("dist1 =", "\n")
    # print(dist1)
    # cat("-----------------------------------------------", "\n")
     # cat("dist0 =", "\n")
     # print(dist0)
     # cat("-----------------------------------------------", "\n")
    # cat("-----------------------------------------------", "\n")
    # cat("new_dist2 =", "\n")
    # print(new_dist2)
    # cat("-----------------------------------------------", "\n")
    # cat("new_dist1 =", "\n")
    # print(new_dist1)
     # cat("-----------------------------------------------", "\n")
     # cat("new_dist0 =", "\n")
     # print(new_dist0)
     # cat("-----------------------------------------------", "\n")
   # cat("-----------------------------------------------", "\n")

  #
  # Get trans_prob0,trans_prob1 on times from untimes
  #
  new_trans_prob0=rep(0,nunique*m*m)
  dim(new_trans_prob0)=c(m,m,nunique)
  new_trans_prob1=rep(0,nunique*m*m)
  dim(new_trans_prob1)=c(m,m,nunique)
  con_count=1
  trt_count=1

  for (j in 1:nunique) {
    if (con_count < nunique0) {
      if (untimes[j]==untimes0[con_count]) {
        new_trans_prob0[1:m,1:m,j]=trans_prob0[1:m,1:m,con_count]
        con_count=con_count+1
      } else {
        new_trans_prob0[1:m,1:m,j]=0
      }
    }
    if (trt_count < nunique1) {
      if (untimes[j]==untimes1[trt_count]) {
        new_trans_prob1[1:m,1:m,j]=trans_prob1[1:m,1:m,trt_count]
        trt_count=trt_count+1
      } else {
        new_trans_prob1[1:m,1:m,j]=0
      }
    }
  }

# cat("new trans_probs finished", "\n")
   # cat("-----------------------------------------------", "\n")
   # cat("new_trans_prob1 =", "\n")
   # print(new_trans_prob1)
   # cat("-----------------------------------------------", "\n")
   # cat("new_trans_prob0 =", "\n")
   # print(new_trans_prob0)
   # cat("-----------------------------------------------", "\n")

  nunique0=length(untimes[untimes <= maxfollow0])
  nunique1=length(untimes[untimes <= maxfollow1])


   # cat("-----------------------------------------------", "\n")
   # cat("new trtkm =", "\n")
   # print(new_trtkm)
   # cat("-----------------------------------------------", "\n")
   # cat("new conkm =", "\n")
   # print(new_conkm)
   # cat("-----------------------------------------------", "\n")
   # cat("new dist1 =", "\n")
   # print(new_dist1)
   # cat("-----------------------------------------------", "\n")
   # cat("new dist0 =", "\n")
   # print(new_dist0)
   # cat("-----------------------------------------------", "\n")

  # cat("-----------------------------------------------", "\n")
  # cat("maxfollow2 =", maxfollow2, "\n")
  # cat("maxfollow0 =", maxfollow0, "\n")
  # cat("maxfollow1 =", maxfollow1, "\n")
   # cat("Combined unique times =", "\n")
   # print(untimes)
   # cat("-----------------------------------------------", "\n")
   # cat("nunique0=",nunique0,"\n")
   # cat("nunique1=",nunique1,"\n")
   # cat("-----------------------------------------------", "\n")
  # cat("unique control times =", "\n")
  # print(untimes0)
  # cat("-----------------------------------------------", "\n")
  #
  #
  # cat("-----------------------------------------------", "\n")
  # cat("Largest # of times combined arm for redist-to-the-right =", nunique2, "\n")
  # cat("Largest time combined arm for redist-to-the-right =", untimes2[nunique2], "\n")
  # cat("Largest # of times for same arm for redist-to-the-right con arm =", nunique0, "\n")
  # cat("Largest time for same arm for redist-to-the-right con arm =", untimes2[nunique0], "\n")
  # cat("Largest # of times for same arm for redist-to-the-right trt arm =", nunique1, "\n")
  # cat("Largest time for same arm for redist-to-the-right trt arm =", untimes2[nunique1], "\n")
  # cat("time_restriction =", time_restriction, "\n")
  # cat("-----------------------------------------------", "\n")


  # if (new_dist0[m+1,nunique0]==1) {cat('new_dist0 extends','\n')}
  # if (new_dist1[m+1,nunique1]==1) {cat('new_dist1 extends','\n')}
  # if (new_dist2[m+1,nunique2]==1) {cat('new_dist2 extends','\n')}

  #-------------------------------------------------------
  # Set jsamemark
  #-------------------------------------------------------
  if (new_dist0[m+1,nunique0]==1) {
    if (new_dist1[m+1,nunique1]==1) {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=max(max(maxfollow0,maxfollow1),maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=maxfollow2
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    } else {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=maxfollow1
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=min(maxfollow1,maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    }
  } else {
    if (new_dist1[m+1,nunique1]==1) {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=maxfollow0
        #cat('Trigger max_time=',max_time,'\n')
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=min(maxfollow0,maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    } else {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=min(maxfollow0,maxfollow1)
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=min(min(maxfollow0,maxfollow1),maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    }
  }

  #currentseed=.Random.seed[2]
  #currentseed=.Random.seed[currentseed+2]
  #cat("current seed for EWTPR calculation =",.Random.seed, "\n")
  # cat("-----------------------------------------------", "\n")

  #--------------------------------------------------------
  # FOR COMPARISON WITH FORTRAN
  # set.seed(1092423368)
  # set.seed(18945611)
  # set.seed(99803332)
  # random=runif(n*nunique*nimp)
  # iran=1
  #------------------------------------------------------
  #cat("Next 10 random numbers =",random[1:10], "\n")
  #cat("-----------------------------------------------", "\n")


  ewtpr_time=rep(0,nimp)
  ewtpr_time_var=rep(0,nimp)

  # Set jfinalmark
  #jfinalmark=nunique-1


  # START MULTIPLE IMPUTATION LOOP
  for (imp in 1:nimp) {

      # cat('---------------------------------','\n')
      # cat('imp=',imp,'\n')
      # cat('iran=',iran,'\n')
      # cat('next 10 random=',random[iran:(iran+9)],'\n')


    # Initialize temporary variables
    ewtpr <- numeric(n)
    ewtpr_components <- matrix(0,nrow=m,ncol=n)

  # START LOOP OVER SUBJECTS
  for (i in 1:n) {
     # cat('---------------------------------','\n')
     # cat('i=',i,'\n')

#     if (i==1) {
       # cat("-----------------------------------------------", "\n")
       # cat("-----------------------------------------------", "\n")
       # cat("i=" ,i, "\n")
       # cat("time[,i]" ,time[,i], "\n")
       # cat("delta[,i]" ,delta[,i], "\n")
#       cat("trt[i]" ,trt[i], "\n")
#       cat("nunique0=" ,nunique0, "\n")
#       cat("nunique1=" ,nunique1, "\n")
#       cat("nunique2=" ,nunique2, "\n")
# #      cat("trans_prob[1,1,]" ,trans_prob[1,1,], "\n")
# #      cat("trans_prob[1,2,]" ,trans_prob[1,2,], "\n")
# #      cat("trans_prob[1,3,]" ,trans_prob[1,3,], "\n")
# #      cat("trans_prob[2,2,]" ,trans_prob[2,2,], "\n")
# #      cat("trans_prob[2,3,]" ,trans_prob[2,3,], "\n")
# #      cat("trans_prob[3,3,]" ,trans_prob[3,3,], "\n")
#      cat("-----------------------------------------------", "\n")
#    }

    # Set jmax
    jmax <- 0
    for (j in 1:nunique) {
      if (untimes[j] < time[1,i]) {
        jmax <- jmax + 1
      }
    }
    # if (markov_ind == FALSE) {
    #   jmax <- min(maxfollow2,jmax)
    # }
    if (delta[1,i] == 1 | jmax >= nunique) {
      jmax <- nunique - 1
    }


    #if (trt[i]==1) {jsamemark=nunique1-1}
    #if (trt[i]==0) {jsamemark=nunique0-1}


    #jsamemark=min(min(nunique0,nunique1),nunique2)-1
    #if (jsamemark == nunique2) {jsamemark=nunique2-1}


    # cat('Before enforce comparison ends at jsamemark','\n')
    # cat('jmax=',jmax,'\n')
    # cat('jsamemark=',jsamemark,'\n')

    # Enforce comparison ends at jsamemark
    jmax=min(jmax,jsamemark)

    # cat('After enforce on jmax','\n')
     # cat('jmax=',jmax,'\n')
     # cat('jsamemark=',jsamemark,'\n')

    #if (jsamemark < jmax) {jsamemark=jmax}


   #  if (i==1) {
      # cat('i=',i,'\n')
      # cat('After enforce on jmax so comparison ends at jsamemark ','\n')
      # cat('jmax=',jmax,'\n')
   #   cat('jfinalmark=',jfinalmark,'\n')
      #cat('jsamemark=',jsamemark,'\n')
   #   cat('nunique0=',nunique0,'\n')
   #   cat('nunique1=',nunique1,'\n')
   #   cat('nunique2=',nunique2,'\n')
   #   cat('length(new_conkm[1,])=',length(new_conkm[1,]),'\n')
      # cat('time[,i]=',time[,i],'\n')
      # cat('delta[,i]=',delta[,i],'\n')
   # }

    state=0
    if (jmax != 0) {
# Comparison of known state to combined arm dist
      for (j in 1:jmax) {
        #cat('j=',j,'\n')
        state <- 0
        for (current_state in m:1) {
          #cat('current_state=',current_state,'\n')
          temp_time_index <- m - current_state + 1
          #cat('temp_time_index=',temp_time_index,'\n')
          if (time[temp_time_index,i] <= untimes[j] && delta[temp_time_index,i] == 1) {
            state <- current_state
            break
          }
        }
        # if (i==1) {
        #   cat("-----------------------------------------------", "\n")
        #   cat('j=',j,'\n')
        #   cat('state=',state,'\n')
        #   cat("untimes2[j]" ,untimes2[j], "\n")
        #   cat("untimes2[j+1]" ,untimes2[j+1], "\n")
        #   cat('dist2[,j]=','\n')
        #   print(dist2[,j])
        #   cat("-----------------------------------------------", "\n")
        # }

        # Calculate ewtpr

        # Calculate wins
        for (state_num in 0:(m-1)) {
          if (state_num == state) {
            # Add probabilities from higher states
            for (k in (state_num+1):m) {
              ewtpr[i] <- ewtpr[i] + new_dist2[k+1,j] * (untimes[j+1] - untimes[j])
              ewtpr_components[k,i] <- ewtpr_components[k,i] + new_dist2[k+1,j] * (untimes[j+1]-untimes[j])
            }
            break
          }
        }

        # Calculate Losses
        for (current in 1:m) {
          if (current == state) {
            # Subtract probabilities from lower states
            for (k in 1:current) {
              ewtpr[i] <- ewtpr[i] - new_dist2[k,j] * (untimes[j+1] - untimes[j])
              ewtpr_components[current,i] <- ewtpr_components[current,i] - new_dist2[k,j] * (untimes[j+1]-untimes[j])
            }
            break
          }
        }
      }
    }

# Initialize dist_state
    state_dist=rep(0,m+1)
    for (k in 0:m) {
      if (state==k) {state_dist[k+1]=1}
    }
    new_state_dist=state_dist

    #if (i==4) {
    # cat("-----------------------------------------------", "\n")
    # cat('After jmax','\n')
    # cat('comparison to time=',untimes[jmax],'\n')
    # cat('ewtpr before redistribution=',ewtpr[i],'\n')
    # cat('state=',state,'\n')
    # cat('state_dist=',state_dist,'\n')
    # #   # cat('comkm[,36]=',comkm[,36],'\n')
    # #   # cat('comkm[,37]=',comkm[,37],'\n')
    #}
#
# Start Redistribution-to-the-right using same arm
#
    if (jmax < jsamemark) {
    for (j in (jmax+1):jsamemark) {

       # if (i==4) {
       # cat("-----------------------------------------------", "\n")
       # cat('j> jmax=',j,'\n')
       # cat('iran=',iran,'\n')
       # cat('untimes[j]=',untimes[j],'\n')
       # cat('untimes[j+1]=',untimes[j+1],'\n')
       # }
#
# CHECK FOR NO EXTENDED COMPARISON
#
      if (!(trt[i]==0 & new_dist0[m+1,j]==1) & !(trt[i]==1 & new_dist1[m+1,j]==1)) {

# Update state_dist

      if (markov_ind == 0) {
      # KM Model

      if (trt[i]==0) {
      # RTTR by con arm

        sum=state_dist[1]
        # if (i==20 & j==37) {
        #   cat('sum=',sum,'\n')
        # }

        if (j != 1) {
          for (k in 1:m) {
            if (new_conkm[k,j-1] != 0) {
              new_state_dist[k]=sum*new_conkm[k,j]/new_conkm[k,j-1]
              sum=sum+state_dist[k]
            } else {
              new_state_dist[k]=sum*new_conkm[k,j]
              sum=sum+state_dist[k]
            }
          }
        } else {
          for (k in 1:m) {
            new_state_dist[k]=sum*new_conkm[k,j]
            sum=sum+state_dist[k]
          }
        }
# Enforce monitonicity
        for (k in 2:m) {
          if (new_state_dist[k] < new_state_dist[k-1]) {new_state_dist[k]=new_state_dist[k-1]}
        }
        for (k in 1:(m+1)) {
          if (k==m+1) {
            state_dist[k]=1-new_state_dist[m]
          } else {
            if (k==1) {
              state_dist[k]=new_state_dist[1]
            } else {
              state_dist[k]=new_state_dist[k]-new_state_dist[k-1]
            }
          }
        }
      } else {
      # RTTR by trt arm

        sum=state_dist[1]
        # if (i==20 & j==37) {
        #   cat('sum=',sum,'\n')
        # }

        if (j !=1) {
          for (k in 1:m) {
            if (new_trtkm[k,j-1] != 0) {
              new_state_dist[k]=sum*new_trtkm[k,j]/new_trtkm[k,j-1]
              sum=sum+state_dist[k]
            } else {
              new_state_dist[k]=sum*new_trtkm[k,j]
              sum=sum+state_dist[k]
            }
          }
        } else {
          for (k in 1:m) {
            new_state_dist[k]=sum*new_trtkm[k,j]
            sum=sum+state_dist[k]
          }
        }
        # Enforce monitonicity
        for (k in 2:m) {
          if (new_state_dist[k] < new_state_dist[k-1]) {new_state_dist[k]=new_state_dist[k-1]}
        }
        for (k in 1:(m+1)) {
          if (k==m+1) {
            state_dist[k]=1-new_state_dist[m]
          } else {
            if (k==1) {
              state_dist[k]=new_state_dist[1]
            } else {
              state_dist[k]=new_state_dist[k]-new_state_dist[k-1]
            }
          }
        }
      }
      } else {
      # Markov Model

      if (trt[i]==0) {
      # RTTR by con arm

        trans_out <- array(data=0,dim=c(m))
        for (l in 1:m) {
          for (k in l:m) {
#           if (i==1) {
#             cat('trans_out[l] calculation for l=',l,' with k=',k,'\n')
#           }
            trans_out[l]=trans_out[l]+new_trans_prob0[l,k,j]
          }
        }
        trans_in <- array(data=0,dim=c(m,m))
        for (l in 1:m) {
         for (k in 1:l) {
#            if (i==1) {
#              cat('trans_in[l] calculation for l=',l,' with k=',k,'\n')
#            }
            trans_in[k,l]=trans_in[k,l]+new_trans_prob0[k,l,j]
          }
        }

       # if (i==13 & j==103) {
       #   cat(' trans_in=','\n')
       #   print(trans_in)
       #   cat(' trans_out=','\n')
       #   print(trans_out)
       #   cat(' new_trans_prob0[,,j]=','\n')
       #   print(new_trans_prob0[,,j])
       # }


        for (k in 1:(m+1)) {
          if (k <= m) {
            new_state_dist[k]=state_dist[k]*(1-trans_out[k])
          } else {
            new_state_dist[k]=state_dist[k]
          }
          if (k > 1) {
            for (l in 1:(k-1)) {
              new_state_dist[k]=new_state_dist[k]+state_dist[l]*trans_in[l,k-1]
            }
          }
        }
        state_dist=new_state_dist
      } else {
      # RTTR by trt arm

        trans_out <- array(data=0,dim=c(m))
        for (l in 1:m) {
          for (k in l:m) {
            #           if (i==1) {
            #             cat('trans_out[l] calculation for l=',l,' with k=',k,'\n')
            #           }
            trans_out[l]=trans_out[l]+new_trans_prob1[l,k,j]
          }
        }
        trans_in <- array(data=0,dim=c(m,m))
        for (l in 1:m) {
          for (k in 1:l) {
            #            if (i==1) {
            #              cat('trans_in[l] calculation for l=',l,' with k=',k,'\n')
            #            }
            trans_in[k,l]=trans_in[k,l]+new_trans_prob1[k,l,j]
          }
        }

               # if (i==13 & j==103) {
               #   cat(' trans_in=',trans_in,'\n')
               #   cat(' trans_out=',trans_out,'\n')
               # }


        for (k in 1:(m+1)) {
          if (k <= m) {
            new_state_dist[k]=state_dist[k]*(1-trans_out[k])
          } else {
            new_state_dist[k]=state_dist[k]
          }
          if (k > 1) {
            for (l in 1:(k-1)) {
              new_state_dist[k]=new_state_dist[k]+state_dist[l]*trans_in[l,k-1]
            }
          }
        }
        state_dist=new_state_dist
      }
      }
      #END State Dist Update


       # if (i==4 & j==34) {
       #   cat(' Before randomness new state_dist=',state_dist,'\n')
       #   cat(' iran=',iran,'\n')
       #   cat(' random=',random[iran],'\n')
       # }

#-----------------------------------------------
# Use Randomness to determine a state
#-----------------------------------------------
# NOT FOR COMPARISON WITH FORTRAN
      zz=runif(1)

      sum=state_dist[1]
      for (k in 1:(m+1)) {

# NOT FOR COMPARISON WITH FORTRAN
        if (zz < sum | k==m+1) {
# FOR COMPARISON WITH FORTRAN
        #if (random[iran] < sum | k==m+1) {
        #  iran=iran+1
# END: FOR COMPARISON WITH FORTRAN

          state_dist[1:(m+1)]=0
          state_dist[k]=1

          break
        }
        sum=sum+state_dist[k+1]
      }

      # if (i==13 & j==103) {
      #   cat(' After randomness state_dist=',state_dist,'\n')
      # }


      } else {
# EXTENDED COMPARISON
        state_dist[1:(m+1)]=0
        state_dist[m+1]=1
      }
# END OF CHECK FOR EXTENDED COMPARISON

# Update EWTPR for Wins
      sum1=sum(new_dist2[2:(m+1),j])
      for (k in 1:m) {
        # if (i==6 & j==76) {
        #   cat(' k=',k,'\n')
        #   cat('sum1=',sum1,'\n')
        # }
        ewtpr[i] <- ewtpr[i] + state_dist[k] * sum1 * (untimes[j+1] - untimes[j])
        # if (i==6 & j==76) {
        #   cat('ewtpr=',ewtpr[i],'\n')
        # }
        sum1=sum1-new_dist2[k+1,j]
        for (l in (k+1):(m+1)) {
          ewtpr_components[l-1,i] <- ewtpr_components[l-1,i] + state_dist[k] * new_dist2[l,j] * (untimes[j+1]-untimes[j])
        }
      }
      # if (i==6 & j==76) {
      #   cat('After Wins ewtpr=',ewtpr[i],'\n')
      # }

      # Update EWTPR for Losses
      sum1=sum(new_dist2[1:m,j])
      for (k in (m+1):2) {
        # if (i==6 & j==76) {
        #   cat('k=',k,'\n')
        #   cat('state_dist[k]=',state_dist[k],'\n')
        # }
        ewtpr[i] <- ewtpr[i] - state_dist[k] * sum1 * (untimes[j+1] - untimes[j])
        ewtpr_components[k-1,i] <- ewtpr_components[k-1,i] - state_dist[k] * sum1 * (untimes[j+1]-untimes[j])
        sum1=sum1-new_dist2[k-1,j]
      }

      # if (i==4) {
      #   cat('After update j>jmax=',j,'\n')
      #   cat('ewtpr[i]=',ewtpr[i],'\n')
      #   cat('state_dist=',state_dist,'\n')
      #    cat("untimes[j]" ,untimes[j], "\n")
      #    cat("untimes[j+1]" ,untimes[j+1], "\n")
      #    cat('dist2[,j]=','\n')
      #    print(dist2[,j])
      #   cat("-----------------------------------------------", "\n")
      # }
    }
    }
# End Redistribution-to-the-right using same arm

    # if (i==13) {
    #   cat('After jsamemark=','\n')
    #   cat('ewtpr[i]=',ewtpr[i],'\n')
    #   cat('state=',state,'\n')
    #   cat("-----------------------------------------------", "\n")
    # }



# Start Redistribution-to-the-right using combined arms

    # if (jsamemark < jfinalmark) {
    #   for (j in (jsamemark+1):jfinalmark) {
    #
    #      # if (i==13 & j>90 & j<120) {
    #      #   cat('j> jsamemark=',j,'\n')
    #      #   cat('iran=',iran,'\n')
    #      # }
    #
    #     # Update state_dist
    #     if (markov_ind == 0) {
    #
    #       # KM Model
    #
    #       sum=state_dist[1]
    #         # if (i==20 & j==37) {
    #         #   cat('sum=',sum,'\n')
    #         # }
    #
    #       if (j !=1) {
    #         for (k in 1:m) {
    #           if (comkm[k,j-1] != 0) {
    #             new_state_dist[k]=sum*comkm[k,j]/comkm[k,j-1]
    #             sum=sum+state_dist[k]
    #           } else {
    #             new_state_dist[k]=sum*comkm[k,j]
    #             sum=sum+state_dist[k]
    #           }
    #         }
    #       } else {
    #         for (k in 1:m) {
    #           new_state_dist[k]=sum*comkm[k,j]
    #           sum=sum+state_dist[k]
    #         }
    #       }
    #       # Enforce monitonicity
    #       for (k in 2:m) {
    #         if (new_state_dist[k] < new_state_dist[k-1]) {new_state_dist[k]=new_state_dist[k-1]}
    #       }
    #       for (k in 1:(m+1)) {
    #         if (k==m+1) {
    #           state_dist[k]=1-new_state_dist[m]
    #         } else {
    #           if (k==1) {
    #             state_dist[k]=new_state_dist[1]
    #           } else {
    #             state_dist[k]=new_state_dist[k]-new_state_dist[k-1]
    #           }
    #         }
    #       }
    #     } else {
    #     # Markov Model
    #
    #       trans_out <- array(data=0,dim=c(m))
    #       for (l in 1:m) {
    #         for (k in l:m) {
    #             #           if (i==1) {
    #             #             cat('trans_out[l] calculation for l=',l,' with k=',k,'\n')
    #             #           }
    #           trans_out[l]=trans_out[l]+trans_prob2[l,k,j]
    #         }
    #       }
    #       trans_in <- array(data=0,dim=c(m,m))
    #       for (l in 1:m) {
    #         for (k in 1:l) {
    #             #            if (i==1) {
    #             #              cat('trans_in[l] calculation for l=',l,' with k=',k,'\n')
    #             #            }
    #           trans_in[k,l]=trans_in[k,l]+trans_prob2[k,l,j]
    #         }
    #       }
    #
    #         #        if (i==1) {
    #         #          cat(' trans_in=',trans_in,'\n')
    #         #          cat(' trans_out=',trans_out,'\n')
    #         #        }
    #
    #
    #       for (k in 1:(m+1)) {
    #         if (k <= m) {
    #            new_state_dist[k]=state_dist[k]*(1-trans_out[k])
    #         } else {
    #            new_state_dist[k]=state_dist[k]
    #          }
    #         if (k > 1) {
    #           for (l in 1:(k-1)) {
    #             new_state_dist[k]=new_state_dist[k]+state_dist[l]*trans_in[l,k-1]
    #           }
    #         }
    #       }
    #       state_dist=new_state_dist
    #
    #     }
    #     #END State Dist Update
    #
    #
    #     #      if (i==1) {
    #     #        cat(' new state_dist=',state_dist,'\n')
    #     #      }
    #
    #     #
    #     # Use Randomness to determine a state
    #     #
    #     sum=state_dist[1]
    #     for (k in 1:(m+1)) {
    #       if (runif(1) < sum | k==m+1) {
    #       #if (random[iran] < sum | k==m+1) {
    #         state_dist[1:(m+1)]=0
    #         state_dist[k]=1
    #         #iran=iran+1
    #         break
    #       #} else {
    #         #iran=iran+1
    #       }
    #       sum=sum+state_dist[k+1]
    #     }
    #
    #     # Update EWTPR for Wins
    #     sum1=sum(dist2[2:(m+1),j])
    #     for (k in 1:m) {
    #       ewtpr[i] <- ewtpr[i] + state_dist[k] * sum1 * (untimes2[j+1] - untimes2[j])
    #       sum1=sum1-dist2[k+1,j]
    #       for (l in (k+1):(m+1)) {
    #         ewtpr_components[l-1,i] <- ewtpr_components[l-1,i] + state_dist[k] * dist2[l,j] * (untimes2[j+1]-untimes2[j])
    #       }
    #     }
    #
    #     # Update EWTPR for Losses
    #     sum1=sum(dist2[1:m,j])
    #     for (k in (m+1):2) {
    #       ewtpr[i] <- ewtpr[i] - state_dist[k] * sum1 * (untimes2[j+1] - untimes2[j])
    #       ewtpr_components[k-1,i] <- ewtpr_components[k-1,i] - state_dist[k] * sum1 * (untimes2[j+1]-untimes2[j])
    #       sum1=sum1-dist2[k-1,j]
    #     }

         # if (i==13 & j>90 & j<120) {
         #   cat('After update','\n')
         #   cat('ewtpr[i]=',ewtpr[i],'\n')
         #   cat('state=',state,'\n')
         #   cat("untimes2[j]" ,untimes2[j], "\n")
         #   cat("untimes2[j+1]" ,untimes2[j+1], "\n")
         #   cat('dist2[,j]=','\n')
         #   print(dist2[,j])
         #   cat("-----------------------------------------------", "\n")
    #      # }
    #   }
    # }
    # End Redistribution-to-the-right using combined arms

  }
# END LOOP OVER SUBJECTS

  # cat('----------------------------------------------------','\n')
  # cat('ewd=','\n')
  # print(ewtpr)
  # cat('trt=','\n')
  # print(trt[1:10])

  #cat('trt=','\n')
  #print(trt)
  #cat('covariate=','\n')
  #print(cov)
  #cat('----------------------------------------------------','\n')

  # Get treatment estimate and variance for Z statistic
  fit_comp <- vector("list",m)
  if (!is.null(cov)) {
    fite=lm(ewtpr~trt+cov)
    for (k in 1:m) {
      outcome <- ewtpr_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt+cov)
    }
#    cat('----------------------------------------------------','\n')
#    cat('fit of ewtpr with trt and baseage=','\n')
#    print(fite)
#    cat('----------------------------------------------------','\n')
  }
  else {
    fite <- lm(ewtpr~trt)
    for (k in 1:m) {
      outcome <- ewtpr_components[k,]
      dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt)
    }
  }
  ewtpr_time[imp]=coef(fite)[2]
  ewtpr_time_var[imp]=vcov(fite)[2,2]

  for (k in 1:m) {
    imp_components[k,imp] <- coef(fit_comp[[k]])[2]
    imp_components_var[k,imp] <- vcov(fit_comp[[k]])[2,2]
  }

  }
  # END MULTIPLE IMPUTATION LOOP
  ewtpr_est=mean(ewtpr_time)
  ewtpr_var_est=mean(ewtpr_time_var)+((nimp+1)/nimp)*var(ewtpr_time)
  z_ewtpr <- ewtpr_est/sqrt(ewtpr_var_est)

  #cat('----------------------------------------------------','\n')
  #cat('imp_components=','\n')
  #print(imp_components)
  #cat('----------------------------------------------------','\n')

  for (k in 1:m) {
    components[k] <- mean(imp_components[k,])
    components_var[k] <- mean(imp_components_var[k,])+((nimp+1)/nimp)*var(imp_components[k,])
  }

  #cat('----------------------------------------------------','\n')
  #cat('components=','\n')
  #print(components)
  #cat('----------------------------------------------------','\n')


  return(list(ewtpr_est,ewtpr_var_est,z_ewtpr,components,components_var,max_time))
}

#' Resample using bootstraps
#'
#' This function reruns the desired wintime package method on a given number of bootstrap samples. This resampling method is recommended
#' for all pairwise wintime methods including Win time ratio (WTR), Restricted win time ratio (RWTR), Pairwise win time (PWT), and
#' Restricted Pairwise win time (RPWT). This function is also recommended for the EWTR_composite max test (MAX).
#'
#' @param type A string value indicating the wintime package method that will run with resampling.
#' @param time_restriction The time cutoff value (days).
#' @param model A string value indicating the model used on observed data ('markov' or 'km').
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should
#' be in increasing order of clinical severity.
#' @param trt A numeric vector of treatment arm indicators (1 for treatment, 0 for control).
#' @param cov A n x p matrix of covariate values, where p is the number of covariates. Rows should represent participants and columns should
#' represent covariate values.
#' @param z_ewtr The Z-statistic of EWTR.
#' @param z_comp The Z-statistic of the composite event approach.
#' @param resample_num The number of desired bootstraps.
#' @param seed The seed used for random number generation.
#' @param nimp The number of random imputations for Redistribution-to-the-right.
#' @return A list of a vector of length resample_num containing the calculated treatment effect estimates (for type='max' these are z-statistics) for each bootstrap,
#' a m x resample_num matrix of the components of the treatment effect.

# ---------------------------
# Bootstrap
# ---------------------------
bootstrap <- function(type,time_restriction,model,n,m,Time,Delta,trt,cov,z_ewtr,z_comp,resample_num,seed,nimp) {
  # Initialize vector to hold bootstrap values
  y <- rep(0,times = resample_num)
  components <- matrix(0,nrow=m,ncol=resample_num)


  time <- Time[m:1, ]
  delta <- Delta[m:1, ]
  iboot <- 1
  while (iboot <= resample_num) {
    # Generate bootstrap samples
    bootstrap_indices <- sample(1:n, size = n, replace = TRUE)

    trt_boot <- trt[bootstrap_indices]
    Time_boot <- Time[ ,bootstrap_indices,drop = FALSE]
    Delta_boot <- Delta[ ,bootstrap_indices,drop = FALSE]
    time_boot <- time[ ,bootstrap_indices,drop = FALSE]
    delta_boot <- delta[ ,bootstrap_indices,drop = FALSE]
#    cat("dim(cov)=","\n")
#    print(dim(cov))
    cov_boot <- cov[bootstrap_indices, ,drop = FALSE]
    # cat("cov_boot =", "\n")
    # print(cov_boot)

    indices <- order(trt_boot)

    # Reorder bootstrap samples (control before treatment)
    trt_boot <- trt_boot[indices]
    Time_boot <- Time_boot[ ,indices]
    Delta_boot <- Delta_boot[ ,indices]
    time_boot <- time_boot[ ,indices]
    delta_boot <- delta_boot[ ,indices]
    # cat("dim cov_boot =", dim(cov_boot), "\n")
    cov_boot <- cov_boot[indices, ]

    # Set parameters for model fitting calls
    n0 <- sum(trt_boot == 0)
    n1 <- sum(trt_boot == 1)
    m <- nrow(Time_boot)
    tau <- max(Time_boot[m, ])

    # Type function calls on bootstrap data
    z <- NULL
    markov_ind <- FALSE

    if (!is.null(model)) {
      # If a Markov model is specified, call the function to fit a Markov model
      if (model == "markov") {
        z <- markov(n0,n1,m,Time_boot,Delta_boot)
        markov_ind <- TRUE
      }
      # If no model is specified, use the default (KM) model
      else {
        z <- km(n0,n1,m,Time_boot,Delta_boot)
      }
    }

    if (!is.null(z)) {
      # Parameters for type function calls
      dist_state0_boot <- z[[1]]
      dist_state1_boot <- z[[2]]
      untimes0_boot <- z[[3]]
      untimes1_boot <- z[[4]]
      nuntimes0_boot <- z[[5]]
      nuntimes1_boot <- z[[6]]
      max_follow0_boot <- z[[7]]
      max_follow1_boot <- z[[8]]
      dist_state2_boot <- z[[9]]
      untimes2_boot <- z[[10]]
      nuntimes2_boot <- z[[11]]
      max_follow2_boot <- z[[12]]
      if (model == "km") {
        comkm_boot <- z[[13]]
        trtkm_boot <- z[[14]]
        conkm_boot <- z[[15]]
        trans_prob2_boot <- array(data=0,dim=c(m,m,nuntimes2_boot))
        trans_prob1_boot <- array(data=0,dim=c(m,m,nuntimes1_boot))
        trans_prob0_boot <- array(data=0,dim=c(m,m,nuntimes0_boot))
      }
      if (model == "markov") {
        trans_prob2_boot <- z[[13]]
        trans_prob1_boot <- z[[14]]
        trans_prob0_boot <- z[[15]]
        comkm_boot <- array(data=0,dim=c(m,nuntimes2_boot))
        trtkm_boot <- array(data=0,dim=c(m,nuntimes1_boot))
        conkm_boot <- array(data=0,dim=c(m,nuntimes0_boot))
      }
    }

    # Type function calls
    if (type == "ewt") {
      temp <- EWT(m,dist_state0_boot,dist_state1_boot,untimes0_boot,untimes1_boot,nuntimes0_boot,nuntimes1_boot)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[2]]
    }
    else if (type == "ewtr") {
      temp <- EWTR(n,m,nuntimes0_boot,max_follow0_boot,untimes0_boot,Time_boot,Delta_boot,dist_state0_boot,markov_ind,cov_boot,trt_boot)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[4]]
    }
    else if (type == "ewtp") {
      temp <- EWTP(n,m,nuntimes2_boot,max_follow2_boot,untimes2_boot,Time_boot,Delta_boot,dist_state2_boot,markov_ind,cov_boot,trt_boot)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[4]]
#      cat('resampled components for iboot=',iboot,'\n')
#      print(components[,iboot])
    }
    else if (type == "rewtp") {
      temp <- REWTP(n,m,nuntimes2_boot,max_follow2_boot,untimes2_boot,Time_boot,Delta_boot,dist_state2_boot,markov_ind,cov_boot,trt_boot,time_restriction)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[4]]
      #      cat('resampled components for iboot=',iboot,'\n')
      #      print(components[,iboot])
    }
    else if (type == "ewtpr") {
      temp <- EWTPR(n,m,nuntimes2_boot,max_follow2_boot,untimes2_boot,Time_boot,Delta_boot,dist_state2_boot,markov_ind,cov_boot,trt_boot,comkm_boot,trans_prob2_boot,nuntimes1_boot,max_follow1_boot,untimes1_boot,dist_state1_boot,trtkm_boot,trans_prob1_boot,nuntimes0_boot,max_follow0_boot,untimes0_boot,dist_state0_boot,conkm_boot,trans_prob0_boot,nimp)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[4]]
      #      cat('resampled components for iboot=',iboot,'\n')
      #      print(components[,iboot])
    }
    else if (type == "rewtpr") {
      temp <- REWTPR(n,m,nuntimes2_boot,max_follow2_boot,untimes2_boot,Time_boot,Delta_boot,dist_state2_boot,markov_ind,cov_boot,trt_boot,comkm_boot,trans_prob2_boot,time_restriction,nuntimes1_boot,max_follow1_boot,untimes1_boot,dist_state1_boot,trtkm_boot,trans_prob1_boot,nuntimes0_boot,max_follow0_boot,untimes0_boot,dist_state0_boot,conkm_boot,trans_prob0_boot,nimp)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[4]]
      #      cat('resampled components for iboot=',iboot,'\n')
      #      print(components[,iboot])
    }
    else if (type == "rmt") {
      temp <- RMT(m,time_restriction,dist_state0_boot,dist_state1_boot,untimes0_boot,untimes1_boot,nuntimes0_boot,nuntimes1_boot)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[2]]
    }
    else if (type == "wtr") {
      y[iboot] <- WTR(n,m,tau,trt_boot,Time_boot,Delta_boot)[[1]]
      # cat("wtr =", "\n")
      # print(y[iboot])
    }
    else if (type == "rwtr") {
      y[iboot] <- RWTR(n,m,tau,trt_boot,Time_boot,Delta_boot)[[1]]
    }
    else if (type == "pwt") {
      temp <- PWT(n,n0,n1,m,Time_boot,Delta_boot,trt_boot,tau)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[2]]
    }
    else if (type == "rpwt") {
      temp <- RPWT(n,n0,n1,m,Time_boot,Delta_boot,trt_boot,tau,time_restriction)
      y[iboot] <- temp[[1]]
      components[,iboot] <- temp[[2]]
    }
    else if (type == "max") {
      # Observed Z statistic
      # ewtr_time <- EWTR(n,m,nuntimes0,max_follow0,untimes0,Time,Delta,dist_state0,markov_ind,cov)[[1]]
      # ewtr_time_var <- EWTR(n,m,nuntimes0,max_follow0,untimes0,Time,Delta,dist_state0,markov_ind,cov)[[2]]
      # z_ewtr <- ewtr_time/sqrt(ewtr_time_var)

      # Bootstrap Z statistic
      # ewtr_time_boot <- EWTR(n,m,nuntimes0_boot,max_follow0_boot,untimes0_boot,Time_boot,Delta_boot,dist_state0_boot,markov_ind,cov_boot)[[1]]
      # ewtr_time_var_boot <- EWTR(n,m,nuntimes0_boot,max_follow0_boot,untimes0_boot,Time_boot,Delta_boot,dist_state0_boot,markov_ind,cov_boot)[[2]]
      z_ewtr_boot <- EWTR(n,m,nuntimes0_boot,max_follow0_boot,untimes0_boot,Time_boot,Delta_boot,dist_state0_boot,markov_ind,cov_boot,trt_boot)[[3]]
      z_comp_boot <- COMP(n,Time_boot,Delta_boot,cov_boot,trt_boot)[[1]]
      y[iboot] <- max((z_ewtr_boot - z_ewtr),(z_comp_boot - z_comp))
    }
    else {
      stop(paste("Invalid type:", type, "- Please specify one of 'ewt', 'ewtr', 'rmt', 'wtr', 'rwtr', 'pwt', 'ewtp', 'rewtp', 'ewtpr', 'rewtpr' ,'rpwt', 'max'"))
    }
    iboot <- iboot + 1
  }
#  cat('resampled components=','\n')
#  print(components)

  return(list(y,components))
}

#' Restricted win time ratio
#'
#' This function calculates the ratio of losses to wins on treatment. It iterates through all pairs of treatment and control patients and uses
#' their time-to-death (or most severe clinical event) to determine a win or loss. If death is inconclusive, then a winner is determined based
#' on wintime.
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param tau The maximum follow up time (days).
#' @param tg A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).
#' @param Time A m x n matrix of event times (days), where m is the number of events in the hierarchy, and n is the total number of trial participants.
#' Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators, where m is the number of events in the hierarchy, and n is the total number of trial participants.
#' Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.
#' @return A list containing: The ratio of losses to wins on treatment, the total number of wins, and the total number of losses.

# ---------------------------------------------------
# Restricted win time ratio
# ---------------------------------------------------
RWTR <- function(n,m,tau,tg,Time,Delta) {
  rwin <- matrix(0,nrow=n,ncol=n)
  rloss <- matrix(0,nrow=n,ncol=n)

  rwintot <- 0
  rlosstot <- 0

  for (i in 1:n) {
    for (j in 1:n) {
      if (tg[i] == 1 && tg[j] == 0) {
        # If event m favors the treatment person
        if ((Delta[m,j] == 1 && Time[m,i] > Time[m,j]) ||
            (Delta[m,j] == 1 && Time[m,i] == Time[m,j] && Delta[m,i] == 0)) {
          # Win for the treatment person
          rwin[i, j] <- 1
          rwintot <- rwintot + 1
        }

        # If event m favors the control person
        else if ((Delta[m,i] == 1 && Time[m,j] > Time[m,i]) ||
                 (Delta[m,i] == 1 && Time[m,j] == Time[m,i] && Delta[m,j] == 0)) {
          # Loss for the treatment person
          rloss[i, j] <- 1
          rlosstot <- rlosstot + 1
        }
        else {
          # Pick a winner based on win time

          # Set max follow-up time for the control person to their event m time
          follow0 <- Time[m,j]
          # If event m is observed in person j
          if (Delta[m,j] == 1)  {
            follow0 <- tau
          }
          # Set max follow-up time for the treatment person to their event m time
          follow1 <- Time[m,i]

          # If event m is observed in person i
          if (Delta[m,i] == 1)  {
            follow1 <- tau
          }
          follow <- min(follow0,follow1)

          # Temp variables for times and deltas
          time0 <- Time[1:m,j]
          time1 <- Time[1:m,i]
          delta0 <- Delta[1:m,j]
          delta1 <- Delta[1:m,i]

          order <- setEventTimes(m,delta0,delta1,time0,time1,follow)
          temp <- getWintimeIntegral(m,order,time0,time1,delta0,delta1)
          integral <- temp[[1]]

          if (integral > 0)
          {
            rwin[i, j] <- 1
            rwintot <- rwintot + 1
          }
          if(integral < 0)
          {
            rloss[i, j] <- 1
            rlosstot <- rlosstot + 1
          }
        }
      }
    }
  }
  return(list(rlosstot/rwintot,rwintot,rlosstot))
}

#' Time Restricted Expected win time against trial population With redistribution to the right
#'
#' Calculates the combined arm state space probabilities using a Markov model or a Kaplan-Meier model (recommended).
#' This function uses these probabilities to compare each participant's clinical state to a distribution of combined arm states.
#' Calculation is extended by redistribution-to-the-right principles and truncated at the user-specified time_restriction (days).
#'
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param nunique2 The number of unique combined arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow2 The max combined arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes2 A vector containing unique combined arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators Rows should represent events and columns should represent participants. Rows should be
#' in increasing order of clinical severity.
#' @param dist2 A matrix of combined arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param markov_ind An indicator of the model type used (1 for Markov, 0 for Kaplan-Meier).
#' @param cov A n x p matrix of covariate values, where p is the number of covariates.
#' @param trt A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).
#' @param comkm A m x nunique2 matrix of combined arm survival probabilities (returned from wintime::markov() or wintime::km()).
#' @param trans_prob2 A (m x m x number of combined arm event times)
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th combined arm event time. (returned from wintime::markov() or wintime::km()).
#' @param time_restriction The time restriction (days) for calculation.
#' @param nunique1 The number of unique trt arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow1 The max trt arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes1 A vector containing unique trt arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param dist1 A matrix of trt arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param trtkm A m x nunique1 matrix of trt arm survival probabilities (returned from wintime::markov() or wintime::km()).
#' @param trans_prob1 A (m x m x number of trt arm event times)
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th trt arm event time. (returned from wintime::markov() or wintime::km()).
#' @param nunique0 The number of unique control arm event times (returned from wintime::markov() or wintime::km()).
#' @param maxfollow0 The max control arm follow up time (days) (returned from wintime::markov() or wintime::km()).
#' @param untimes0 A vector containing unique control arm event times (days) (returned from wintime::markov() or wintime::km()).
#' @param dist0 A matrix of control arm state probabilities (returned from wintime::markov() or wintime::km()).
#' @param conkm A m x nunique0 matrix of control arm survival probabilities (returned from wintime::markov() or wintime::km()).
#' @param trans_prob0 A (m x m x number of control arm event times)
#' matrix where (i,j,k)'th value is transition probability from state i to state j at k'th control arm event time. (returned from wintime::markov() or wintime::km()).
#'@param nimp The number of random imputations.
#' @return A list containing: The estimated treatment effect from the linear regression model, the variance, the Z-statistic, the components of the treatment effect, the variance of the components, and the maximum time used in comparisons.

# -----------------------------------------------------------------------------
# Expected win time against trial population With Redistribution to the Right
# -----------------------------------------------------------------------------
REWTPR <- function(n,m,nunique2,maxfollow2,untimes2,Time,Delta,dist2,markov_ind,cov,trt,comkm,trans_prob2,time_restriction,nunique1,maxfollow1,untimes1,dist1,trtkm,trans_prob1,nunique0,maxfollow0,untimes0,dist0,conkm,trans_prob0,nimp) {
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]
  # trans_prob2[trans_prob2==-1] <- 0
  # trans_prob1[trans_prob1==-1] <- 0
  # trans_prob0[trans_prob0==-1] <- 0
  components <- rep(NA,m)
  components_var <- rep(NA,m)
  imp_components <- matrix(NA,nrow=m,ncol=nimp)
  imp_components_var <- matrix(NA,nrow=m,ncol=nimp)
  max_time <- NA

  #
  # Get unique times in untimes0,untimes1,untimes2
  #
  untimes=rep(0,nunique0+nunique1+nunique2)
  temp=unique(c(untimes0,untimes1,untimes2))
  temp=temp[temp!=0]
  #  cat("temp=", "\n")
  #  print(temp)
  untimes[1:length(temp)]=temp
  nunique=length(temp)
  untimes=untimes[1:nunique]
  untimes=sort(untimes)

  #
  # Get conkn,trtkm on times from untimes
  #
  new_conkm=rep(0,nunique*m)
  dim(new_conkm)=c(m,nunique)
  new_trtkm=rep(0,nunique*m)
  dim(new_trtkm)=c(m,nunique)
  con_count=1
  trt_count=1

  for (i in 1:nunique) {
    #    cat("-----------------------------------------------", "\n")
    #    cat("i=",i,"\n")
    #    cat("trt_count=",trt_count,"\n")
    #    cat("untimes[i]=",untimes[i],"\n")
    #    cat("untimes1[trt_count]=",untimes1[trt_count],"\n")
    #    cat("-----------------------------------------------", "\n")
    if (con_count < nunique0) {
      if (untimes[i]==untimes0[con_count]) {
        new_conkm[1:m,i]=conkm[1:m,con_count]
        con_count=con_count+1
      } else {
        if (i==1) {
          new_conkm[1:m,i]=1
        } else {
          new_conkm[1:m,i]=new_conkm[1:m,i-1]
        }
      }
    }
    if (trt_count < nunique1) {
      if (untimes[i]==untimes1[trt_count]) {
        new_trtkm[1:m,i]=trtkm[1:m,trt_count]
        trt_count=trt_count+1
      } else {
        if (i==1) {
          new_trtkm[1:m,i]=1
        } else {
          new_trtkm[1:m,i]=new_trtkm[1:m,i-1]
        }
      }
    }
  }
  # cat("-----------------------------------------------", "\n")
  # cat("new conkm and trtkm finished", "\n")
  # cat("-----------------------------------------------", "\n")

  #
  # Get dist0,dist1,dist2 on times from untimes
  #
  new_dist0=rep(0,nunique*(m+1))
  dim(new_dist0)=c(m+1,nunique)
  new_dist1=rep(0,nunique*(m+1))
  dim(new_dist1)=c(m+1,nunique)
  new_dist2=rep(0,nunique*(m+1))
  dim(new_dist2)=c(m+1,nunique)
  con_count=1
  trt_count=1
  com_count=1

  for (j in 1:nunique) {
    if (con_count <= nunique0) {
      if (untimes[j]==untimes0[con_count]) {
        new_dist0[1:(m+1),j]=dist0[1:(m+1),con_count]
        con_count=con_count+1
      } else {
        if (j==1) {
          new_dist0[1,j]=1
          new_dist0[2:(m+1),j]=0
        } else {
          new_dist0[1:(m+1),j]=new_dist0[1:(m+1),j-1]
        }
      }
    } else {
      new_dist0[1:(m+1),j]=new_dist0[1:(m+1),j-1]
    }
    if (trt_count <= nunique1) {
      if (untimes[j]==untimes1[trt_count]) {
        new_dist1[1:(m+1),j]=dist1[1:(m+1),trt_count]
        trt_count=trt_count+1
      } else {
        if (j==1) {
          new_dist1[1,j]=1
          new_dist1[2:(m+1),j]=0
        } else {
          new_dist1[1:(m+1),j]=new_dist1[1:(m+1),j-1]
        }
      }
    } else {
      new_dist1[1:(m+1),j]=new_dist1[1:(m+1),j-1]
    }
    if (com_count <= nunique2) {
      if (untimes[j]==untimes2[com_count]) {
        new_dist2[1:(m+1),j]=dist2[1:(m+1),com_count]
        com_count=com_count+1
      } else {
        if (j==1) {
          new_dist2[1,j]=1
          new_dist2[2:(m+1),j]=0
        } else {
          new_dist2[1:(m+1),j]=new_dist2[1:(m+1),j-1]
        }
      }
    } else {
      new_dist2[1:(m+1),j]=new_dist2[1:(m+1),j-1]
    }
  }

  # cat("-----------------------------------------------", "\n")
  # cat("new dist0 and dist1 finished", "\n")
  # cat("-----------------------------------------------", "\n")

  #
  # Get trans_prob0,trans_prob1 on times from untimes
  #
  new_trans_prob0=rep(0,nunique*m*m)
  dim(new_trans_prob0)=c(m,m,nunique)
  new_trans_prob1=rep(0,nunique*m*m)
  dim(new_trans_prob1)=c(m,m,nunique)
  con_count=1
  trt_count=1

  for (i in 1:nunique) {
    if (con_count < nunique0) {
      if (untimes[i]==untimes0[con_count]) {
        new_trans_prob0[1:m,1:m,i]=trans_prob0[1:m,1:m,con_count]
        con_count=con_count+1
      } else {
        new_trans_prob0[1:m,1:m,i]=0
      }
    }
    if (trt_count < nunique1) {
      if (untimes[i]==untimes1[trt_count]) {
        new_trans_prob1[1:m,1:m,i]=trans_prob1[1:m,1:m,trt_count]
        trt_count=trt_count+1
      } else {
        new_trans_prob1[1:m,1:m,i]=0
      }
    }
  }

  nunique0=length(untimes[untimes <= maxfollow0])
  nunique1=length(untimes[untimes <= maxfollow1])

  # cat("-----------------------------------------------", "\n")
  # cat("maxfollow2 =", maxfollow2, "\n")
  # cat("maxfollow0 =", maxfollow0, "\n")
  # cat("maxfollow1 =", maxfollow1, "\n")
  # cat("Combined unique times =", "\n")
  # print(untimes2)
  # cat("-----------------------------------------------", "\n")
  # cat("Control unique times =", "\n")
  # print(untimes0)
  # cat("-----------------------------------------------", "\n")
  #
  #
  # cat("-----------------------------------------------", "\n")
  # cat("Largest # of times combined arm for redist-to-the-right =", nunique2, "\n")
  # cat("Largest time combined arm for redist-to-the-right =", untimes2[nunique2], "\n")
  # cat("Largest # of times for same arm for redist-to-the-right con arm =", nunique0, "\n")
  # cat("Largest time for same arm for redist-to-the-right con arm =", untimes2[nunique0], "\n")
  # cat("Largest # of times for same arm for redist-to-the-right trt arm =", nunique1, "\n")
  # cat("Largest time for same arm for redist-to-the-right trt arm =", untimes2[nunique1], "\n")
  # cat("time_restriction =", time_restriction, "\n")
  # cat("-----------------------------------------------", "\n")

  #-------------------------------------------------
  # Set jsamemark
  #-------------------------------------------------
  if (new_dist0[m+1,nunique0]==1) {
    if (new_dist1[m+1,nunique1]==1) {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=max(max(maxfollow0,maxfollow1),maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=maxfollow2
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    } else {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=maxfollow1
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=min(maxfollow1,maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    }
  } else {
    if (new_dist1[m+1,nunique1]==1) {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=maxfollow0
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=min(maxfollow0,maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    } else {
      if (new_dist2[m+1,nunique2]==1) {
        max_time=min(maxfollow0,maxfollow1)
        jsamemark=length(untimes[untimes <= max_time])-1
      } else {
        max_time=min(min(maxfollow0,maxfollow1),maxfollow2)
        jsamemark=length(untimes[untimes <= max_time])-1
      }
    }
  }
#--------------------------------------------------------
# FOR COMPARISON WITH FORTRAN
#  set.seed(99803332)
#  random=runif(n*nunique*nimp)
#  iran=1
#--------------------------------------------------------
#--------------------------------------------------------
# cat("random numbers =",random[1:10], "\n")
# cat("-----------------------------------------------", "\n")

  rewtpr_time=rep(0,nimp)
  rewtpr_time_var=rep(0,nimp)

  # Set jfinalmark
  #jfinalmark=nunique-1


  # START MULTIPLE IMPUTATION LOOP
  for (imp in 1:nimp) {
     # cat("-----------------------------------------------", "\n")
     # cat("imp=" ,imp, "\n")
     # cat('iran=',iran,'\n')
     # cat('next 10 random=',random[iran:(iran+9)],'\n')


    # Initialize temporary variables
    rewtpr <- rep(0,n)
    rewtpr_components <- matrix(0,nrow=m,ncol=n)


  # START LOOP OVER SUBJECTS
  for (i in 1:n) {

#if (i<=13) {
  # cat("-----------------------------------------------", "\n")
  # cat("-----------------------------------------------", "\n")
  # cat("i=" ,i, "\n")
  # cat('iran=',iran,'\n')
  # cat("time[,i]" ,time[,i], "\n")
  # cat("delta[,i]" ,delta[,i], "\n")
  # cat("trans_prob[1,1,]" ,trans_prob[1,1,], "\n")
  # cat("trans_prob[1,2,]" ,trans_prob[1,2,], "\n")
  # cat("trans_prob[1,3,]" ,trans_prob[1,3,], "\n")
  # cat("trans_prob[2,2,]" ,trans_prob[2,2,], "\n")
  # cat("trans_prob[2,3,]" ,trans_prob[2,3,], "\n")
  # cat("trans_prob[3,3,]" ,trans_prob[3,3,], "\n")
  #cat("-----------------------------------------------", "\n")
#}

    # Set jmax
    jmax <- 0
    for (j in 1:nunique) {
      if (untimes[j] < time[1,i]) {
        jmax <- jmax + 1
      }
    }
    #if (markov_ind == FALSE) {
    #  jmax <- min(maxfollow2,jmax)
    #}
    if (delta[1,i] == 1 | jmax >= nunique) {
      jmax <- nunique - 1
    }

    #if (trt[i]==1) {jsamemark=nunique1-1}
    #if (trt[i]==0) {jsamemark=nunique0-1}
    #jsamemark=min(min(nunique0,nunique1),nunique2)-1
    #if (jsamemark == nunique2) {jsamemark=nunique2-1}

    # Enforce comparison ends at jsamemark
    jmax=min(jmax,jsamemark)

    #if (jsamemark < jmax) {jsamemark=jmax}

    # if (i==79) {
    #   cat('i=',i,'\n')
    #   cat('jmax=',jmax,'\n')
    #   cat('jsamemark=',jsamemark,'\n')
    # cat('jfinalmark=',jfinalmark,'\n')
    #  cat('time[,i]=',time[,i],'\n')
    #  cat('delta[,i]=',delta[,i],'\n')
    #}

    state=0
    if (jmax != 0) {
# Comparison of known state to combined arm dist
      for (j in 1:jmax) {
        if (untimes[j]>=time_restriction) {break}
        if (untimes[j+1]>time_restriction) {
          time_inc=time_restriction-untimes[j]
        } else {
          time_inc=untimes[j+1]-untimes[j]
        }
        state <- 0
        for (current_state in m:1) {
          temp_time_index <- m - current_state + 1
          if (time[temp_time_index,i] <= untimes[j] && delta[temp_time_index,i] == 1) {
            state <- current_state
            break
          }
        }

      # if (i==13) {
      #    cat("-----------------------------------------------", "\n")
      #    cat('j=',j,'\n')
      #    cat('state=',state,'\n')
      #    cat("untimes2[j]" ,untimes2[j], "\n")
      #    cat("untimes2[j+1]" ,untimes2[j+1], "\n")
      #    cat('dist2[,j]=','\n')
      #    print(dist2[,j])
      #    cat("-----------------------------------------------", "\n")
      #  }

        # Calculate rewtpr

        # Calculate wins
        for (state_num in 0:(m-1)) {
          if (state_num == state) {
            # Add probabilities from higher states
            for (k in (state_num+1):m) {
              rewtpr[i] <- rewtpr[i] + new_dist2[k+1,j] * (time_inc)
              rewtpr_components[k,i] <- rewtpr_components[k,i] + new_dist2[k+1,j] * (time_inc)
            }
            break
          }
        }

        # Calculate Losses
        for (current in 1:m) {
          if (current == state) {
            # Subtract probabilities from lower states
            for (k in 1:current) {
              rewtpr[i] <- rewtpr[i] - new_dist2[k,j] * (time_inc)
              rewtpr_components[current,i] <- rewtpr_components[current,i] - new_dist2[k,j] * (time_inc)
            }
            break
          }
        }
        # if (i==13) {
        #   cat('rewtpr[13]=',rewtpr[13],'\n')
        # }
      }
    }

# Initialize dist_state
    state_dist=rep(0,m+1)
    for (k in 0:m) {
      if (state==k) {state_dist[k+1]=1}
    }
    new_state_dist=state_dist

    #if (i==79) {
    #  cat('After jmax rewtpr=',rewtpr[i],'\n')
    #  cat('state=',state,'\n')
    #  cat('state_dist=',state_dist,'\n')
    #}
    #-------------------------------------------------------------
    # Start Redistribution-to-the-right using same arm
    #-------------------------------------------------------------

    if (jmax < jsamemark) {
    for (j in (jmax+1):jsamemark) {


      if (untimes[j]>=time_restriction) {break}
      if (untimes[j+1]>time_restriction) {
        time_inc=time_restriction-untimes[j]
      } else {
        time_inc=untimes[j+1]-untimes[j]
      }

      #if (i==79) {
        # cat("-----------------------------------------------", "\n")
        # cat('j> max=',j,'\n')
        # cat('state_dist=',state_dist,'\n')
        # cat("untimes2[j]" ,untimes2[j], "\n")
        # cat("untimes2[j+1]" ,untimes2[j+1], "\n")
     #   cat("trans_prob0[1,1,j]" ,new_trans_prob0[1,1,j], "\n")
     #   cat("trans_prob0[1,2,j]" ,new_trans_prob0[1,2,j], "\n")
     #  cat("-----------------------------------------------", "\n")
     # }

      #
      # CHECK FOR NO EXTENDED COMPARISON
      #
      if (!(trt[i]==0 & new_dist0[m+1,j]==1) & !(trt[i]==1 & new_dist1[m+1,j]==1)) {


      # Update state_dist
      if (markov_ind == 0) {
      # KM Model
        if (trt[i]==0) {
          # RTTR by con arm

          sum=state_dist[1]
          # if (i==20 & j==37) {
          #   cat('sum=',sum,'\n')
          # }

          if (j !=1) {
            for (k in 1:m) {
              if (new_conkm[k,j-1] != 0) {
                new_state_dist[k]=sum*new_conkm[k,j]/new_conkm[k,j-1]
                sum=sum+state_dist[k]
              } else {
                new_state_dist[k]=sum*new_conkm[k,j]
                sum=sum+state_dist[k]
              }
            }
          } else {
            for (k in 1:m) {
              new_state_dist[k]=sum*new_conkm[k,j]
              sum=sum+state_dist[k]
            }
          }
          # Enforce monitonicity
          for (k in 2:m) {
            if (new_state_dist[k] < new_state_dist[k-1]) {new_state_dist[k]=new_state_dist[k-1]}
          }
          for (k in 1:(m+1)) {
            if (k==m+1) {
              state_dist[k]=1-new_state_dist[m]
            } else {
              if (k==1) {
                state_dist[k]=new_state_dist[1]
              } else {
                state_dist[k]=new_state_dist[k]-new_state_dist[k-1]
              }
            }
          }
        } else {
          # RTTR by trt arm

          sum=state_dist[1]
          # if (i==20 & j==37) {
          #   cat('sum=',sum,'\n')
          # }

          if (j !=1) {
            for (k in 1:m) {
              if (new_trtkm[k,j-1] != 0) {
                new_state_dist[k]=sum*new_trtkm[k,j]/new_trtkm[k,j-1]
                sum=sum+state_dist[k]
              } else {
                new_state_dist[k]=sum*new_trtkm[k,j]
                sum=sum+state_dist[k]
              }
            }
          } else {
            for (k in 1:m) {
              new_state_dist[k]=sum*new_trtkm[k,j]
              sum=sum+state_dist[k]
            }
          }
          # Enforce monitonicity
          for (k in 2:m) {
            if (new_state_dist[k] < new_state_dist[k-1]) {new_state_dist[k]=new_state_dist[k-1]}
          }
          for (k in 1:(m+1)) {
            if (k==m+1) {
              state_dist[k]=1-new_state_dist[m]
            } else {
              if (k==1) {
                state_dist[k]=new_state_dist[1]
              } else {
                state_dist[k]=new_state_dist[k]-new_state_dist[k-1]
              }
            }
          }
        }
      } else {
        # Markov Model

        if (trt[i]==0) {
          # RTTR by con arm

          trans_out <- array(data=0,dim=c(m))
          for (l in 1:m) {
            for (k in l:m) {
              #           if (i==1) {
              #             cat('trans_out[l] calculation for l=',l,' with k=',k,'\n')
              #           }
              trans_out[l]=trans_out[l]+new_trans_prob0[l,k,j]
            }
          }
          trans_in <- array(data=0,dim=c(m,m))
          for (l in 1:m) {
            for (k in 1:l) {
              #            if (i==1) {
              #              cat('trans_in[l] calculation for l=',l,' with k=',k,'\n')
              #            }
              trans_in[k,l]=trans_in[k,l]+new_trans_prob0[k,l,j]
            }
          }

          #        if (i==1) {
          #          cat(' trans_in=',trans_in,'\n')
          #          cat(' trans_out=',trans_out,'\n')
          #        }


          for (k in 1:(m+1)) {
            if (k <= m) {
              new_state_dist[k]=state_dist[k]*(1-trans_out[k])
            } else {
              new_state_dist[k]=state_dist[k]
            }
            if (k > 1) {
              for (l in 1:(k-1)) {
                new_state_dist[k]=new_state_dist[k]+state_dist[l]*trans_in[l,k-1]
              }
            }
          }
          state_dist=new_state_dist
        } else {
          # RTTR by trt arm

          trans_out <- array(data=0,dim=c(m))
          for (l in 1:m) {
            for (k in l:m) {
              #           if (i==1) {
              #             cat('trans_out[l] calculation for l=',l,' with k=',k,'\n')
              #           }
              trans_out[l]=trans_out[l]+new_trans_prob1[l,k,j]
            }
          }
          trans_in <- array(data=0,dim=c(m,m))
          for (l in 1:m) {
            for (k in 1:l) {
              #            if (i==1) {
              #              cat('trans_in[l] calculation for l=',l,' with k=',k,'\n')
              #            }
              trans_in[k,l]=trans_in[k,l]+new_trans_prob1[k,l,j]
            }
          }

          #        if (i==1) {
          #          cat(' trans_in=',trans_in,'\n')
          #          cat(' trans_out=',trans_out,'\n')
          #        }


          for (k in 1:(m+1)) {
            if (k <= m) {
              new_state_dist[k]=state_dist[k]*(1-trans_out[k])
            } else {
              new_state_dist[k]=state_dist[k]
            }
            if (k > 1) {
              for (l in 1:(k-1)) {
                new_state_dist[k]=new_state_dist[k]+state_dist[l]*trans_in[l,k-1]
              }
            }
          }
          state_dist=new_state_dist
        }
      }
      #END State Dist Update via RTTR using Same Arm

       # if (i==79 & j==55) {
       #   cat(' Before Randomness','\n')
       #   cat(' new state_dist=',state_dist,'\n')
       # }
      # if (i==79 & j==55) {
      #   cat('iran=',iran,'\n')
      #   cat('random=',random[iran],'\n')
      # }

      #-----------------------------------------------
      # Use Randomness to determine a state
      #-----------------------------------------------

# NOT FOR COMPARISON WITH FORTRAN
      zz=runif(1)

      sum=state_dist[1]
      for (k in 1:(m+1)) {

        # if (i==79 & j==55) {
        #   cat('k=',k,'\n')
        #   cat('sum=',sum,'\n')
        #   cat('iran=',iran,'\n')
        #   cat('random=',random[iran],'\n')
        # }
        #--------------------------------------------------------
# NOT FOR COMPARISON WITH FORTRAN
        if (zz < sum | k==m+1) {
# FOR COMPARISON WITH FORTRAN
        #if (random[iran] < sum | k==m+1) {
        #  iran=iran+1
# END: FOR COMPARISON WITH FORTRAN

          state_dist[1:(m+1)]=0
          state_dist[k]=1

          break
        }
        sum=sum+state_dist[k+1]
      }

      #if (i==8) {
      # cat('j=',j,'\n')
      #  cat(' After Randomness','\n')
      # cat(' new state_dist=',state_dist,'\n')
      #  cat('iran=',iran,'\n')
      #}

      } else {
        # EXTENDED COMPARISON
        state_dist[1:(m+1)]=0
        state_dist[m+1]=1
      }
      # END OF CHECK FOR EXTENDED COMPARISON


# Update REWTPR for Wins
      sum1=sum(new_dist2[2:(m+1),j])
      for (k in 1:m) {
        rewtpr[i] <- rewtpr[i] + state_dist[k] * sum1 * (time_inc)
        sum1=sum1-new_dist2[k+1,j]
        for (l in (k+1):(m+1)) {
          rewtpr_components[l-1,i] <- rewtpr_components[l-1,i] + state_dist[k] * new_dist2[l,j] * (time_inc)
        }
      }

      # Update REWTPR for Losses
      sum1=sum(new_dist2[1:m,j])
      for (k in (m+1):2) {
        rewtpr[i] <- rewtpr[i] - state_dist[k] * sum1 * (time_inc)
        rewtpr_components[k-1,i] <- rewtpr_components[k-1,i] - state_dist[k] * sum1 * (time_inc)
        sum1=sum1-new_dist2[k-1,j]
      }

      # if (i==79) {
      #   cat('j=',j,'\n')
      #   cat('RTTR updated rewtpr[i]=',rewtpr[i],'\n')
      #   cat("-----------------------------------------------", "\n")
      # }

    }
    }
    #-------------------------------------------------------------
    # End Redistribution-to-the-right using same arm
    #-------------------------------------------------------------
    #if (i<=8) {
    #  cat('After jsamemax','\n')
    #  cat('rewtpr[i]=',rewtpr[i],'\n')
    #   cat('state_dist=',state_dist,'\n')
    #   cat('untimes2[j]' ,untimes2[j], "\n")
    #   cat('untimes2[j+1]' ,untimes2[j+1], "\n")
    #   cat('dist2[,j]=','\n')
    #   print(dist2[,j])
    #  cat("-----------------------------------------------", "\n")
    #}
    #------------------------------------------------------
    # Start Redistribution-to-the-right using combined arms
    #-------------------------------------------------------------
    #
    # if (jsamemark < jfinalmark) {
    #   for (j in (jsamemark+1):jfinalmark) {
    #
    #     if (untimes2[j]>=time_restriction) {break}
    #     if (untimes2[j+1]>time_restriction) {
    #       time_inc=time_restriction-untimes2[j]
    #     } else {
    #       time_inc=untimes2[j+1]-untimes2[j]
    #     }
    #
    #     # if (i==13) {
    #     #   cat('j> jsamemark=',j,'\n')
    #     #   cat('untimes2[j]' ,untimes2[j], "\n")
    #     #   cat('untimes2[j+1]' ,untimes2[j+1], "\n")
    #     #   cat('state_dist=',state_dist,'\n')
    #     #  #cat('markov_ind=',markov_ind,'\n')
    #     # }
    #
    #     # Update state_dist
    #     if (markov_ind == 0) {
    #
    #       # KM Model
    #
    #       sum=state_dist[1]
    #       # if (i==20 & j==37) {
    #       #   cat('sum=',sum,'\n')
    #       # }
    #
    #       if (j !=1) {
    #         for (k in 1:m) {
    #           if (comkm[k,j-1] != 0) {
    #             new_state_dist[k]=sum*comkm[k,j]/comkm[k,j-1]
    #             sum=sum+state_dist[k]
    #           } else {
    #             new_state_dist[k]=sum*comkm[k,j]
    #             sum=sum+state_dist[k]
    #           }
    #         }
    #       } else {
    #         for (k in 1:m) {
    #           new_state_dist[k]=sum*comkm[k,j]
    #           sum=sum+state_dist[k]
    #         }
    #       }
    #       # Enforce monitonicity
    #       for (k in 2:m) {
    #         if (new_state_dist[k] < new_state_dist[k-1]) {new_state_dist[k]=new_state_dist[k-1]}
    #       }
    #       for (k in 1:(m+1)) {
    #         if (k==m+1) {
    #           state_dist[k]=1-new_state_dist[m]
    #         } else {
    #           if (k==1) {
    #             state_dist[k]=new_state_dist[1]
    #           } else {
    #             state_dist[k]=new_state_dist[k]-new_state_dist[k-1]
    #           }
    #         }
    #       }
    #     } else {
    #       # Markov Model
    #
    #       trans_out <- array(data=0,dim=c(m))
    #       for (l in 1:m) {
    #         for (k in l:m) {
    #           #           if (i==1) {
    #           #             cat('trans_out[l] calculation for l=',l,' with k=',k,'\n')
    #           #           }
    #           trans_out[l]=trans_out[l]+trans_prob2[l,k,j]
    #         }
    #       }
    #       trans_in <- array(data=0,dim=c(m,m))
    #       for (l in 1:m) {
    #         for (k in 1:l) {
    #           #            if (i==1) {
    #           #              cat('trans_in[l] calculation for l=',l,' with k=',k,'\n')
    #           #            }
    #           trans_in[k,l]=trans_in[k,l]+trans_prob2[k,l,j]
    #         }
    #       }
    #
    #       #        if (i==1) {
    #       #          cat(' trans_in=',trans_in,'\n')
    #       #          cat(' trans_out=',trans_out,'\n')
    #       #        }
    #
    #
    #       for (k in 1:(m+1)) {
    #         if (k <= m) {
    #           new_state_dist[k]=state_dist[k]*(1-trans_out[k])
    #         } else {
    #           new_state_dist[k]=state_dist[k]
    #         }
    #         if (k > 1) {
    #           for (l in 1:(k-1)) {
    #             new_state_dist[k]=new_state_dist[k]+state_dist[l]*trans_in[l,k-1]
    #           }
    #         }
    #       }
    #       state_dist=new_state_dist
    #
    #     }
    #     #END State Dist Update
    #
    #     #cat('i=',i,'\n')
    #     # if (i==13) {
    #     #   cat('After update state_dist=',state_dist,'\n')
    #     #   cat('-----------------------------------------------', '\n')
    #     # }
    #
    #     #
    #     # Use Randomness to determine a state
    #     #
    #     sum=state_dist[1]
    #     for (k in 1:(m+1)) {
    #
    #       if (runif(1) < sum | k==m+1) {
    #       # FOR COMPARISON WITH FORTRAN
    #       #if (random[iran] < sum | k==m+1) {
    #
    #         state_dist[1:(m+1)]=0
    #         state_dist[k]=1
    #         #iran=iran+1
    #         break
    #       #} else {
    #       #  iran=iran+1
    #       }
    #       sum=sum+state_dist[k+1]
    #     }
    #
        # # Update EWTPR for Wins
        # sum1=sum(dist2[2:(m+1),j])
        # for (k in 1:m) {
        #   rewtpr[i] <- rewtpr[i] + state_dist[k] * sum1 * (time_inc)
        #   sum1=sum1-dist2[k+1,j]
        #   for (l in (k+1):(m+1)) {
        #     rewtpr_components[l-1,i] <- rewtpr_components[l-1,i] + state_dist[k] * dist2[l,j] * (time_inc)
        #   }
        # }
        #
        # # Update EWTPR for Losses
        # sum1=sum(dist2[1:m,j])
        # for (k in (m+1):2) {
        #   rewtpr[i] <- rewtpr[i] - state_dist[k] * sum1 * (time_inc)
        #   rewtpr_components[k-1,i] <- rewtpr_components[k-1,i] - state_dist[k] * sum1 * (time_inc)
        #   sum1=sum1-dist2[k-1,j]
        # }

         # if (i==13) {
         #   cat('ewtpr[i]=',ewtpr[i],'\n')
         #   cat('state_dist=',state_dist,'\n')
        #   cat('untimes[j]' ,untimes[j], "\n")
        #   cat('untimes[j+1]' ,untimes[j+1], "\n")
        #   cat('dist[,j]=','\n')
        #   print(dist[,j])
         #   cat("-----------------------------------------------", "\n")
         # }
    #   }
    # }
    # End Redistribution-to-the-right using combined arms

  }
# END LOOP OVER SUBJECTS

   # cat('----------------------------------------------------','\n')
   # cat('rewtpr=','\n')
   # print(rewtpr)
  # cat('rewtpr_components=','\n')
  # print(rewtpr_components)
  # cat('trt=','\n')
  # print(trt)
  # cat('covariate=','\n')
  # print(cov)
  # cat('----------------------------------------------------','\n')

  # Get treatment estimate and variance for Z statistic
  fit_comp <- vector("list",m)
  if (!is.null(cov)) {
    fite=lm(rewtpr~trt+cov)
    for (k in 1:m) {
       outcome <- rewtpr_components[k,]
       dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt+cov)
    }
    #cat('----------------------------------------------------','\n')
    #cat('fit of rewtpr with trt and baseage=','\n')
    #print(fite)
    #cat('----------------------------------------------------','\n')
  }
  else {
    fite <- lm(rewtpr~trt)
    for (k in 1:m) {
       outcome <- rewtpr_components[k,]
       dim(outcome) <- c(n)
      fit_comp[[k]] <- lm(outcome~trt)
    }
  }
  rewtpr_time[imp] <- coef(fite)[2]
  rewtpr_time_var[imp] <- vcov(fite)[2,2]
  for (k in 1:m) {
    imp_components[k,imp] <- coef(fit_comp[[k]])[2]
    imp_components_var[k,imp] <- vcov(fit_comp[[k]])[2,2]
  }

  }
  # END MULTIPLE IMPUTATION LOOP

  #cat('----------------------------------------------------','\n')
  #cat('rewtpr_time=',rewtpr_time[1:10],'\n')

  rewtpr_est=mean(rewtpr_time)
  rewtpr_var_est=mean(rewtpr_time_var)+((nimp+1)/nimp)*var(rewtpr_time)
  z_rewtpr <- rewtpr_est/sqrt(rewtpr_var_est)
  for (k in 1:m) {
    components[k] <- mean(imp_components[k,])
    components_var[k] <- mean(imp_components_var[k,])+((nimp+1)/nimp)*var(imp_components[k,])
  }
  max_time=min(max_time,time_restriction)

  return(list(rewtpr_est,rewtpr_var_est,z_rewtpr,components,components_var,max_time))
}

#' Time Restricted Pairwise win time
#'
#' This function calculates the sum of each pair's win time difference (truncated at the user-specified time_restriction (days)) divided by the total number of pairs.
#'
#' @param n The total number of trial participants.
#' @param n0 The number of control arm patients.
#' @param n1 The number of treatment arm patients.
#' @param m The number of events in the hierarchy.
#' @param Time A m x n matrix of event time (days). Rows should represent events and columns should represent participants. Event rows should
#' be in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows
#' should be in increasing order of clinical severity.
#' @param tg A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).
#' @param tau The maximum follow up time (days).
#' @param time_restriction The time restriction (days) for calculation.
#' @return The pairwise win time, and the components of the pairwise win time.


# -----------------------------------------
# Time Restricted Pairwise win time
# -----------------------------------------
RPWT <- function(n,n0,n1,m,Time,Delta,tg,tau,time_restriction) {
  total <- 0
  npairs <- 0
  count <- 0
  icount <- 0
  WTD <- rep(0,times=n0*n1)
  dim(WTD) <- c(n1,n0)
  tgroup <- c(1:n)[tg==1]
  cgroup <- c(1:n)[tg==0]
  components <- rep(0,m)
  for (i in tgroup) {
    icount <- icount+1
    jcount <- 0
    for (j in cgroup) {
      jcount <- jcount+1
#  for (i in 1:n) {
#    for (j in 1:n) {
#      if (tg[i] == 1 && tg[j] == 0) {
        count <- count+1
        # Set max follow-up time for the control person to their event m time
        follow0 <- Time[m,j]
        # If event m is observed in person j
        if (Delta[m,j] == 1)  {
          follow0 <- tau
        }
        # Set max follow-up time for the treatment person to their event m time
        follow1 <- Time[m,i]

        # If event m is observed in person i
        if (Delta[m,i] == 1)  {
          follow1 <- tau
        }
        follow <- min(follow0,follow1)

        # Temp variables for times and deltas
        time0 <- Time[1:m,j]
        time1 <- Time[1:m,i]
        delta0 <- Delta[1:m,j]
        delta1 <- Delta[1:m,i]

        # if (icount==1 & jcount==2) {
        #   cat('icount=',icount,'\n')
        #   cat('time1=',time1,'\n')
        #   cat('delta1=',delta1,'\n')
        #   cat('jcount=',jcount,'\n')
        #   cat('time0=',time0,'\n')
        #   cat('delta0=',delta0,'\n')
        # }

        order <- setEventTimes(m,delta0,delta1,time0,time1,follow)

        # if (icount==1 & jcount==2) {
        #   cat('order=',order,'\n')
        # }

        temp <- getWintimeIntegral_rest(m,order,time0,time1,delta0,delta1,time_restriction)
        WTD[icount,jcount] <- temp[[1]]
        comp <- temp[[2]]
        total <- total + WTD[icount,jcount]
        components <- components + comp
        npairs <- npairs + 1
#      }
    }
  }
#  cat('final total=',total,'\n')
#  cat('WTD[1,2]=','\n')
#  print(WTD[1,2])
#  cat('npairs=',npairs,'\n')

  return(list(total/npairs,components/npairs))
}

#' Run composite analysis
#'
#' This function fits a Cox Model to time-to-event data and calculates the z statistic. In the wintime package, this function is used for the
#' EWTR-composite max test (MAX) method.
#'
#' @param n The total number of trial participants.
#' @param Time A m x n matrix of event times (days), where m is the number of events in the hierarchy. Rows should represent events and columns
#' should represent participants. Event rows should be in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators, where m is the number of events in the hierarchy. Rows should represent events and
#' columns should represent participants. Event rows should be in increasing order of clinical severity.
#' @param cov A n x p matrix of covariate values, where p is the number of covariates. Rows should represent participants and columns
#' should represent covariate values.
#' @param trt A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).
#' @return A list containing: The z-statistic of the treatment effect from the Cox Model fit, the treatment effect estimate, the variance of the treatment effect estimate, the p-value for treatment effect.

# --------------------------------
# Composite Analysis
# --------------------------------
COMP <- function(n,Time,Delta,cov,trt) {
  TIME_COMP <- numeric(n)
  DELTA_COMP <- numeric(n)
  m=length(Time[,1])

  for (i in 1:n) {
    TIME_COMP[i] <- Time[m,i]
    TIME_COMP[i] <- min(TIME_COMP[i],Time[Delta[,i]==1,i])
#    TIME_COMP[i] <- min(Time[ ,i])
    DELTA_COMP[i] <- max(Delta[ ,i])
  }

  if (!is.null(cov)) {
    fit=survival::coxph(survival::Surv(TIME_COMP,DELTA_COMP)~trt+cov)
  }
  else {
    fit <- survival::coxph(survival::Surv(TIME_COMP,DELTA_COMP)~trt)
  }
  z_COMP <- -1*coef(fit)[1]/sqrt(vcov(fit)[1,1])
  est_COMP <- exp(coef(fit)[1])
  var_COMP=vcov(fit)[1,1]
  p_COMP <- 2*(1-pnorm(abs(z_COMP),mean=0,sd=1))
  return(list(z_COMP,est_COMP,var_COMP,p_COMP))
}

#' Run a win time calculation
#'
#' This function runs one of the win time methods on observed and resampled data.
#'
#' @param type A string value indicating the desired win time method. Methods include 'ewt', 'ewtr', 'rmt', 'max', 'wtr', 'rwtr', 'pwt', 'ewtp', 'rewtp', 'ewtpr','rewtpr', and 'rpwt'.
#' @param Time A `m x n` matrix of event times (days), where `m` is the number of events in the hierarchy, and `n` is the total number of trial participants.
#' Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.
#' @param Delta A `m x n` matrix of event indicators, where `m` is the number of events in the hierarchy, and `n` is the total number of trial participants.
#' Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.
#' @param trt A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).
#' @param cov Optional. A `n x p` matrix of covariate values, where `n` is the total number of trial participants and `p` is the number of covariates.
#' Rows should represent participants and columns should represent covariate values.
#' @param model Optional. String value. The type of model used to calculate state distributions. Options include 'km' and 'markov'.  Default depends on `type`.
#' @param resample Optional. String value. The resampling method run after the observed data calculation. Options include 'boot' and 'perm'.  Default depends on `type`.
#' @param resample_num Optional. The number of desired resamples. Default is 0.
#' @param time_restriction Required only for `type` = 'rmt','rewtp','rewtpr', and 'rpwt'. The cutoff time (days).
#' @param seed Optional. Seed used for random number generation in resampling.
#' @param nimp Required only for `type` = 'ewtpr','rewtpr'. The number of random imputations for Redistribution-to-the-right.
#' @return A list containing: the observed treatment effect, a vector of length `resample_num` containing resampled treatment effects, a message
#' indicating the method ran and the type of resampling done, the variance, the p-value (one-sided for treatment benefit), the total wins on treatment (pairwise methods only),
#' the total losses on treatment (pairwise methods only), a vector of length 'm' with the components of the treatment effect,
#' a vector of length 'm' with the variance of the components, the maximum time used in the comparisons (for EWT, RMT, EWTP, REWTP, EWTPR, REWTPR).
#' A warning message will be printed for combinations of `type` and `model`/`resample`
#' that are not recommended.
#' @details The type parameter specifies the procedure you would like to run.
#' 'ewt' is Expected Win Time.
#' 'ewtr' is Expected Win Time Against Reference (Control Arm).
#' 'rmt' is Restricted Mean Time in Favor of Treatment.
#' 'max' is the MAX procedure (max(COMP,EWTR)).
#' 'wtr' is Win Time Ratio.
#' 'rwtr' is Restricted Win Time Ratio.
#' 'pwt' is Pairwise Win Time.
#' 'ewtp' is Expected Win Time Against Trial Population.
#' 'ewtpr' is Expected Win Time Against Trial Population With Redistribution.
#' 'rewtp' is Time Restricted Expected Win Time Against Trial Population.
#' 'rewtpr'is Time Restricted Expected Win Time Against Trial Population With Redistribution.
#' 'rpwt' is Time Restricted Pairwise Win Time.
#'
#' The maximum time used in comparisons for EWT, RMT, EWTP, REWTP, EWTPR, and REWTPR are defined below.
#'
#' First define separately for each arm the maximum valid time for estimating the multi-state distribution
#' (max_valid_time0 and max_valid_time1) from either a Markov or KM model as follows.
#' max_valid_time0 is the smallest time for the control arm when the estimated multi-state distribution has a
#' positive probability of a state but there is nobody still at-risk in that state (If this never occurs
#' then max_valid_time0 is defined to be the largest event time of the control arm (max_time0)).
#' max_valid_time1 is the smallest time for the treatment arm when the estimated multi-state distribution has a
#' positive probability of a state but there is nobody still at-risk in that state (If this never occurs
#' then max_valid_time1 is defined to be the largest event time of the treatment arm(max_time1)).
#' Also define the maximum valid time for the combined arms for estimating the multi-state distribution
#' (max_valid_time2) from either a Markov or KM model as follows.
#' max_valid_time2 is the smallest time for the combined arms when the estimated multi-state distribution has a
#' positive probability of a state but there is nobody still at-risk in that state (If this never occurs
#' then max_valid_time2 is defined to be the largest event time of the combined arms (max_time2)).
#'
#' EWT: There are two cases:
#' (1) the arm with the smaller maximum valid time has an estimated multi-state distribution that is degenerate
#' in the absorbing (highest) state at that time.
#' (2) the arm with the smaller maximum valid time has an estimated multi-state distribution that is not degenerate
#' in the absorbing (highest) state at that time.
#' For case (1) the maximum time used in comparison is the max(max_valid_time0,max_valid_time1).
#' For case (2) the maximum time used in comparison is the min(max_valid_time0,max_valid_time1).
#'
#' RMT: the maximum time used in comparison is min(the maximum time used in comparison for EWT, time_restriction).
#'
#' EWTP: There are two cases:
#' (1) the combined arms have an estimated multi-state distribution that is degenerate in the absorbing (highest) state
#' at max_valid_time2.
#' (2) the combined arms have an estimated multi-state distribution that is not degenerate in the absorbing (highest) state
#' at max_valid_time2.
#' For case (1) the maximum time used in comparison is the max(max_valid_time2,max_time2).
#' For case (2) the maximum time used in comparison is the min(max_valid_time2,max_time2).
#'
#' REWTP: the maximum time used in comparison is min(the maximum time used in comparison for EWTP, time_restriction).
#'
#' EWTPR: There are 8 cases based on which of the three (control arm, treatment arm, combined arms) estimated multi-state
#' distributions are degenerate in the absorbing (highest) state at the maximum valid time.
#' The maximum time used in comparison is the minimum of the max_valid_times over the arms (control, treatment, combined)
#' that do not have degenerate estimated multi-state distibutions in the absorbing (highest) state at the maximum valid time
#' (If all have a degenerate estimated multi-state distibutions in the absorbing (highest) state at the maximum valid time
#' the maximum time used in comparison is defined as the maximum of the three max_valid_times).
#'
#'REWTPR: the maximum time used in comparison is min(the maximum time used in comparison for EWTPR, time_restriction).
#'
#' @examples
#' # ------------------------------
#' # Example Inputs
#' # ------------------------------
#'
#' # Event time vectors
#' TIME_1 <- c(256,44,29,186,29,80,11,380,102,33)
#' TIME_2 <- c(128,44,95,186,69,66,153,380,117,33)
#' TIME_3 <- c(435,44,95,186,69,270,1063,380,117,33)
#'
#' # Event time matrix
#' Time <- rbind(TIME_1, TIME_2, TIME_3)
#'
#' # Event indicator vectors
#' DELTA_1 <- c(1,0,1,0,1,1,1,0,1,0)
#' DELTA_2 <- c(1,0,0,0,0,1,1,0,0,0)
#' DELTA_3 <- c(0,0,0,0,0,0,0,0,0,0)
#'
#' # Event indicator matrix
#' Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)
#'
#' # Treatment arm indicator vector
#' trt <- c(1,1,1,1,1,0,0,0,0,0)
#'
#' # Covariate vectors
#' cov1 <- c(54,53,55,61,73,65,63,63,82,58,66,66)
#' cov2 <- c(34.4,32.1,34.7,54.1,55.7,43.6,32.1,44.8,85.2,12.5,33.4,21.4)
#'
#' # Covariate vectors
#' cov1 <- c(66,67,54,68,77,65,55,66,77,54)
#' cov2 <- c(3,6,4,2,3,5,8,5,3,5)
#' cov3 <- c(34.6,543.6,45.8,54.7,44.3,55.6,65.9,54.7,77.9,31.2)
#'
#' # Covariate matrix
#' cov <- cbind(cov1, cov2, cov3)
#'
#' # ------------------------
#' # wintime Examples
#' # ------------------------
#'
#' # Run WTR
#' z <- wintime("wtr", Time, Delta, trt)
#' print(z)
#'
#' # Run EWT with default settings and 10 resamples
#' z <- wintime("ewt", Time, Delta, trt, resample_num = 10)
#' print(z)
#'
#' # Run EWTR with default settings
#' z <- wintime("ewtr", Time, Delta, trt, cov = cov)
#' print(z)
#'
#' # Run EWTR with KM model (This will produce a warning message)
#' z <- wintime("ewtr", Time, Delta, trt, cov = cov, model = "km")
#' print(z)
#'
#' @import survival
#' @export

# ---------------------------------------------
# Main calling function
# ---------------------------------------------
wintime <- function(type,Time,Delta,trt,cov = NULL,model = NULL,resample = NULL,resample_num = 0,time_restriction = NA,seed = NA,nimp = 1) {
#  cat('Wintime called with model=',model,'\n')
  # -------------------------------
  # Validate inputs
  # -------------------------------
  type <- tolower(type)
  if (!(type %in% c("ewt","ewtr","rmt","max","wtr","rwtr","pwt","ewtp","rewtp","ewtpr","rewtpr","rpwt"))) {
    stop(paste("Invalid type:", type, "- Please specify one of 'ewt', 'ewtr', 'rmt', 'max', 'wtr', 'rwtr', 'pwt', 'ewtp', 'rewtp', 'ewtpr', 'rewtpr', 'rpwt'"))
  }
  if (!is.numeric(Time) || !is.numeric(Delta) || !is.numeric(trt)) {
    stop("Input vectors and matrices must be numeric.")
  }
  if (!is.matrix(Time) || !is.matrix(Delta) || !is.vector(trt)) {
    stop("Event times and event indicators must be organized into matrices. Treatment arm indicators must be contained in a single vector.")
  }
  if (nrow(Time) != nrow(Delta) || ncol(Time) != ncol(Delta) || ncol(Delta) != length(trt)) {
    stop("Input matrices and vectors must have compatible dimensions and lengths.")
  }
  if (length(trt) == 0 || length(Time) == 0 || length(Delta) == 0) {
    stop("Input matrices and vectors cannot be empty.")
  }


  # cat('set parameters','\n')
  # -----------------------------
  # Set Parameters
  # -----------------------------
  # Number of control patients
  n0 <- sum(trt == 0)

  # Number of treatment patients
  n1 <- sum(trt == 1)

  # Total patients
  n <- n0 + n1

  # Number of hierarchy levels
  m <- nrow(Time)

  # Tau
  tau <- max(Time[m, ])


  # -------------------------------
  # Further validate inputs
  # -------------------------------
  for (k in 1:(m-1)) {
    if (any(Time[k,] < Time[m,] & Delta[k,]==0)) {
      stop("Input Time of nonfatal event with Delta=0 can't preceed cens/death Time.")
    }
  }
  for (k in 1:(m-1)) {
    if (any(Time[k,]*Delta[k,] > Time[m,])) {
      stop("Input Time of nonfatal event with Delta=1 can't exceed cens/death Time.")
    }
  }

  # Set seed for RNG
  if (!is.na(seed)) {
    set.seed(seed)
  }

  # ----------------------------
  # Sort Input Data
  # ----------------------------
  sorted_indices <- order(trt)
  trt <- trt[sorted_indices]

  Time <- Time[ ,sorted_indices]
  Delta <- Delta[ ,sorted_indices]
  cov <- cov[sorted_indices, ,drop = FALSE]

  # cat('set defaults','\n')
  # ----------------------------
  # Set defaults
  # ----------------------------
  type <- tolower(type)
  if (is.null(model)) {
    if (type %in% c("ewt","rmt","ewtp","rewtp")) {
      model <- "km"
    }
    else if (type %in% c("ewtr","max","ewtpr","rewtpr")) {
      model <- "markov"
    }
    else {
      model <- NULL
    }
  }

  if (is.null(resample)) {
    if (type == "ewt" || type == "rmt") {
      resample <- "perm"
    }
    else if (type == "wtr" || type == "rwtr" || type == "pwt" || type == "max" || type == "rpwt") {
      resample <- "boot"
    }
    else {
      resample <- NULL
    }
  }

  #cat('fit model','\n')
  # ---------------------------
  # Fit model
  # ---------------------------
  z <- NULL
  markov_ind <- FALSE

  if (!is.null(model)) {
    # If a Markov model is specified, call the function to fit a Markov model
    if (model == "markov") {
      z <- markov(n0,n1,m,Time,Delta)
      markov_ind <- TRUE
    }
    # If no model is specified, use the default (KM) model
    if (model == "km") {
      # cat('call km','\n')
      # cat('call n0=',n0,'\n')
      # cat('call n1=',n1,'\n')
      # cat('call m=',m,'\n')
      # cat('Time=','\n')
      # print(Time)
      # cat('Delta=','\n')
      # print(Delta)
      z <- km(n0,n1,m,Time,Delta)
    }
  }
#  cat('is.null(z)=',is.null(z),'\n')
#  cat('model=',model,'\n')

  # Parameters for type function calls
  if (!is.null(z)) {
    dist_state0 <- z[[1]]
    dist_state1 <- z[[2]]
    untimes0 <- z[[3]]
    untimes1 <- z[[4]]
    nuntimes0 <- z[[5]]
    nuntimes1 <- z[[6]]
    max_follow0 <- z[[7]]
    max_follow1 <- z[[8]]
    dist_state2 <- z[[9]]
    untimes2 <- z[[10]]
    nuntimes2 <- z[[11]]
    max_follow2 <- z[[12]]
    if (model == "km") {
      comkm <- z[[13]]
      trtkm <- z[[14]]
      conkm <- z[[15]]

      #cat('km was fit','\n')
      #cat('----------------------------------','\n')
      #cat('nuntimes0=',nuntimes0,'\n')
      #cat('untimes0=',untimes0,'\n')
      #cat('dim(conkm)=',dim(conkm),'\n')
      #cat('conkm=','\n')
      #print(conkm)
      #cat('dist_state0=','\n')
      #print(dist_state0)
      #cat('----------------------------------','\n')
      #cat('nuntimes1=',nuntimes1,'\n')
      #cat('untimes1=',untimes1,'\n')
      #cat('dim(trtkm)=',dim(trtkm),'\n')
      #cat('trtkm=','\n')
      #print(trtkm)
      #cat('dist_state1=','\n')
      #print(dist_state1)
      #cat('----------------------------------','\n')

      trans_prob2 <- array(data=0,dim=c(m,m,nuntimes2))
      trans_prob1 <- array(data=0,dim=c(m,m,nuntimes1))
      trans_prob0 <- array(data=0,dim=c(m,m,nuntimes0))
    }
    if (model == "markov") {
      trans_prob2 <- z[[13]]
      trans_prob1 <- z[[14]]
      trans_prob0 <- z[[15]]
      comkm <- array(data=0,dim=c(m,nuntimes2))
      trtkm <- array(data=0,dim=c(m,nuntimes1))
      conkm <- array(data=0,dim=c(m,nuntimes0))
    }
    z_ewtr <- 0
    z_comp <- 0
    z_ewtp <- 0
    z_rewtp <- 0
    z_ewtpr <- 0
    # z_rewtpr <- 0
  }
   # cat('----------------------------------------------','\n')
   # cat('----------------------------------------------','\n')
   # cat('From MAIN:max_follow0=',max_follow0,'\n')
   # cat('nuntimes0=',nuntimes0,'\n')
   # cat('untimes0=',untimes0,'\n')
   # cat('----------------------------------------------','\n')

   # cat('type function calls','\n')
   # cat('type=',type,'\n')
  # --------------------------
  # Type function calls
  # --------------------------
  obs_data <- NA
  wins <- NA
  losses <- NA
  components <- rep(NA,m)
  components_var <- rep(NA,m)
  max_time <- rep(NA,m)

  if (type == "ewt") {
    obs_data <- EWT(m,dist_state0,dist_state1,untimes0,untimes1,nuntimes0,nuntimes1)
    max_time <- obs_data[[3]]
  }
  else if (type == "ewtr") {
    obs_data <- EWTR(n,m,nuntimes0,max_follow0,untimes0,Time,Delta,dist_state0,markov_ind,cov,trt)
  }
  else if (type == "ewtp") {
    obs_data <- EWTP(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt)
    max_time <- obs_data[[6]]
  }
  else if (type == "rewtp") {
    if (is.na(time_restriction)) {
      stop(paste("Please provide the time_restriction argument for this method."))
    }
    obs_data <- REWTP(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt,time_restriction)
    max_time <- obs_data[[6]]
  }
  else if (type == "ewtpr") {
    obs_data <- EWTPR(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt,comkm,trans_prob2,nuntimes1,max_follow1,untimes1,dist_state1,trtkm,trans_prob1,nuntimes0,max_follow0,untimes0,dist_state0,conkm,trans_prob0,nimp)
    max_time <- obs_data[[6]]
  }
  else if (type == "rewtpr") {
    if (is.na(time_restriction)) {
      stop(paste("Please provide the time_restriction argument for this method."))
    }
    obs_data <- REWTPR(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt,comkm,trans_prob2,time_restriction,nuntimes1,max_follow1,untimes1,dist_state1,trtkm,trans_prob1,nuntimes0,max_follow0,untimes0,dist_state0,conkm,trans_prob0,nimp)
    max_time <- obs_data[[6]]
  }
  else if (type == "rmt") {
    if (is.na(time_restriction)) {
      stop(paste("Please provide the time_restriction argument for this method."))
    }
    obs_data <- RMT(m,time_restriction,dist_state0,dist_state1,untimes0,untimes1,nuntimes0,nuntimes1)
    max_time <- obs_data[[3]]
  }
  else if (type == "wtr") {
#    cat('call WTR for obs data','\n')
#    temp <- WTR(n,m,tau,trt,Time,Delta)
#    cat('return WTR for obs data','\n')
    obs_data <- WTR(n,m,tau,trt,Time,Delta)
    wins <- obs_data[[2]]
    losses <- obs_data[[3]]
  }
  else if (type == "rwtr") {
    obs_data <- RWTR(n,m,tau,trt,Time,Delta)
    wins <- obs_data[[2]]
    losses <- obs_data[[3]]
  }
  else if (type == "pwt") {
#    cat('call PWT for obs data','\n')
    obs_data <- PWT(n,n0,n1,m,Time,Delta,trt,tau)
#    cat('return PWT for obs data','\n')
  }
  else if (type == "rpwt") {
    obs_data <- RPWT(n,n0,n1,m,Time,Delta,trt,tau,time_restriction)
  }
  # type == "max"
  else {
    z_ewtr <- EWTR(n,m,nuntimes0,max_follow0,untimes0,Time,Delta,dist_state0,markov_ind,cov,trt)[[3]]
#    z_ewtp <- EWTP(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt)[[3]]
#    z_rewtp <- REWTP(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt,time_restriction)[[3]]
#    z_ewtpr <- EWTPR(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt,comkm,trans_prob)[[3]]
#    z_rewtpr <- REWTPR(n,m,nuntimes2,max_follow2,untimes2,Time,Delta,dist_state2,markov_ind,cov,trt,comkm,trans_prob,time_restriction)[[3]]
    z_comp <- COMP(n,Time,Delta,cov,trt)[[1]]
    obs_data <- max(z_ewtr,z_comp)
  }

  #cat('set data used for p-value','\n')
  # Set data used for p-value calculation
  data <- NA
  if (type == "wtr" || type == "rwtr") {
    data <- obs_data[[1]]
  }
  else if (type %in% c("ewtr","ewtp","rewtp")) {
    data <- obs_data[[1]]
    components <- obs_data[[4]]
    components_var <- obs_data[[5]]
  }
  else if (type %in% c("pwt","rpwt","rmt","ewt")) {
    data <- obs_data[[1]]
    components <- obs_data[[2]]
  }
  else if (type %in% c("ewtpr","rewtpr")) {
    data <- obs_data[[1]]
    components <- obs_data[[4]]
    components_var <- obs_data[[5]]
  }
  else {
    data <- obs_data
  }

  #cat('hypothesis tests','\n')
  #cat('data=','\n')
  #print(data)
  #cat('components=','\n')
  #print(components)
  #cat('components_var=','\n')
  #print(components_var)
  # ----------------------------
  # Hypothesis testing
  # ----------------------------
  message <- NULL
  p <- NA
  resample_data <- NA
  resample_components <- matrix(NA,nrow=m,ncol=resample_num)
  sd_type <- NA
  sd_components <- rep(NA,m)
  variance <- NA
  #components_var <- rep(NA,m)
  if (!is.null(resample)) {
    components_var <- rep(NA,m)
    # Run type function on bootstrap data
    if (resample == "boot" || resample == "bootstrap") {
      message <- paste("Resampling of ", type, " done on ", resample_num, " bootstraps.")
      temp <- bootstrap(type,time_restriction,model,n,m,Time,Delta,trt,cov,z_ewtr,z_comp,resample_num,seed,nimp)
      resample_data <- temp[[1]]
      if (type %in% c("ewtr","ewtp","rewtp","ewtpr","rewtpr","pwt","rpwt","rmt","ewt")) {
        resample_components <- temp[[2]]
      }
      # cat('In Main resample_components=','\n')
      # print(resample_components)
    }
    # Run type function on permuted data
    else if (resample == "perm" || resample == "permutation" || resample == "perms" || resample == "permutations") {
      message <- paste("Resampling of ", type, " done on ", resample_num, " permutations.")
      temp <- perm(type,time_restriction,model,n,m,Time,Delta,trt,cov,z_ewtr,z_comp,resample_num,seed,nimp)
      resample_data <- temp[[1]]
      if (type %in% c("ewtr","ewtp","rewtp","ewtpr","rewtpr","pwt","rpwt","rmt","ewt")) {
        resample_components <- temp[[2]]
      }
    }
    else {
      message <- paste("No resampling done. Analysis performed only on observed data.'",resample,"' is not a valid resampling technique. For resampling, please enter 'boot' for bootstraps, 'perm' for permutations, or allow the default resampling for this method.")
      resample <- NA
    }
  }
  #cat('resample_data=',resample_data,'\n')
  if (!is.null(resample)) {
    # Calculate SD and p-value
    sd_type <- sd(resample_data)
    variance <- sd_type^2
    for (k in 1:m) {
      temp <- resample_components[k,]
      sd_components[k] <- sd(temp)
      # cat('sd_components[k]=',sd_components[k],'\n')
      # cat('resample_components[k,]=',resample_components[k,],'\n')
    }
    components_var <- sd_components^2
    if (type == "wtr" || type == "rwtr") {
      p <- (1-pnorm(abs(data-1)/sd_type,mean=0,sd=1))
    }
    else if (type == "ewt" || type == "rmt" || type== "ewtpr" || type== "rewtpr" || type == "max") {
      p <- (sum(data < resample_data) + 1)/(resample_num + 1)
      #cat('num=',sum(data < resample_data)+1,'\n')
      #cat('denom=',resample_num+1,'\n')
    }
    else {
      p <- (1-pnorm(abs(data)/sd_type,mean=0,sd=1))
    }
  }
  else {
    # message <- paste("No resampling method specified; analysis performed on original data.")
    if (type %in% c("ewtr","ewtp","rewtp","ewtpr","rewtpr")) {
      variance <- obs_data[[2]]
      #cat('variance=',variance,'\n')
      p <- (1-pnorm(abs(data)/sqrt(variance),mean=0,sd=1))
    }
  }
  #cat('components=','\n')
  #print(components)
  #cat('components_var=','\n')
  #print(components_var)
#  cat('warnings','\n')
  # ---------------------------------------------------------------------------------------------------
  # Warning message for certain combinations of type and model/resampling method
  # ---------------------------------------------------------------------------------------------------
  if ((type %in% c("ewt","rmt")) && (model == "markov" || (resample %in% c("boot","bootstrap")))) {
    warning_message <- warning("For this method, it is recommended to use a KM model and to resample using permutations. These are set as defaults.")
  }
  if ((type %in% c("ewtr","ewtpr","rewtpr")) && model == "km") {
    warning_message <- warning("Results with a KM model are meaningless! For this method, you must use a Markov model. This is set as a default.")
  }
  if (type %in% c("wtr","rwtr","pwt","rpwt") && resample %in% c("perm","perms","permutation","permutations")) {
    warning_message <- warning("For this method, it is recommended to resample using bootstraps. This is set as a default.")
  }
  if (type == "max" && (model == "km" || resample %in% c("perm","perms","permutation","permutations"))) {
    warning_message <- warning("For this method, it is recommended to use a Markov model and to resample using bootstraps. These are set as defaults.")
  }
    return(list(trt_effect = data, resample_data = resample_data, message = message, variance = variance, p = p, wins = wins, losses = losses, components=components, components_var=components_var, max_time=max_time))
}

library(testthat)
library(survival)
library(wintime)

# ------------------------------------------------
# Example matrices and vectors for testing
# ------------------------------------------------

# Event time vectors
TIME_1 <- c(256,44,29,186,29,190,292,26,11,858,47,33,11,230,394,58,179,303,186,110)
TIME_2 <- c(128,44,95,186,69,190,332,7,11,497,47,33,234,321,219,58,29,303,172,427)
TIME_3 <- c(435,44,95,186,69,190,332,325,11,1348,47,33,234,1085,442,58,219,303,186,544)

# Event time matrix
Time <- rbind(TIME_1, TIME_2, TIME_3)

# Event indicator vectors
DELTA_1 <- c(1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1)
DELTA_2 <- c(1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1)
DELTA_3 <- c(0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0)

# Event indicator matrix
Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)

# Treatment arm indicator vector
trt <- c(1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0)

# Covariate vectors
cov1 <- c(66,67,54,68,77,65,55,66,77,54,23,13,87,105,32,5,22,19,66,92)
cov2 <- c(3,6,4,2,3,5,8,5,3,5,1,1,9,14,0,3,7,2,1,5)
cov3 <- c(34.6,543.6,45.8,54.7,44.3,55.6,65.9,54.7,77.9,31.2,11,55.8,79.0,323.5,54.2,28.6,452.1,77.9,41.6,89.3)

# Covariate matrix
cov <- cbind(cov1, cov2, cov3)

# ---------------------------------
# Test wintime::wintime()
# ---------------------------------

test_that("wintime function works with valid inputs", {
  # Test 'ewt' method with default settings
  result <- wintime("ewt", Time, Delta, trt)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'ewtr' method with default settings
  result <- wintime("ewtr", Time, Delta, trt, cov = cov)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'rmt' method with default settings and 10 resamples
  result <- wintime("rmt", Time, Delta, trt, time_restriction = 365, resample_num = 10)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'max' method with default settings and 10 resamples
  result <- wintime("max", Time, Delta, trt, resample_num = 10)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'wtr' method with default settings
  result <- wintime("wtr", Time, Delta, trt)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'rwtr' method with default settings and 5 resamples
  result <- wintime("rwtr", Time, Delta, trt, resample_num = 5)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'pwt' method with default settings
  result <- wintime("pwt", Time, Delta, trt)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'rpwt' method with default settings
  result <- wintime("rpwt", Time, Delta, trt, time_restriction = 365)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'ewtp' method with default settings
  result <- wintime("ewtp", Time, Delta, trt)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'rewtp' method with default settings
  result <- wintime("rewtp", Time, Delta, trt, time_restriction = 365)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'ewtpr' method with default settings
  result <- wintime("ewtpr", Time, Delta, trt, nimp = 1)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))

  # Test 'rewtpr' method with default settings
  result <- wintime("rewtpr", Time, Delta, trt, time_restriction = 365, nimp = 1)
  expect_type(result, "list")
  expect_named(result, c("trt_effect","resample_data","message","variance","p","wins","losses","components","components_var","max_time"))
})

test_that("wintime function handles invalid inputs", {
  # Invalid type
  expect_error(wintime("invalid_type", Time, Delta, trt), "Invalid type:")

  # Non-numeric input for Time
  expect_error(wintime("ewtr", "invalid", Delta, trt), "Input vectors and matrices must be numeric.")

  # Non-matrix input for Delta
  expect_error(wintime("ewtr", Time, Delta[1, ], trt), "Event times and event indicators must be organized into matrices. Treatment arm indicators must be contained in a single vector.")

  # Mismatched dimensions
  TIME_1_mismatch <- c(256,44,29,186,29,80,11,380,102)
  TIME_2_mismatch <- c(128,44,95,186,69,66,153,380,117)
  TIME_3_mismatch <- c(435,44,95,186,69,270,1063,380,117)

  # Event time matrix
  Time_mismatch <- rbind(TIME_1_mismatch, TIME_2_mismatch, TIME_3_mismatch)
  expect_error(wintime("ewtr", Time_mismatch, Delta, trt), "Input matrices and vectors must have compatible dimensions and lengths.")
})

test_that("wintime function handles edge cases", {
  # Test with empty inputs
  Time_empty <- matrix(numeric(0), nrow = 0, ncol = 0)
  Delta_empty <- matrix(numeric(0), nrow = 0, ncol = 0)
  trt_empty <- numeric(0)
  expect_error(wintime("ewtr", Time_empty, Delta_empty, trt_empty), "Input matrices and vectors cannot be empty.")
})

test_that("wintime function handles warnings", {
  # Test warnings for 'ewt' with Markov model
  expect_warning(result <- wintime("ewt", Time, Delta, trt, model = "markov"), "For this method, it is recommended to use a KM model and to resample using permutations. These are set as defaults.")
})

#' Fit a Kaplan-Meier model
#'
#' This function fits Kaplan-Meier models to calculate the state probabilities for each arm. In the wintime package, the returned state probability
#' distributions are used in all non-pairwise methods. The Kaplan-Meier model is recommended for the Expected win time (EWT) method and the Restricted mean survival
#' in favor of treatment (RMT) method.
#'
#' @param n0 The number of participants in the control arm.
#' @param n1 The number of participants in the treatment arm.
#' @param m The number of events in the hierarchy.
#' @param Time A `m x (n0 + n1)` matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
#' in increasing order of clinical severity.
#' @param Delta A `m x (n0 + n1)` matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should be
#' in increasing order of clinical severity.
#' @return A list containing: a matrix of control arm state probabilities, a matrix of treatment arm state probabilities,
#' a vector of unique control arm event times (days), a vector of unique treatment arm event times (days), the number of unique control
#' arm event times, the number of unique treatment arm event times, the control arm max follow time (days), the treatment arm
#' max follow time (days), a matrix of combined arm state probabilities, a vector of unique combined arm event times (days),
#' the number of unique combined arm event times, the combined arm max follow time (days), a (m x number unique combined arm event times)
#' matrix of combined arm km survival probabilities, matrix of trt arm km survival probabilities, matrix of control arm km survival probabilities.
#' @examples
#' # -----------------------------
#' # Example inputs
#' # -----------------------------
#'
#' # Event time vectors
#' TIME_1 <- c(256,44,29,186,29,80,11,380,102,33)
#' TIME_2 <- c(128,44,95,186,69,66,153,380,117,33)
#' TIME_3 <- c(435,44,95,186,69,270,1063,380,117,33)
#'
#' # Event time matrix
#' Time <- rbind(TIME_1, TIME_2, TIME_3)
#'
#' # Event indicator vectors
#' DELTA_1 <- c(1,0,1,0,1,1,1,0,1,0)
#' DELTA_2 <- c(1,0,0,0,0,1,1,0,0,0)
#' DELTA_3 <- c(0,0,0,0,0,0,0,0,0,0)
#'
#' # Event indicator matrix
#' Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)
#'
#' # Treatment arm indicator vector
#' trt <- c(1,1,1,1,1,0,0,0,0,0)
#'
#' # Number of control arm patients
#' n0 <- sum(trt == 0)
#'
#' # Number of treatment arm patients
#' n1 <- sum(trt == 1)
#'
#' # Number of events in the hierarchy
#' m <- nrow(Time)
#'
#' # ------------------------
#' # km Examples
#' # ------------------------
#'
#' z <- km(n0, n1, m, Time, Delta)
#' print(z)
#'
#' @import survival
#' @export

# -------------------------
# Kaplan-Meier model
# -------------------------
km <- function(n0,n1,m,Time,Delta) {
  # -------------------------------
  # Validate inputs
  # -------------------------------
  if (!is.numeric(Time) || !is.numeric(Delta) || !is.numeric(n0) || !is.numeric(n1) || !is.numeric(m)) {
    stop("Input data must be numeric.")
  }
  if (length(Time) == 0 || length(Delta) == 0) {
    stop("Input matrices cannot be empty.")
  }
  if (nrow(Time) != nrow(Delta) || ncol(Time) != ncol(Delta)) {
    stop("Input matrices must have compatible dimensions.")
  }
  if (nrow(Time) != m || nrow(Delta) != m || ncol(Time) != (n0+n1) || ncol(Delta) != (n0+n1)) {
    stop("Rows of input matrices must correspond to events. Columns must correspond to participants. Ensure that the number of events
         is equal to the number of rows and that the number of participants is equal to the columns.")
  }

  n <- n0 + n1
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]

  # Set KM time/delta matrices
  result <- setKM(n,m,time,delta)
  time_km <- result[[1]]
  delta_km <- result[[2]]

  # cat('----------------------------------------------------','\n')
  # cat('From km: time_km=','\n')
  # print(time_km)
  # cat('----------------------------------------------------','\n')

  #---------------------------------------------------------------------
  # CONTROL GROUP
  #
  #---------------------------------------------------------------------

  # Create KM survival curve matrices
  Time0 <- matrix(0,nrow=m,ncol=n)
  Surv0 <- matrix(0,nrow=m,ncol=n)
  NRisk0 <- matrix(0,nrow=m,ncol=n)
  conkm <- matrix(0,nrow=m,ncol=m*n)

  for (k in 1:m) {
    # temp <- survfit(Surv(time_km[k,dataset1$trt==0],delta_km[k,dataset1$trt==0])~1)$time
    # temp2 <- survfit(Surv(time_km[k,dataset1$trt==0],delta_km[k,dataset1$trt==0])~1)$surv
    temp <- survival::survfit(survival::Surv(time_km[k,1:n0],delta_km[k,1:n0])~1)$time
    temp2 <- survival::survfit(survival::Surv(time_km[k,1:n0],delta_km[k,1:n0])~1)$surv
    temp3 <- survival::survfit(survival::Surv(time_km[k,1:n0],delta_km[k,1:n0])~1)$n.risk
    Time0[k,1:length(temp)] <- temp
    Surv0[k,1:length(temp2)] <- temp2
    NRisk0[k,1:length(temp3)] <- temp3
  }

  # cat('','\n')
  # cat('-------------------------------------------','\n')
  # cat('-------------------------------------------','\n')
  # cat('START KM Control group','\n')
  # cat('-------------------------------------------','\n')
  # cat('n=',n,'\n')
  # cat('-------------------------------------------','\n')
  # cat('Control group times for m-k+1+ events (reverse order), Time0[k,]=','\n')
  # print(Time0)

  # cat('Control group survival estimates for m-k+1+ events (reverse order), Surv0[k,]=','\n')
  # print(Surv0)

  # cat('Control group # at risk for m-k+1+ events (reverse order), NRisk0[k,]=','\n')
  # print(NRisk0)

  nkm <- numeric(m)
  for (k in 1:m) {
    nkm[k] <- sum(Time0[k,] != 0)
  }

  # cat('Control group # of times, nkm[k]=','\n')
  # print(nkm)

  dist_state0 <- matrix(0,nrow=(m+1),ncol=sum(nkm))
  unique_event_times0 <- rep(0,times=sum(nkm))

  # Get max follow time
  control_follow_times <- rep(0,times=m)
  for (i in 1:m) {
    control_follow_times[i] <- Time0[i,nkm[i]]
  }
  max_follow0 <- min(control_follow_times)

  #cat('Control group max_follow0=',max_follow0,'\n')

  time <- 0
  count <- rep(0,m)
  max_times <- rep(0,m)
  nunique_event_times0 <- 0

  next_count <- rep(0,m)
  KM_end=0

   while (KM_end==0 & time < max(Time0)) {
  #while (time < max_follow0) {
    for (i in 1:m) {
      next_count[i] <- count[i] + 1
    }
    for (i in 1:m) {
      if (next_count[i] > nkm[i]) {
        next_count[i] <- nkm[i]
        max_times[i] <- 1
      }
    }

    time <- max(Time0)
    for (k in 1:m) {
      if (max_times[k] == 0) {
        if (Time0[k,next_count[k]] < time) {
          time <- Time0[k,next_count[k]]
        }
      }
    }

    nunique_event_times0=nunique_event_times0+1
    unique_event_times0[nunique_event_times0]=time

    # cat('-------------------------','\n')
    # cat('Add unique event time=',unique_event_times0[nunique_event_times0],'\n')
    # cat('nunique_event_times0=',nunique_event_times0,'\n')
    # cat('unique_event_times0=','\n')
    # print(unique_event_times0)
    # cat('-------------------------','\n')

    # cat('-------------------------','\n')
    # cat('Before Updating  conkm[k,]','\n')
    # cat('Control group # at risk for m-k+1+ events (reverse order), NRisk0[k,]=','\n')
    # print(NRisk0)
    # cat('next_count[k]=','\n')
    # print(next_count[])
    # cat('dist_state0[k,] for being in state k-1=','\n')
    # print(dist_state0)
    # cat('-------------------------','\n')


    #  Update conkm[]
    #
    for (k in 1:m) {
      if (Time0[k,next_count[k]] == time) {
        conkm[k,nunique_event_times0] <- Surv0[k,next_count[k]]
      }
      else {
        if (nunique_event_times0==1) {
          conkm[k,nunique_event_times0] <- 1.0
        }
        else {
          conkm[k,nunique_event_times0] <- conkm[k,nunique_event_times0-1]
        }
      }
    }

    # cat('-------------------------','\n')
    # cat('Updated conkm[k,] for m-k+1+ events(reverse order)=','\n')
    # print(conkm)
    # cat('-------------------------','\n')

    for (k in 1:m) {
      if (Time0[k,next_count[k]] <= time) {
        count[k] <- next_count[k]
      }
    }

    # cat('Updated count[k] (reverse order)=','\n')
    # print(count[])

  #---------------------------------------------------------
  # Calculate # at risk after current time in each state
  #---------------------------------------------------------
    NRisk_state0=rep(0,times=m+1)
    sum=0
    if (count[m]==n0) {
      NRisk_state0[1]=0
    } else {
      NRisk_state0[1]=NRisk0[m,count[m]+1]
      sum=NRisk0[m,count[m]+1]
    }
  #  cat('# at risk in state k-1 NRisk_state0[1]=','\n')
  #  print(NRisk_state0[1])
    for (k in 2:m) {
  #      cat('k=',k,'\n')
  #      cat('# at risk in state k-2 NRisk_state0[k-1]=','\n')
  #      print(NRisk_state0[k-1])
  #      cat('count[m-k+1]=',count[m-k+1],'\n')
  #      cat('NRisk0[(m-k+1),count[m-k+1]]=','\n')
  #      print(NRisk0[(m-k+1),count[m-k+1]])
      if (count[m-k+1]==n0) {
        NRisk_state0[k]=0
      } else {
        NRisk_state0[k]=NRisk0[(m-k+1),count[m-k+1]+1]-sum
        sum=sum+NRisk_state0[k]
      }
    }
    NRisk_state0[m+1]=n0-sum

    # cat('# at risk after current time in state k-1 NRisk_state0[k]=','\n')
    # print(NRisk_state0)


    # Set probability of being in state 0
    if (count[m] > 0) {
      dist_state0[1,nunique_event_times0] <- Surv0[m,count[m]]
    }
    else {
      dist_state0[1,nunique_event_times0] <- 1
    }

    # Set probability of being in current state
    for (i in 1:m) {
      state_num <- (m - i) + 1
      if (count[i] > 0) {
        dist_state0[state_num+1,nunique_event_times0] <- 1 - Surv0[i,count[i]]
        # Subtract previously calculated probabilities
        if (state_num < m) {
          for (j in (state_num+1):m) {
            dist_state0[(state_num+1),nunique_event_times0] <- dist_state0[(state_num+1),nunique_event_times0] - dist_state0[(j+1),nunique_event_times0]
          }
        }
      }
    }
    # cat('-------------------------','\n')
    # cat('conkm[k,] for m-k+1+ events(reverse order)=','\n')
    # print(conkm)
    # cat('dist_state0[k,] for being in state k-1=','\n')
    # print(dist_state0)
    # cat('-------------------------','\n')

    #------------------------------------------------------------------
    # Check if a state without anyone at risk has positive probability
    #------------------------------------------------------------------

    for (k in 1:(m-1)) {
        #      cat('k=',k,'\n')
        #      cat('nunique_event_times0=',nunique_event_times0,'\n')
        #      cat('NRisk_state0[k]=','\n')
        #      print(NRisk_state0[k])
        #      cat('dist_state0[k,nunique_event_times0]=','\n')
        #      print(dist_state0[k,nunique_event_times0])
      if (NRisk_state0[k]==0 & dist_state0[k,nunique_event_times0]>0) {KM_end=1}
    }
    max_follow0=time

    # cat('KM_end=',KM_end,'\n')
    # cat('time=',time,'\n')
    # cat('Control arm max_follow0=',max_follow0,'\n')

  }
  unique_event_times0=unique_event_times0[1:nunique_event_times0]
  conkm=conkm[1:m,1:nunique_event_times0]

  #---------------------------------------------------------------------
  # TREATMENT GROUP
  #
  #---------------------------------------------------------------------

  # Create KM survival curve matrices
  Time1 <- matrix(0,nrow=m,ncol=n)
  Surv1 <- matrix(0,nrow=m,ncol=n)
  NRisk1 <- matrix(0,nrow=m,ncol=n)
  trtkm <- matrix(0,nrow=m,ncol=m*n)

  for (k in 1:m) {
    # temp <- survfit(Surv(time_km[k,dataset1$trt==1],delta_km[k,dataset1$trt==1])~1)$time
    # temp2 <- survfit(Surv(time_km[k,dataset1$trt==1],delta_km[k,dataset1$trt==1])~1)$surv
    temp <- survival::survfit(survival::Surv(time_km[k,(n0+1):n],delta_km[k,(n0+1):n])~1)$time
    temp2 <- survival::survfit(survival::Surv(time_km[k,(n0+1):n],delta_km[k,(n0+1):n])~1)$surv
    temp3 <- survival::survfit(survival::Surv(time_km[k,(n0+1):n],delta_km[k,(n0+1):n])~1)$n.risk
    NRisk1[k,1:length(temp3)] <- temp3
    Time1[k,1:length(temp)] <- temp
    Surv1[k,1:length(temp2)] <- temp2
  }

  # cat('                  ','\n')
  # cat('                  ','\n')
  # cat('START KM Trt group','\n')
  # cat('-------------------------------------------','\n')
  # cat('-------------------------------------------','\n')
  # cat('Trt group times for m-k+1+ events (reverse order), Time1[k,]=','\n')
  # print(Time1)

  # cat('Trt group survival estimates for m-k+1+ events (reverse order), Surv1[k,]=','\n')
  # print(Surv1)

  # cat('Trt group # at risk for m-k+1+ events (reverse order), NRisk1[k,]=','\n')
  # print(NRisk1)

  nkm <- numeric(m)
  for (k in 1:m) {
    nkm[k] <- sum(Time1[k,] != 0)
  }

  # cat('Trt group # of times, nkm[k]=','\n')
  # print(nkm)

  dist_state1 <- matrix(0,nrow=(m+1),ncol=sum(nkm))
  unique_event_times1 <- rep(0,times=sum(nkm))

  # Get max follow time
  trt_follow_times <- rep(0,times=m)
  for (i in 1:m) {
    trt_follow_times[i] <- Time1[i,nkm[i]]
  }

  max_follow1 <- min(trt_follow_times)

  #cat('Trt group max_follow1=',max_follow1,'\n')

  time <- 0
  count <- rep(0,m)
  max_times <- rep(0,m)
  nunique_event_times1 <- 0

  next_count <- rep(0,m)
  KM_end=0

  while (KM_end==0 & time < max(Time1)) {
  #while (time < max_follow1) {
    for (i in 1:m) {
      next_count[i] <- count[i] + 1
    }
    for (i in 1:m) {
      if (next_count[i] > nkm[i]) {
        next_count[i] <- nkm[i]
        max_times[i] <- 1
      }
    }

    time <- max(Time1)
    for (k in 1:m) {
      if (max_times[k] == 0) {
        if (Time1[k,next_count[k]] < time) {
          time <- Time1[k,next_count[k]]
        }
      }
    }

    nunique_event_times1=nunique_event_times1+1
    unique_event_times1[nunique_event_times1]=time

    # cat('-------------------------','\n')
    # cat('Add unique event time=',unique_event_times1[nunique_event_times1],'\n')
    # cat('nunique_event_times1=',nunique_event_times1,'\n')
    # cat('unique_event_times1=','\n')
    # print(unique_event_times1)
    # cat('-------------------------','\n')

    # cat('-------------------------','\n')
    # cat('Before Updating  trtkm[k,]','\n')
    # cat('Trt group # at risk for m-k+1+ events (reverse order), NRisk1[k,]=','\n')
    # print(NRisk1)
    # cat('next_count[k]=','\n')
    # print(next_count[])
    # cat('dist_state1[k,] for being in state k-1=','\n')
    # print(dist_state1)
    # cat('-------------------------','\n')

    #  Update trtkm[]
    #
    for (k in 1:m) {
      if (Time1[k,next_count[k]] == time) {
        trtkm[k,nunique_event_times1] <- Surv1[k,next_count[k]]
      }
      else {
        if (nunique_event_times1==1) {
          trtkm[k,nunique_event_times1] <- 1.0
        }
        else {
          trtkm[k,nunique_event_times1] <- trtkm[k,nunique_event_times1-1]
        }
      }
    }

    # cat('-------------------------','\n')
    # cat('Updated trtkm[k,] for m-k+1+ events(reverse order)=','\n')
    # print(trtkm)
    # cat('-------------------------','\n')

    for (k in 1:m) {
      if (Time1[k,next_count[k]] <= time) {
        count[k] <- next_count[k]
      }
    }

    # cat('Updated count[k]=','\n')
    # print(count[])

    #---------------------------------------------------------
    # Calculate # at risk after current time in each state
    #---------------------------------------------------------
    NRisk_state1=rep(0,times=m)
    sum=0
    if (count[m]==n1) {
      NRisk_state1[1]=0
    } else {
      NRisk_state1[1]=NRisk1[m,count[m]+1]
      sum=NRisk1[m,count[m]+1]
    }
    #  cat('# at risk in state k-1 NRisk_state1[1]=','\n')
    #  print(NRisk_state1[1])
    for (k in 2:m) {
      #      cat('k=',k,'\n')
      #      cat('# at risk in state k-2 NRisk_state1[k-1]=','\n')
      #      print(NRisk_state1[k-1])
      #      cat('count[m-k+1]=',count[m-k+1],'\n')
      #      cat('NRisk1[(m-k+1),count[m-k+1]]=','\n')
      #      print(NRisk1[(m-k+1),count[m-k+1]])
      if (count[m-k+1]==n1) {
        NRisk_state1[k]=0
      } else {
        NRisk_state1[k]=NRisk1[(m-k+1),count[m-k+1]+1]-sum
        sum=sum+NRisk_state1[k]
      }
    }

    # cat('# at risk after current time in state k-1 NRisk_state1[k]=','\n')
    # print(NRisk_state1)


    # Set probability of being in state 0
    if (count[m] > 0) {
      dist_state1[1,nunique_event_times1] <- Surv1[m,count[m]]
    }
    else {
      dist_state1[1,nunique_event_times1] <- 1
    }

    # Set probability of being in current state
    for (i in 1:m) {
      state_num <- (m - i) + 1
      if (count[i] > 0) {
        dist_state1[state_num+1,nunique_event_times1] <- 1 - Surv1[i,count[i]]
        # Subtract previously calculated probabilities
        if (state_num < m) {
          for (j in (state_num+1):m) {
            dist_state1[(state_num+1),nunique_event_times1] <- dist_state1[(state_num+1),nunique_event_times1] - dist_state1[(j+1),nunique_event_times1]
          }
        }
      }
    }

    # cat('-------------------------','\n')
    # cat('trtkm[k,] for m-k+1+ events(reverse order)=','\n')
    # print(trtkm)
    # cat('dist_state1[k,] for being in state k-1=','\n')
    # print(dist_state1)
    # cat('-------------------------','\n')

    #------------------------------------------------------------------
    # Check if a state without anyone at risk has positive probability
    #------------------------------------------------------------------

    for (k in 1:(m-1)) {
      #      cat('k=',k,'\n')
      #      cat('nunique_event_times1=',nunique_event_times1,'\n')
      #      cat('NRisk_state1[k]=','\n')
      #      print(NRisk_state1[k])
      #      cat('dist_state1[k,nunique_event_times1]=','\n')
      #      print(dist_state1[k,nunique_event_times1])
      if (NRisk_state1[k]==0 & dist_state1[k,nunique_event_times1]>0) {KM_end=1}
    }
    max_follow1=time

    # cat('KM_end=',KM_end,'\n')
    # cat('time=',time,'\n')
    # cat('Trt group max_follow1=',max_follow1,'\n')


  }
  unique_event_times1=unique_event_times1[1:nunique_event_times1]
  trtkm=trtkm[1:m,1:nunique_event_times1]

  #---------------------------------------------------------------------
  # COMBINED ARM
  #
  #---------------------------------------------------------------------

  # Create KM survival curve matrices
  Time2 <- matrix(0,nrow=m,ncol=n)
  Surv2 <- matrix(0,nrow=m,ncol=n)
  NRisk2 <- matrix(0,nrow=m,ncol=n)

  comkm <- matrix(0,nrow=m,ncol=m*n)

  for (k in 1:m) {
    # temp <- survfit(Surv(time_km[k,dataset1$trt==0],delta_km[k,dataset1$trt==0])~1)$time
    # temp2 <- survfit(Surv(time_km[k,dataset1$trt==0],delta_km[k,dataset1$trt==0])~1)$surv
    temp <- survival::survfit(survival::Surv(time_km[k,1:n],delta_km[k,1:n])~1)$time
    temp2 <- survival::survfit(survival::Surv(time_km[k,1:n],delta_km[k,1:n])~1)$surv
    temp3 <- survival::survfit(survival::Surv(time_km[k,1:n],delta_km[k,1:n])~1)$n.risk
    NRisk2[k,1:length(temp3)] <- temp3
    Time2[k,1:length(temp)] <- temp
    Surv2[k,1:length(temp2)] <- temp2
  }
#  cat('length(Time2[1,])=',length(Time2[1,]),'\n')
#  cat('length(Time2[2,])=',length(Time2[2,]),'\n')
#  cat('length(Time2[3,])=',length(Time2[3,]),'\n')
#  cat('Time2[1,1:120]=',Time2[1,1:120],'\n')
#  cat('Surv2[1,1:15]=',Surv2[1,1:15],'\n')
#  cat('Time2[2,1:120]=',Time2[2,1:120],'\n')
#  cat('Surv2[2,1:15]=',Surv2[2,1:15],'\n')
#  cat('Time2[3,1:120]=',Time2[3,1:120],'\n')
#  cat('Surv2[3,1:15]=',Surv2[3,1:15],'\n')
#  cat('------------------------------------------','\n')

  # cat('                  ','\n')
  # cat('                  ','\n')
  # cat('START KM Combined group','\n')
  # cat('-------------------------------------------','\n')
  # cat('-------------------------------------------','\n')
  # cat('Combined group times for m-k+1+ events (reverse order), Time2[k,]=','\n')
  # print(Time2)

  # cat('Combined group survival estimates for m-k+1+ events (reverse order), Surv2[k,]=','\n')
  # print(Surv2)

  # cat('Combined group # at risk for m-k+1+ events (reverse order), NRisk2[k,]=','\n')
  # print(NRisk2)

  nkm <- numeric(m)
  for (k in 1:m) {
    nkm[k] <- sum(Time2[k,] != 0)
  }

  # cat('Combined group # of times, nkm[k]=','\n')
  # print(nkm)

#  cat('nkm[1]=',nkm[1],'\n')
#  cat('nkm[2]=',nkm[2],'\n')
#  cat('nkm[3]=',nkm[3],'\n')
#  cat('------------------------------------------','\n')


  dist_state2 <- matrix(0,nrow=(m+1),ncol=sum(nkm))
  unique_event_times2 <- rep(0,times=sum(nkm))

  # Get max follow time
  com_follow_times <- rep(0,times=m)
  for (i in 1:m) {
    com_follow_times[i] <- Time2[i,nkm[i]]
  }
  max_follow2 <- min(com_follow_times)

  # cat('Combined group max_follow2=',max_follow2,'\n')
  # cat('------------------------------------------','\n')


  time <- 0
  count <- rep(0,m)
  max_times <- rep(0,m)
  nunique_event_times2 <- 0

  next_count <- rep(0,m)
  KM_end=0

  while (KM_end==0 & time < max(Time2)) {
  #while (time < max_follow2) {
    for (i in 1:m) {
      next_count[i] <- count[i] + 1
    }
    for (i in 1:m) {
      if (next_count[i] > nkm[i]) {
        next_count[i] <- nkm[i]
        max_times[i] <- 1
      }
    }

    time <- max(Time2)
    for (k in 1:m) {
      if (max_times[k] == 0) {
        if (Time2[k,next_count[k]] < time) {
          time <- Time2[k,next_count[k]]
        }
      }
    }

    nunique_event_times2=nunique_event_times2+1
    unique_event_times2[nunique_event_times2]=time

    # cat('-------------------------','\n')
    # cat('Add unique event time=',unique_event_times2[nunique_event_times2],'\n')
    # cat('nunique_event_times2=',nunique_event_times2,'\n')
    # cat('unique_event_times2=','\n')
    # print(unique_event_times2)
    # cat('-------------------------','\n')

    # cat('-------------------------','\n')
    # cat('Before Updating  comkm[k,]','\n')
    # cat('Combined group # at risk for m-k+1+ events (reverse order), NRisk2[k,]=','\n')
    # print(NRisk2)
    # cat('next_count[k]=','\n')
    # print(next_count[])
    # cat('dist_state2[k,] for being in state k-1=','\n')
    # print(dist_state2)
    # cat('-------------------------','\n')

    #  Update comkm[]
    #
    for (k in 1:m) {
      if (Time2[k,next_count[k]] == time) {
        comkm[k,nunique_event_times2] <- Surv2[k,next_count[k]]
      }
      else {
        if (nunique_event_times2==1) {
          comkm[k,nunique_event_times2] <- 1.0
        }
        else {
          comkm[k,nunique_event_times2] <- comkm[k,nunique_event_times2-1]
        }
      }
    }

    # cat('-------------------------','\n')
    # cat('Updated comkm[k,] for m-k+1+ events(reverse order)=','\n')
    # print(comkm)
    # cat('-------------------------','\n')

    # if (time < Time2[3,15]) {
    # cat('time=',time,'\n')
    # cat('nunique_event_times2=',nunique_event_times2,'\n')
    # cat('next_count[1]=',next_count[1],'\n')
    # cat('next_count[2]=',next_count[2],'\n')
    # cat('next_count[3]=',next_count[3],'\n')
    # cat('comkm[1,1:15]=',comkm[1,1:15],'\n')
    # cat('comkm[2,1:15]=',comkm[2,1:15],'\n')
    # cat('comkm[3,1:15]=',comkm[3,1:15],'\n')
    # cat('------------------------------------------','\n')
    # }

    for (k in 1:m) {
      if (Time2[k,next_count[k]] <= time) {
        count[k] <- next_count[k]
      }
    }

    # cat('Updated count[k]=','\n')
    # print(count[])

    #---------------------------------------------------------
    # Calculate # at risk after current time in each state
    #---------------------------------------------------------
    NRisk_state2=rep(0,times=m)
    sum=0
    if (count[m]==n) {
      NRisk_state2[1]=0
    } else {
       NRisk_state2[1]=NRisk2[m,count[m]+1]
       sum=NRisk2[m,count[m]+1]
    }
    #  cat('# at risk in state k-1 NRisk_state2[1]=','\n')
    #  print(NRisk_state2[1])
    for (k in 2:m) {
      #      cat('k=',k,'\n')
      #      cat('# at risk in state k-2 NRisk_state2[k-1]=','\n')
      #      print(NRisk_state2[k-1])
      #      cat('count[m-k+1]=',count[m-k+1],'\n')
      #      cat('NRisk2[(m-k+1),count[m-k+1]]=','\n')
      #      print(NRisk2[(m-k+1),count[m-k+1]])
      if (count[m-k+1]==n) {
        NRisk_state2[k]=0
      } else {
        NRisk_state2[k]=NRisk2[(m-k+1),count[m-k+1]+1]-sum
        sum=sum+NRisk_state2[k]
      }
    }

    # cat('# at risk after current time in state k-1 NRisk_state2[k]=','\n')
    # print(NRisk_state2)

    # Set probability of being in state 0
    if (count[m] > 0) {
      dist_state2[1,nunique_event_times2] <- Surv2[m,count[m]]
    }
    else {
      dist_state2[1,nunique_event_times2] <- 1
    }

    # Set probability of being in current state
    for (i in 1:m) {
      state_num <- (m - i) + 1
      if (count[i] > 0) {
        dist_state2[state_num+1,nunique_event_times2] <- 1 - Surv2[i,count[i]]
        # Subtract previously calculated probabilities
        if (state_num < m) {
          for (j in (state_num+1):m) {
            dist_state2[(state_num+1),nunique_event_times2] <- dist_state2[(state_num+1),nunique_event_times2] - dist_state2[(j+1),nunique_event_times2]
          }
        }
      }
    }

    # cat('-------------------------','\n')
    # cat('comkm[k,] for m-k+1+ events(reverse order)=','\n')
    # print(comkm)
    # cat('dist_state2[k,] for being in state k-1=','\n')
    # print(dist_state2)
    # cat('-------------------------','\n')

    #------------------------------------------------------------------
    # Check if a state without anyone at risk has positive probability
    #------------------------------------------------------------------

    for (k in 1:(m-1)) {
      #      cat('k=',k,'\n')
      #      cat('nunique_event_times2=',nunique_event_times2,'\n')
      #      cat('NRisk_state2[k]=','\n')
      #      print(NRisk_state2[k])
      #      cat('dist_state2[k,nunique_event_times2]=','\n')
      #      print(dist_state2[k,nunique_event_times2])
      if (NRisk_state2[k]==0 & dist_state2[k,nunique_event_times2]>0) {KM_end=1}
    }
    max_follow2=time

    # cat('KM_end=',KM_end,'\n')
    # cat('time=',time,'\n')
    # cat('Combined group max_follow2=',max_follow2,'\n')


  }
  unique_event_times2=unique_event_times2[1:nunique_event_times2]
  dist_state2=dist_state2[1:(m+1),1:nunique_event_times2]
  comkm=comkm[1:m,1:nunique_event_times2]

   # cat('----------------------------------------------------','\n')
   # cat('From km: nunique_event_times2=',nunique_event_times2,'\n')
   # cat('From km: unique_event_times2=',unique_event_times2,'\n')
   # cat('From km: dist_state2=','\n')
   # print(dist_state2)
   # cat('----------------------------------------------------','\n')

  return(list(dist_state0 = dist_state0,dist_state1 = dist_state1,unique_event_times0 = unique_event_times0,unique_event_times1 = unique_event_times1,
              nunique_event_times0 = nunique_event_times0,nunique_event_times1 = nunique_event_times1,max_follow0 = max_follow0,max_follow1 = max_follow1,
              dist_state2 = dist_state2, unique_event_times2 = unique_event_times2, nunique_event_times2 = nunique_event_times2, max_follow2 = max_follow2,
              comkm=comkm,trtkm=trtkm,conkm=conkm))
}

#' Resample using permutations
#'
#' This function reruns the desired wintime package method on a given number of permutations. This resampling method is recommended
#' for the Expected win time (EWT) and Restricted mean survival in favor of treatment (RMT) methods.
#'
#' @param type A string value indicating the wintime package method that will run with resampling.
#' @param time_restriction The time cutoff value (days).
#' @param model A string value indicating the model used on observed data ('markov' or 'km').
#' @param n The total number of trial participants.
#' @param m The number of events in the hierarchy.
#' @param Time A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
#' in increasing order of clinical severity.
#' @param Delta A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should
#' be in increasing order of clinical severity.
#' @param trt A numeric vector of treatment arm indicators (1 for treatment, 0 for control).
#' @param cov A n x p matrix of covariate values, where p is the number of covariates. Rows should represent participants and columns should
#' represent covariate values.
#' @param z_ewtr The Z-statistic of EWTR.
#' @param z_comp The Z-statistic of the composite event approach.
#' @param resample_num The number of desired permutations.
#' @param seed The seed used for random number generation.
#' @param nimp The number of random imputations for Redistribution-to-the-right.
#' @return A list of a vector of length resample_num containing the treatment effect estimates (for type='max' these are z-statistics) for each permutation,
#' a m x resample_num matrix of the components of the treatment effect..

# -------------------------------
# Permutations
# -------------------------------
perm <- function(type,time_restriction,model,n,m,Time,Delta,trt,cov,z_ewtr,z_comp,resample_num,seed,nimp) {
  # Initialize vector to hold permuted data values
  y <- rep(0,times = resample_num)
  components <- matrix(0,nrow=m,ncol=resample_num)
  time <- Time[m:1, ]
  delta <- Delta[m:1, ]
  iperm <- 1
  while (iperm <= resample_num) {
    # Generate permutations
    perm_indices <- sample(1:n, size = length(trt), replace = FALSE)
    trt_perm <- trt[perm_indices]
    Time_perm <- Time[, perm_indices, drop = FALSE]
    Delta_perm <- Delta[, perm_indices, drop = FALSE]
    time_perm <- time[, perm_indices, drop = FALSE]
    delta_perm <- delta[, perm_indices, drop = FALSE]
    cov_perm <- cov[perm_indices,, drop = FALSE]

    # Set parameters for model fitting calls
    n0 <- sum(trt_perm == 0)
    n1 <- sum(trt_perm == 1)
    trt_perm[1:n0] <- 0
    trt_perm[(n0+1):n] <- 1
    m <- nrow(Time_perm)
    tau <- max(Time_perm[m, ])

    # Type function calls on permuted data
    z <- NULL
    markov_ind <- FALSE
    if (!is.null(model)) {
      # If a Markov model is specified, call the function to fit a Markov model
      if (model == "markov") {
        z <- markov(n0,n1,m,Time_perm,Delta_perm)
        markov_ind <- TRUE
      }
      # If no model is specified, use the default (KM) model
      else {
        z <- km(n0,n1,m,Time_perm,Delta_perm)
      }
    }

    if (!is.null(z)) {
      # Parameters for type function calls
      dist_state0_perm <- z[[1]]
      dist_state1_perm <- z[[2]]
      untimes0_perm <- z[[3]]
      untimes1_perm <- z[[4]]
      nuntimes0_perm <- z[[5]]
      nuntimes1_perm <- z[[6]]
      max_follow0_perm <- z[[7]]
      max_follow1_perm <- z[[8]]
      dist_state2_perm <- z[[9]]
      untimes2_perm <- z[[10]]
      nuntimes2_perm <- z[[11]]
      max_follow2_perm <- z[[12]]
      if (model == "km") {
        comkm_perm <- z[[13]]
        trtkm_perm <- z[[14]]
        conkm_perm <- z[[15]]
        trans_prob2_perm <- array(data=0,dim=c(m,m,nuntimes2_perm))
        trans_prob1_perm <- array(data=0,dim=c(m,m,nuntimes1_perm))
        trans_prob0_perm <- array(data=0,dim=c(m,m,nuntimes0_perm))
      }
      if (model == "markov") {
        trans_prob2_perm <- z[[13]]
        trans_prob1_perm <- z[[14]]
        trans_prob0_perm <- z[[15]]
        comkm_perm <- array(data=0,dim=c(m,nuntimes2_perm))
        trtkm_perm <- array(data=0,dim=c(m,nuntimes1_perm))
        conkm_perm <- array(data=0,dim=c(m,nuntimes0_perm))
      }
    }

    # Type function calls
    type <- tolower(type)
    if (type == "ewt") {
      temp <- EWT(m,dist_state0_perm,dist_state1_perm,untimes0_perm,untimes1_perm,nuntimes0_perm,nuntimes1_perm)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[2]]
      }
    else if (type == "ewtr") {
      temp <- EWTR(n,m,nuntimes0_perm,max_follow0_perm,untimes0_perm,Time_perm,Delta_perm,dist_state0_perm,markov_ind,cov_perm,trt_perm)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[4]]
    }
    else if (type == "ewtp") {
      temp <- EWTP(n,m,nuntimes2_perm,max_follow2_perm,untimes2_perm,Time_perm,Delta_perm,dist_state2_perm,markov_ind,cov_perm,trt_perm)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[4]]
      #      cat('resampled components for iperm=',iperm,'\n')
      #      print(components[,iperm])
    }
    else if (type == "rewtp") {
      temp <- REWTP(n,m,nuntimes2_perm,max_follow2_perm,untimes2_perm,Time_perm,Delta_perm,dist_state2_perm,markov_ind,cov_perm,trt_perm,time_restriction)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[4]]
      #      cat('resampled components for iperm=',iperm,'\n')
      #      print(components[,iperm])
    }
    else if (type == "ewtpr") {
      temp <- EWTPR(n,m,nuntimes2_perm,max_follow2_perm,untimes2_perm,Time_perm,Delta_perm,dist_state2_perm,markov_ind,cov_perm,trt_perm,comkm_perm,trans_prob2_perm,nuntimes1_perm,max_follow1_perm,untimes1_perm,dist_state1_perm,trtkm_perm,trans_prob1_perm,nuntimes0_perm,max_follow0_perm,untimes0_perm,dist_state0_perm,conkm_perm,trans_prob0_perm,nimp)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[4]]
      #      cat('resampled components for iperm=',iperm,'\n')
      #      print(components[,iperm])
    }
    else if (type == "rewtpr") {
      temp <- REWTPR(n,m,nuntimes2_perm,max_follow2_perm,untimes2_perm,Time_perm,Delta_perm,dist_state2_perm,markov_ind,cov_perm,trt_perm,comkm_perm,trans_prob2_perm,time_restriction,nuntimes1_perm,max_follow1_perm,untimes1_perm,dist_state1_perm,trtkm_perm,trans_prob1_perm,nuntimes0_perm,max_follow0_perm,untimes0_perm,dist_state0_perm,conkm_perm,trans_prob0_perm,nimp)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[4]]
      #      cat('resampled components for iperm=',iperm,'\n')
      #      print(components[,iperm])
    }
    else if (type == "rmt") {
      temp <- RMT(m,time_restriction,dist_state0_perm,dist_state1_perm,untimes0_perm,untimes1_perm,nuntimes0_perm,nuntimes1_perm)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[2]]
    }
    else if (type == "wtr") {
      y[iperm] <- WTR(n,m,tau,trt_perm,Time_perm,Delta_perm)[[1]]
    }
    else if (type == "rwtr") {
      y[iperm] <- RWTR(n,m,tau,trt_perm,Time_perm,Delta_perm)[[1]]
    }
    else if (type == "pwt") {
#      cat('call PWT for perm data','\n')
      temp <- PWT(n,n0,n1,m,Time_perm,Delta_perm,trt_perm,tau)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[2]]
#      cat('return PWT for perm data','\n')
    }
    else if (type == "rpwt") {
#      cat('call RPWT for perm data','\n')
      temp <- RPWT(n,n0,n1,m,Time_perm,Delta_perm,trt_perm,tau,time_restriction)
      y[iperm] <- temp[[1]]
      components[,iperm] <- temp[[2]]
#      cat('return RPWT for perm data','\n')
    }
    else if (type == "max") {
      # Permutation Z statistic
      z_ewtr_perm <- EWTR(n,m,nuntimes0_perm,max_follow0_perm,untimes0_perm,Time_perm,Delta_perm,dist_state0_perm,markov_ind,cov_perm,trt_perm)[[3]]
      z_comp_perm <- COMP(n,Time_perm,Delta_perm,cov_perm,trt_perm)[[1]]
      y[iperm] <- max((z_ewtr_perm - z_ewtr),(z_comp_perm - z_comp))
    }
    else {
      stop(paste("Invalid type:", type, "- Please specify one of 'ewt', 'ewtr', 'rmt', 'wtr', 'rwtr', 'pwt', 'ewtp', 'rewtp', 'ewtpr', 'rewtpr', 'rpwt','max'"))
    }
    iperm <- iperm + 1
  }
  return(list(y,components))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{setKM}
\alias{setKM}
\title{Set event times and indicators used in the Kaplan-Meier survival curve calculation}
\usage{
setKM(n, m, time, delta)
}
\arguments{
\item{n}{The total number of trial participants.}

\item{m}{The number of events in the hierarchy.}

\item{time}{The row reversal of the Time matrix (days) (created inside wintime::km()).}

\item{delta}{The row reversal of the Delta matrix (created inside wintime::km()).}
}
\value{
A list containing the event time matrix and the event indicator matrix used in wintime::km().
}
\description{
This function creates the time_km and delta_km matrices used for wintime::km().
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pwt.R
\name{PWT}
\alias{PWT}
\title{Pairwise win time}
\usage{
PWT(n, n0, n1, m, Time, Delta, tg, tau)
}
\arguments{
\item{n}{The total number of trial participants.}

\item{n0}{The number of control arm patients.}

\item{n1}{The number of treatment arm patients.}

\item{m}{The number of events in the hierarchy.}

\item{Time}{A m x n matrix of event time (days). Rows should represent events and columns should represent participants. Event rows should
be in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows
should be in increasing order of clinical severity.}

\item{tg}{A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).}

\item{tau}{The maximum follow up time (days).}
}
\value{
The pairwise win time, and the components of the pairwise win time.
}
\description{
This function calculates the sum of each pair's win time difference divided by the total number of pairs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap}
\alias{bootstrap}
\title{Resample using bootstraps}
\usage{
bootstrap(
  type,
  time_restriction,
  model,
  n,
  m,
  Time,
  Delta,
  trt,
  cov,
  z_ewtr,
  z_comp,
  resample_num,
  seed,
  nimp
)
}
\arguments{
\item{type}{A string value indicating the wintime package method that will run with resampling.}

\item{time_restriction}{The time cutoff value (days).}

\item{model}{A string value indicating the model used on observed data ('markov' or 'km').}

\item{n}{The total number of trial participants.}

\item{m}{The number of events in the hierarchy.}

\item{Time}{A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should
be in increasing order of clinical severity.}

\item{trt}{A numeric vector of treatment arm indicators (1 for treatment, 0 for control).}

\item{cov}{A n x p matrix of covariate values, where p is the number of covariates. Rows should represent participants and columns should
represent covariate values.}

\item{z_ewtr}{The Z-statistic of EWTR.}

\item{z_comp}{The Z-statistic of the composite event approach.}

\item{resample_num}{The number of desired bootstraps.}

\item{seed}{The seed used for random number generation.}

\item{nimp}{The number of random imputations for Redistribution-to-the-right.}
}
\value{
A list of a vector of length resample_num containing the calculated treatment effect estimates (for type='max' these are z-statistics) for each bootstrap,
a m x resample_num matrix of the components of the treatment effect.
}
\description{
This function reruns the desired wintime package method on a given number of bootstrap samples. This resampling method is recommended
for all pairwise wintime methods including Win time ratio (WTR), Restricted win time ratio (RWTR), Pairwise win time (PWT), and
Restricted Pairwise win time (RPWT). This function is also recommended for the EWTR_composite max test (MAX).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wtr.R
\name{WTR}
\alias{WTR}
\title{Win time ratio}
\usage{
WTR(n, m, tau, tg, Time, Delta)
}
\arguments{
\item{n}{The total number of trial participants.}

\item{m}{The number of events in the hierarchy.}

\item{tau}{The maximum follow up time (days).}

\item{tg}{A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).}

\item{Time}{A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should
be in increasing order of clinical severity.}
}
\value{
A list containing: The ratio of losses to wins on treatment, the total number of wins, and the total number of losses.
}
\description{
This function calculates the ratio of losses to wins on treatment. It iterates through all pairs of treatment and control patients and uses
their win time difference as the deciding factor of a win or loss.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{getWintimeIntegral_rest}
\alias{getWintimeIntegral_rest}
\title{Win time difference with time restriction}
\usage{
getWintimeIntegral_rest(
  m,
  etimes,
  time0,
  time1,
  delta0,
  delta1,
  time_restriction
)
}
\arguments{
\item{m}{The number of events in the hierarchy.}

\item{etimes}{A sorted vector of event times (days) (returned from wintime::setEventTimes()).}

\item{time0}{A vector containing the control person's event times (days).}

\item{time1}{A vector containing the treatment person's event times (days).}

\item{delta0}{A vector containing the control person's event indicators.}

\item{delta1}{A vector containing the treatment person's event indicators.}

\item{time_restriction}{The time restriction (days) for calculation.}
}
\value{
A list of the win time difference integral and its components.
}
\description{
This function calculates the win time difference integral for a single pair with truncation at time_restriction.
This function is used in all pairwise win time methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rwtr.R
\name{RWTR}
\alias{RWTR}
\title{Restricted win time ratio}
\usage{
RWTR(n, m, tau, tg, Time, Delta)
}
\arguments{
\item{n}{The total number of trial participants.}

\item{m}{The number of events in the hierarchy.}

\item{tau}{The maximum follow up time (days).}

\item{tg}{A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).}

\item{Time}{A m x n matrix of event times (days), where m is the number of events in the hierarchy, and n is the total number of trial participants.
Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators, where m is the number of events in the hierarchy, and n is the total number of trial participants.
Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.}
}
\value{
A list containing: The ratio of losses to wins on treatment, the total number of wins, and the total number of losses.
}
\description{
This function calculates the ratio of losses to wins on treatment. It iterates through all pairs of treatment and control patients and uses
their time-to-death (or most severe clinical event) to determine a win or loss. If death is inconclusive, then a winner is determined based
on wintime.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markov.R
\name{markov}
\alias{markov}
\title{Fit a Markov model}
\usage{
markov(n0, n1, m, Time, Delta)
}
\arguments{
\item{n0}{The number of participants in the control arm.}

\item{n1}{The number of participants in the active treatment arm.}

\item{m}{The number of events in the hierarchy.}

\item{Time}{A \verb{m x (n0 + n1)} matrix of event times (days). Rows should represent events and columns should represent participants. Event
rows should be in increasing order of clinical severity.}

\item{Delta}{A \verb{m x (n0 + n1)} matrix of event indicators. Rows should represent events and columns should represent participants.
Event rows should be in increasing order of clinical severity.}
}
\value{
A list containing: a matrix of control arm state probabilities, a matrix of treatment arm state probabilities,
a vector of unique control arm event times (days), a vector of unique treatment arm event times (days), the number of unique control
arm event times, the number of unique treatment arm event times, the control arm max follow time (days), the treatment arm
max follow time (days), a matrix of combined arm state probabilities, a vector of unique combined arm event times (days),
the number of unique combined arm event times, the combined arm max follow time (days), a (m x m x number of combined arm event times)
matrix where (i,j,k)'th value is transition probability from state i to state j at k'th combined arm event time,
matrix where (i,j,k)'th value is transition probability from state i to state j at k'th trt arm event time,
matrix where (i,j,k)'th value is transition probability from state i to state j at k'th control arm event time.
}
\description{
This function fits an extended Markov model to calculate the state probabilities for each arm. In the wintime package, the returned state
probability distributions are used in all non-pairwise methods. The extended  Markov model is recommended for the Expected win time against
reference (EWTR) method and the EWTR-composite max test (MAX) method.
}
\examples{
# -----------------------------
# Example inputs
# -----------------------------

# Event time vectors
TIME_1 <- c(256,44,29,186,29,80,11,380,102,33)
TIME_2 <- c(128,44,95,186,69,66,153,380,117,33)
TIME_3 <- c(435,44,95,186,69,270,1063,380,117,33)

# Event time matrix
Time <- rbind(TIME_1, TIME_2, TIME_3)

# Event indicator vectors
DELTA_1 <- c(1,0,1,0,1,1,1,0,1,0)
DELTA_2 <- c(1,0,0,0,0,1,1,0,0,0)
DELTA_3 <- c(0,0,0,0,0,0,0,0,0,0)

# Event indicator matrix
Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)

# Treatment arm indicator vector
trt <- c(1,1,1,1,1,0,0,0,0,0)

# Number of control arm patients
n0 <- sum(trt == 0)

# Number of treatment arm patients
n1 <- sum(trt == 1)

# Number of events in the hierarchy
m <- nrow(Time)

# -------------------------
# markov Examples
# -------------------------

z <- markov(n0, n1, m, Time, Delta)
print(z)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wintime.R
\name{wintime}
\alias{wintime}
\title{Run a win time calculation}
\usage{
wintime(
  type,
  Time,
  Delta,
  trt,
  cov = NULL,
  model = NULL,
  resample = NULL,
  resample_num = 0,
  time_restriction = NA,
  seed = NA,
  nimp = 1
)
}
\arguments{
\item{type}{A string value indicating the desired win time method. Methods include 'ewt', 'ewtr', 'rmt', 'max', 'wtr', 'rwtr', 'pwt', 'ewtp', 'rewtp', 'ewtpr','rewtpr', and 'rpwt'.}

\item{Time}{A \verb{m x n} matrix of event times (days), where \code{m} is the number of events in the hierarchy, and \code{n} is the total number of trial participants.
Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.}

\item{Delta}{A \verb{m x n} matrix of event indicators, where \code{m} is the number of events in the hierarchy, and \code{n} is the total number of trial participants.
Rows should represent events and columns should represent participants. Event rows should be in increasing order of clinical severity.}

\item{trt}{A numeric vector containing treatment arm indicators (1 for treatment, 0 for control).}

\item{cov}{Optional. A \verb{n x p} matrix of covariate values, where \code{n} is the total number of trial participants and \code{p} is the number of covariates.
Rows should represent participants and columns should represent covariate values.}

\item{model}{Optional. String value. The type of model used to calculate state distributions. Options include 'km' and 'markov'.  Default depends on \code{type}.}

\item{resample}{Optional. String value. The resampling method run after the observed data calculation. Options include 'boot' and 'perm'.  Default depends on \code{type}.}

\item{resample_num}{Optional. The number of desired resamples. Default is 0.}

\item{time_restriction}{Required only for \code{type} = 'rmt','rewtp','rewtpr', and 'rpwt'. The cutoff time (days).}

\item{seed}{Optional. Seed used for random number generation in resampling.}

\item{nimp}{Required only for \code{type} = 'ewtpr','rewtpr'. The number of random imputations for Redistribution-to-the-right.}
}
\value{
A list containing: the observed treatment effect, a vector of length \code{resample_num} containing resampled treatment effects, a message
indicating the method ran and the type of resampling done, the variance, the p-value (one-sided for treatment benefit), the total wins on treatment (pairwise methods only),
the total losses on treatment (pairwise methods only), a vector of length 'm' with the components of the treatment effect,
a vector of length 'm' with the variance of the components, the maximum time used in the comparisons (for EWT, RMT, EWTP, REWTP, EWTPR, REWTPR).
A warning message will be printed for combinations of \code{type} and \code{model}/\code{resample}
that are not recommended.
}
\description{
This function runs one of the win time methods on observed and resampled data.
}
\details{
The type parameter specifies the procedure you would like to run.
'ewt' is Expected Win Time.
'ewtr' is Expected Win Time Against Reference (Control Arm).
'rmt' is Restricted Mean Time in Favor of Treatment.
'max' is the MAX procedure (max(COMP,EWTR)).
'wtr' is Win Time Ratio.
'rwtr' is Restricted Win Time Ratio.
'pwt' is Pairwise Win Time.
'ewtp' is Expected Win Time Against Trial Population.
'ewtpr' is Expected Win Time Against Trial Population With Redistribution.
'rewtp' is Time Restricted Expected Win Time Against Trial Population.
'rewtpr'is Time Restricted Expected Win Time Against Trial Population With Redistribution.
'rpwt' is Time Restricted Pairwise Win Time.

The maximum time used in comparisons for EWT, RMT, EWTP, REWTP, EWTPR, and REWTPR are defined below.

First define separately for each arm the maximum valid time for estimating the multi-state distribution
(max_valid_time0 and max_valid_time1) from either a Markov or KM model as follows.
max_valid_time0 is the smallest time for the control arm when the estimated multi-state distribution has a
positive probability of a state but there is nobody still at-risk in that state (If this never occurs
then max_valid_time0 is defined to be the largest event time of the control arm (max_time0)).
max_valid_time1 is the smallest time for the treatment arm when the estimated multi-state distribution has a
positive probability of a state but there is nobody still at-risk in that state (If this never occurs
then max_valid_time1 is defined to be the largest event time of the treatment arm(max_time1)).
Also define the maximum valid time for the combined arms for estimating the multi-state distribution
(max_valid_time2) from either a Markov or KM model as follows.
max_valid_time2 is the smallest time for the combined arms when the estimated multi-state distribution has a
positive probability of a state but there is nobody still at-risk in that state (If this never occurs
then max_valid_time2 is defined to be the largest event time of the combined arms (max_time2)).

EWT: There are two cases:
(1) the arm with the smaller maximum valid time has an estimated multi-state distribution that is degenerate
in the absorbing (highest) state at that time.
(2) the arm with the smaller maximum valid time has an estimated multi-state distribution that is not degenerate
in the absorbing (highest) state at that time.
For case (1) the maximum time used in comparison is the max(max_valid_time0,max_valid_time1).
For case (2) the maximum time used in comparison is the min(max_valid_time0,max_valid_time1).

RMT: the maximum time used in comparison is min(the maximum time used in comparison for EWT, time_restriction).

EWTP: There are two cases:
(1) the combined arms have an estimated multi-state distribution that is degenerate in the absorbing (highest) state
at max_valid_time2.
(2) the combined arms have an estimated multi-state distribution that is not degenerate in the absorbing (highest) state
at max_valid_time2.
For case (1) the maximum time used in comparison is the max(max_valid_time2,max_time2).
For case (2) the maximum time used in comparison is the min(max_valid_time2,max_time2).

REWTP: the maximum time used in comparison is min(the maximum time used in comparison for EWTP, time_restriction).

EWTPR: There are 8 cases based on which of the three (control arm, treatment arm, combined arms) estimated multi-state
distributions are degenerate in the absorbing (highest) state at the maximum valid time.
The maximum time used in comparison is the minimum of the max_valid_times over the arms (control, treatment, combined)
that do not have degenerate estimated multi-state distibutions in the absorbing (highest) state at the maximum valid time
(If all have a degenerate estimated multi-state distibutions in the absorbing (highest) state at the maximum valid time
the maximum time used in comparison is defined as the maximum of the three max_valid_times).

REWTPR: the maximum time used in comparison is min(the maximum time used in comparison for EWTPR, time_restriction).
}
\examples{
# ------------------------------
# Example Inputs
# ------------------------------

# Event time vectors
TIME_1 <- c(256,44,29,186,29,80,11,380,102,33)
TIME_2 <- c(128,44,95,186,69,66,153,380,117,33)
TIME_3 <- c(435,44,95,186,69,270,1063,380,117,33)

# Event time matrix
Time <- rbind(TIME_1, TIME_2, TIME_3)

# Event indicator vectors
DELTA_1 <- c(1,0,1,0,1,1,1,0,1,0)
DELTA_2 <- c(1,0,0,0,0,1,1,0,0,0)
DELTA_3 <- c(0,0,0,0,0,0,0,0,0,0)

# Event indicator matrix
Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)

# Treatment arm indicator vector
trt <- c(1,1,1,1,1,0,0,0,0,0)

# Covariate vectors
cov1 <- c(54,53,55,61,73,65,63,63,82,58,66,66)
cov2 <- c(34.4,32.1,34.7,54.1,55.7,43.6,32.1,44.8,85.2,12.5,33.4,21.4)

# Covariate vectors
cov1 <- c(66,67,54,68,77,65,55,66,77,54)
cov2 <- c(3,6,4,2,3,5,8,5,3,5)
cov3 <- c(34.6,543.6,45.8,54.7,44.3,55.6,65.9,54.7,77.9,31.2)

# Covariate matrix
cov <- cbind(cov1, cov2, cov3)

# ------------------------
# wintime Examples
# ------------------------

# Run WTR
z <- wintime("wtr", Time, Delta, trt)
print(z)

# Run EWT with default settings and 10 resamples
z <- wintime("ewt", Time, Delta, trt, resample_num = 10)
print(z)

# Run EWTR with default settings
z <- wintime("ewtr", Time, Delta, trt, cov = cov)
print(z)

# Run EWTR with KM model (This will produce a warning message)
z <- wintime("ewtr", Time, Delta, trt, cov = cov, model = "km")
print(z)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp.R
\name{COMP}
\alias{COMP}
\title{Run composite analysis}
\usage{
COMP(n, Time, Delta, cov, trt)
}
\arguments{
\item{n}{The total number of trial participants.}

\item{Time}{A m x n matrix of event times (days), where m is the number of events in the hierarchy. Rows should represent events and columns
should represent participants. Event rows should be in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators, where m is the number of events in the hierarchy. Rows should represent events and
columns should represent participants. Event rows should be in increasing order of clinical severity.}

\item{cov}{A n x p matrix of covariate values, where p is the number of covariates. Rows should represent participants and columns
should represent covariate values.}

\item{trt}{A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).}
}
\value{
A list containing: The z-statistic of the treatment effect from the Cox Model fit, the treatment effect estimate, the variance of the treatment effect estimate, the p-value for treatment effect.
}
\description{
This function fits a Cox Model to time-to-event data and calculates the z statistic. In the wintime package, this function is used for the
EWTR-composite max test (MAX) method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rewtp.R
\name{REWTP}
\alias{REWTP}
\title{Expected win time against trial population}
\usage{
REWTP(
  n,
  m,
  nunique,
  maxfollow,
  untimes,
  Time,
  Delta,
  dist,
  markov_ind,
  cov,
  trt,
  time_restriction
)
}
\arguments{
\item{n}{The total number of trial participants.}

\item{m}{The number of events in the hierarchy.}

\item{nunique}{The number of unique combined arm event times (returned from wintime::markov() or wintime::km()).}

\item{maxfollow}{The max combined arm follow up time (days) (returned from wintime::markov() or wintime::km()).}

\item{untimes}{A vector containing unique combined arm event times (days) (returned from wintime::markov() or wintime::km()).}

\item{Time}{A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Rows should be
in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators Rows should represent events and columns should represent participants. Rows should be
in increasing order of clinical severity.}

\item{dist}{A matrix of combined arm state probabilities (returned from wintime::markov() or wintime::km()).}

\item{markov_ind}{An indicator of the model type used (1 for Markov, 0 for Kaplan-Meier).}

\item{cov}{A n x p matrix of covariate values, where p is the number of covariates.}

\item{trt}{A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).}

\item{time_restriction}{The time restriction (days) for calculation.}
}
\value{
A list containing: The estimated treatment effect from the linear regression model, the variance, the Z-statistic,
the components of the treatment effect, the variance of the components, and the maximum time used in comparisons.
}
\description{
Calculates the combined arm state space probabilities using a Markov model or a Kaplan-Meier model (recommended). This function uses these
probabilities to compare each participant's clinical state to a distribution of combined arm states.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{getWintimeIntegral}
\alias{getWintimeIntegral}
\title{Helper functions for package functions}
\usage{
getWintimeIntegral(m, etimes, time0, time1, delta0, delta1)
}
\arguments{
\item{m}{The number of events in the hierarchy.}

\item{etimes}{A sorted vector of event times (days) (returned from wintime::setEventTimes()).}

\item{time0}{A vector containing the control person's event times (days).}

\item{time1}{A vector containing the treatment person's event times (days).}

\item{delta0}{A vector containing the control person's event indicators.}

\item{delta1}{A vector containing the treatment person's event indicators.}
}
\value{
A list of the win time difference integral and its components.
}
\description{
Win time difference
}
\details{
This function calculates the win time difference integral for a single pair. This function is used in all pairwise win time methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{setEventTimes}
\alias{setEventTimes}
\title{Created a sorted vector of event times}
\usage{
setEventTimes(m, delta0, delta1, time0, time1, follow)
}
\arguments{
\item{m}{The number of events in the hierarchy.}

\item{delta0}{A vector of event indicators for the control person.}

\item{delta1}{A vector of event indicators for the treatment person.}

\item{time0}{A vector of event times (days) for the control person.}

\item{time1}{A vector of event times (days) for the treatment person.}

\item{follow}{The maximum follow up time (days).}
}
\value{
A sorted vector of event times (days) for a given pair.
}
\description{
This function creates a sorted vector of event times for a pair. This function is used in all pairwise functions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rewtpr.R
\name{REWTPR}
\alias{REWTPR}
\title{Time Restricted Expected win time against trial population With redistribution to the right}
\usage{
REWTPR(
  n,
  m,
  nunique2,
  maxfollow2,
  untimes2,
  Time,
  Delta,
  dist2,
  markov_ind,
  cov,
  trt,
  comkm,
  trans_prob2,
  time_restriction,
  nunique1,
  maxfollow1,
  untimes1,
  dist1,
  trtkm,
  trans_prob1,
  nunique0,
  maxfollow0,
  untimes0,
  dist0,
  conkm,
  trans_prob0,
  nimp
)
}
\arguments{
\item{n}{The total number of trial participants.}

\item{m}{The number of events in the hierarchy.}

\item{nunique2}{The number of unique combined arm event times (returned from wintime::markov() or wintime::km()).}

\item{maxfollow2}{The max combined arm follow up time (days) (returned from wintime::markov() or wintime::km()).}

\item{untimes2}{A vector containing unique combined arm event times (days) (returned from wintime::markov() or wintime::km()).}

\item{Time}{A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Rows should be
in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators Rows should represent events and columns should represent participants. Rows should be
in increasing order of clinical severity.}

\item{dist2}{A matrix of combined arm state probabilities (returned from wintime::markov() or wintime::km()).}

\item{markov_ind}{An indicator of the model type used (1 for Markov, 0 for Kaplan-Meier).}

\item{cov}{A n x p matrix of covariate values, where p is the number of covariates.}

\item{trt}{A vector of length n containing treatment arm indicators (1 for treatment, 0 for control).}

\item{comkm}{A m x nunique2 matrix of combined arm survival probabilities (returned from wintime::markov() or wintime::km()).}

\item{trans_prob2}{A (m x m x number of combined arm event times)
matrix where (i,j,k)'th value is transition probability from state i to state j at k'th combined arm event time. (returned from wintime::markov() or wintime::km()).}

\item{time_restriction}{The time restriction (days) for calculation.}

\item{nunique1}{The number of unique trt arm event times (returned from wintime::markov() or wintime::km()).}

\item{maxfollow1}{The max trt arm follow up time (days) (returned from wintime::markov() or wintime::km()).}

\item{untimes1}{A vector containing unique trt arm event times (days) (returned from wintime::markov() or wintime::km()).}

\item{dist1}{A matrix of trt arm state probabilities (returned from wintime::markov() or wintime::km()).}

\item{trtkm}{A m x nunique1 matrix of trt arm survival probabilities (returned from wintime::markov() or wintime::km()).}

\item{trans_prob1}{A (m x m x number of trt arm event times)
matrix where (i,j,k)'th value is transition probability from state i to state j at k'th trt arm event time. (returned from wintime::markov() or wintime::km()).}

\item{nunique0}{The number of unique control arm event times (returned from wintime::markov() or wintime::km()).}

\item{maxfollow0}{The max control arm follow up time (days) (returned from wintime::markov() or wintime::km()).}

\item{untimes0}{A vector containing unique control arm event times (days) (returned from wintime::markov() or wintime::km()).}

\item{dist0}{A matrix of control arm state probabilities (returned from wintime::markov() or wintime::km()).}

\item{conkm}{A m x nunique0 matrix of control arm survival probabilities (returned from wintime::markov() or wintime::km()).}

\item{trans_prob0}{A (m x m x number of control arm event times)
matrix where (i,j,k)'th value is transition probability from state i to state j at k'th control arm event time. (returned from wintime::markov() or wintime::km()).}

\item{nimp}{The number of random imputations.}
}
\value{
A list containing: The estimated treatment effect from the linear regression model, the variance, the Z-statistic, the components of the treatment effect, the variance of the components, and the maximum time used in comparisons.
}
\description{
Calculates the combined arm state space probabilities using a Markov model or a Kaplan-Meier model (recommended).
This function uses these probabilities to compare each participant's clinical state to a distribution of combined arm states.
Calculation is extended by redistribution-to-the-right principles and truncated at the user-specified time_restriction (days).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perm.R
\name{perm}
\alias{perm}
\title{Resample using permutations}
\usage{
perm(
  type,
  time_restriction,
  model,
  n,
  m,
  Time,
  Delta,
  trt,
  cov,
  z_ewtr,
  z_comp,
  resample_num,
  seed,
  nimp
)
}
\arguments{
\item{type}{A string value indicating the wintime package method that will run with resampling.}

\item{time_restriction}{The time cutoff value (days).}

\item{model}{A string value indicating the model used on observed data ('markov' or 'km').}

\item{n}{The total number of trial participants.}

\item{m}{The number of events in the hierarchy.}

\item{Time}{A m x n matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
in increasing order of clinical severity.}

\item{Delta}{A m x n matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should
be in increasing order of clinical severity.}

\item{trt}{A numeric vector of treatment arm indicators (1 for treatment, 0 for control).}

\item{cov}{A n x p matrix of covariate values, where p is the number of covariates. Rows should represent participants and columns should
represent covariate values.}

\item{z_ewtr}{The Z-statistic of EWTR.}

\item{z_comp}{The Z-statistic of the composite event approach.}

\item{resample_num}{The number of desired permutations.}

\item{seed}{The seed used for random number generation.}

\item{nimp}{The number of random imputations for Redistribution-to-the-right.}
}
\value{
A list of a vector of length resample_num containing the treatment effect estimates (for type='max' these are z-statistics) for each permutation,
a m x resample_num matrix of the components of the treatment effect..
}
\description{
This function reruns the desired wintime package method on a given number of permutations. This resampling method is recommended
for the Expected win time (EWT) and Restricted mean survival in favor of treatment (RMT) methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmt.R
\name{RMT}
\alias{RMT}
\title{Restricted mean survival in favor of treatment}
\usage{
RMT(
  m,
  time_restriction,
  dist_state0,
  dist_state1,
  unique_event_times0,
  unique_event_times1,
  nunique_event_times0,
  nunique_event_times1
)
}
\arguments{
\item{m}{The number of events in the hierarchy.}

\item{time_restriction}{The cutoff time point (days) for the calculation.}

\item{dist_state0}{A matrix of control arm state probabilities (returned from wintime::km() or wintime::markov()).}

\item{dist_state1}{A matrix of treatment arm state probabilities (returned from wintime::km() or wintime::markov()).}

\item{unique_event_times0}{A vector of unique control arm event times (days) (returned from wintime::km() or wintime::markov()).}

\item{unique_event_times1}{A vector of unique treatment arm event times (days) (returned from wintime::km() or wintime::markov()).}

\item{nunique_event_times0}{The number of unique control arm event times (returned from wintime::km() or wintime::markov()).}

\item{nunique_event_times1}{The number of unique treatment arm event times (returned from wintime::km() or wintime::markov()).}
}
\value{
A list containing: The restricted mean survival in favor of the treatment arm, the components of the treatment effect, and the maximum time used in comparisons.
}
\description{
Calculates the state space probabilities using a Kaplan-Meier model (recommended) or a Markov model. This function uses these probabilities
to compare both arms and calculate the expected win time of the treatment arm up to a given time point.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/km.R
\name{km}
\alias{km}
\title{Fit a Kaplan-Meier model}
\usage{
km(n0, n1, m, Time, Delta)
}
\arguments{
\item{n0}{The number of participants in the control arm.}

\item{n1}{The number of participants in the treatment arm.}

\item{m}{The number of events in the hierarchy.}

\item{Time}{A \verb{m x (n0 + n1)} matrix of event times (days). Rows should represent events and columns should represent participants. Event rows should be
in increasing order of clinical severity.}

\item{Delta}{A \verb{m x (n0 + n1)} matrix of event indicators. Rows should represent events and columns should represent participants. Event rows should be
in increasing order of clinical severity.}
}
\value{
A list containing: a matrix of control arm state probabilities, a matrix of treatment arm state probabilities,
a vector of unique control arm event times (days), a vector of unique treatment arm event times (days), the number of unique control
arm event times, the number of unique treatment arm event times, the control arm max follow time (days), the treatment arm
max follow time (days), a matrix of combined arm state probabilities, a vector of unique combined arm event times (days),
the number of unique combined arm event times, the combined arm max follow time (days), a (m x number unique combined arm event times)
matrix of combined arm km survival probabilities, matrix of trt arm km survival probabilities, matrix of control arm km survival probabilities.
}
\description{
This function fits Kaplan-Meier models to calculate the state probabilities for each arm. In the wintime package, the returned state probability
distributions are used in all non-pairwise methods. The Kaplan-Meier model is recommended for the Expected win time (EWT) method and the Restricted mean survival
in favor of treatment (RMT) method.
}
\examples{
# -----------------------------
# Example inputs
# -----------------------------

# Event time vectors
TIME_1 <- c(256,44,29,186,29,80,11,380,102,33)
TIME_2 <- c(128,44,95,186,69,66,153,380,117,33)
TIME_3 <- c(435,44,95,186,69,270,1063,380,117,33)

# Event time matrix
Time <- rbind(TIME_1, TIME_2, TIME_3)

# Event indicator vectors
DELTA_1 <- c(1,0,1,0,1,1,1,0,1,0)
DELTA_2 <- c(1,0,0,0,0,1,1,0,0,0)
DELTA_3 <- c(0,0,0,0,0,0,0,0,0,0)

# Event indicator matrix
Delta <- rbind(DELTA_1, DELTA_2, DELTA_3)

# Treatment arm indicator vector
trt <- c(1,1,1,1,1,0,0,0,0,0)

# Number of control arm patients
n0 <- sum(trt == 0)

# Number of treatment arm patients
n1 <- sum(trt == 1)

# Number of events in the hierarchy
m <- nrow(Time)

# ------------------------
# km Examples
# ------------------------

z <- km(n0, n1, m, Time, Delta)
print(z)

}
